cmd_/opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv4/.install := perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/include/linux/netfilter_ipv4 /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv4 x86 ip_queue.h ip_tables.h ipt_CLUSTERIP.h ipt_ECN.h ipt_LOG.h ipt_REJECT.h ipt_SAME.h ipt_TTL.h ipt_ULOG.h ipt_addrtype.h ipt_ah.h ipt_ecn.h ipt_realm.h ipt_ttl.h; perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/x86_64-conda_cos6-linux-gnu/build/build-kernel-headers/include/linux/netfilter_ipv4 /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv4 x86 ; touch /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv4/.install
