cmd_/opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm/.install := perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/arch/x86/include/asm /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm x86 a.out.h auxvec.h bitsperlong.h boot.h bootparam.h byteorder.h debugreg.h e820.h errno.h fcntl.h ioctl.h ioctls.h ipcbuf.h ist.h kvm.h kvm_para.h ldt.h mce.h mman.h msgbuf.h msr-index.h msr.h mtrr.h param.h poll.h posix_types.h posix_types_32.h posix_types_64.h prctl.h processor-flags.h ptrace-abi.h ptrace.h resource.h sembuf.h setup.h shmbuf.h sigcontext.h sigcontext32.h siginfo.h signal.h socket.h sockios.h stat.h statfs.h swab.h termbits.h termios.h types.h ucontext.h unistd.h unistd_32.h unistd_64.h vm86.h vsyscall.h; perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/x86_64-conda_cos6-linux-gnu/build/build-kernel-headers/arch/x86/include/asm /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm x86 ; touch /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm/.install
