cmd_/opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm-generic/.install := perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/include/asm-generic /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm-generic x86 auxvec.h bitsperlong.h errno-base.h errno.h fcntl.h int-l64.h int-ll64.h ioctl.h ioctls.h ipcbuf.h mman-common.h mman.h msgbuf.h param.h poll.h posix_types.h resource.h sembuf.h setup.h shmbuf.h shmparam.h siginfo.h signal-defs.h signal.h socket.h sockios.h stat.h statfs.h swab.h termbits.h termios.h types.h ucontext.h unistd.h; perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/x86_64-conda_cos6-linux-gnu/build/build-kernel-headers/include/asm-generic /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm-generic x86 ; touch /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/asm-generic/.install
