cmd_/opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv6/.install := perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/include/linux/netfilter_ipv6 /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv6 x86 ip6_tables.h ip6t_HL.h ip6t_LOG.h ip6t_REJECT.h ip6t_ah.h ip6t_frag.h ip6t_hl.h ip6t_ipv6header.h ip6t_mh.h ip6t_opts.h ip6t_rt.h; perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/x86_64-conda_cos6-linux-gnu/build/build-kernel-headers/include/linux/netfilter_ipv6 /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv6 x86 ; touch /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/linux/netfilter_ipv6/.install
