cmd_/opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/drm/.install := perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/include/drm /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/drm x86 drm.h drm_mode.h drm_sarea.h i810_drm.h i830_drm.h i915_drm.h mga_drm.h r128_drm.h radeon_drm.h savage_drm.h sis_drm.h via_drm.h; perl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/src/linux-2.6.32.71/scripts/headers_install.pl /home/rdonnelly/mc/conda-bld/compilers_linux-64_1534627447954/work/.build/x86_64-conda_cos6-linux-gnu/build/build-kernel-headers/include/drm /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/drm x86 ; touch /opt/anaconda1anaconda2anaconda3/x86_64-conda_cos6-linux-gnu/sysroot/usr/include/drm/.install
