/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_x86_fnstsw_1 = 47,
  CODE_FOR_x86_sahf_1 = 48,
  CODE_FOR_insvhi_1 = 108,
  CODE_FOR_insvsi_1 = 109,
  CODE_FOR_insvdi_1 = 110,
  CODE_FOR_swapxf = 128,
  CODE_FOR_zero_extendqidi2 = 132,
  CODE_FOR_zero_extendhidi2 = 133,
  CODE_FOR_zero_extendqisi2_and = 134,
  CODE_FOR_zero_extendhisi2_and = 135,
  CODE_FOR_zero_extendqihi2_and = 138,
  CODE_FOR_extendsidi2_1 = 146,
  CODE_FOR_extendqidi2 = 147,
  CODE_FOR_extendhidi2 = 148,
  CODE_FOR_extendhisi2 = 149,
  CODE_FOR_extendqisi2 = 151,
  CODE_FOR_extendqihi2 = 153,
  CODE_FOR_truncxfsf2_i387_noop = 164,
  CODE_FOR_truncxfdf2_i387_noop = 165,
  CODE_FOR_fix_truncsfsi_sse = 170,
  CODE_FOR_fix_truncsfdi_sse = 171,
  CODE_FOR_fix_truncdfsi_sse = 172,
  CODE_FOR_fix_truncdfdi_sse = 173,
  CODE_FOR_fix_trunchi_fisttp_i387_1 = 174,
  CODE_FOR_fix_truncsi_fisttp_i387_1 = 175,
  CODE_FOR_fix_truncdi_fisttp_i387_1 = 176,
  CODE_FOR_fix_trunchi_i387_fisttp = 177,
  CODE_FOR_fix_truncsi_i387_fisttp = 178,
  CODE_FOR_fix_truncdi_i387_fisttp = 179,
  CODE_FOR_fix_trunchi_i387_fisttp_with_temp = 180,
  CODE_FOR_fix_truncsi_i387_fisttp_with_temp = 181,
  CODE_FOR_fix_truncdi_i387_fisttp_with_temp = 182,
  CODE_FOR_fix_truncdi_i387 = 186,
  CODE_FOR_fix_truncdi_i387_with_temp = 187,
  CODE_FOR_fix_trunchi_i387 = 188,
  CODE_FOR_fix_truncsi_i387 = 189,
  CODE_FOR_fix_trunchi_i387_with_temp = 190,
  CODE_FOR_fix_truncsi_i387_with_temp = 191,
  CODE_FOR_x86_fnstcw_1 = 192,
  CODE_FOR_x86_fldcw_1 = 193,
  CODE_FOR_floathisf2 = 194,
  CODE_FOR_floathidf2 = 195,
  CODE_FOR_floathixf2 = 196,
  CODE_FOR_floatsixf2 = 197,
  CODE_FOR_floatdixf2 = 198,
  CODE_FOR_floatdisf2_i387_with_xmm = 207,
  CODE_FOR_floatdidf2_i387_with_xmm = 208,
  CODE_FOR_floatdixf2_i387_with_xmm = 209,
  CODE_FOR_addsi_1_zext = 219,
  CODE_FOR_addqi_ext_1 = 241,
  CODE_FOR_addqi3_carry = 295,
  CODE_FOR_addhi3_carry = 296,
  CODE_FOR_addsi3_carry = 297,
  CODE_FOR_adddi3_carry = 298,
  CODE_FOR_addcarrysi = 300,
  CODE_FOR_addcarrydi = 301,
  CODE_FOR_subqi3_carry = 302,
  CODE_FOR_subhi3_carry = 303,
  CODE_FOR_subsi3_carry = 304,
  CODE_FOR_subdi3_carry = 305,
  CODE_FOR_subborrowsi = 307,
  CODE_FOR_subborrowdi = 308,
  CODE_FOR_divmodsi4_1 = 357,
  CODE_FOR_divmoddi4_1 = 358,
  CODE_FOR_divmodhiqi3 = 365,
  CODE_FOR_udivmodsi4_1 = 366,
  CODE_FOR_udivmoddi4_1 = 367,
  CODE_FOR_udivmodhiqi3 = 376,
  CODE_FOR_andqi_ext_1 = 401,
  CODE_FOR_copysignsf3_const = 486,
  CODE_FOR_copysigndf3_const = 487,
  CODE_FOR_copysigntf3_const = 488,
  CODE_FOR_copysignsf3_var = 489,
  CODE_FOR_copysigndf3_var = 490,
  CODE_FOR_copysigntf3_var = 491,
  CODE_FOR_x86_64_shld = 505,
  CODE_FOR_x86_shld = 506,
  CODE_FOR_x86_64_shrd = 535,
  CODE_FOR_x86_shrd = 536,
  CODE_FOR_ashrdi3_cvt = 537,
  CODE_FOR_ashrsi3_cvt = 539,
  CODE_FOR_ix86_rotldi3_doubleword = 580,
  CODE_FOR_ix86_rotlti3_doubleword = 581,
  CODE_FOR_ix86_rotrdi3_doubleword = 582,
  CODE_FOR_ix86_rotrti3_doubleword = 583,
  CODE_FOR_setcc_sf_sse = 615,
  CODE_FOR_setcc_df_sse = 616,
  CODE_FOR_jump = 649,
  CODE_FOR_blockage = 676,
  CODE_FOR_prologue_use = 678,
  CODE_FOR_simple_return_internal = 679,
  CODE_FOR_interrupt_return = 680,
  CODE_FOR_simple_return_internal_long = 681,
  CODE_FOR_simple_return_pop_internal = 682,
  CODE_FOR_simple_return_indirect_internal = 683,
  CODE_FOR_nop = 684,
  CODE_FOR_nops = 685,
  CODE_FOR_pad = 686,
  CODE_FOR_set_got_rex64 = 689,
  CODE_FOR_set_rip_rex64 = 690,
  CODE_FOR_set_got_offset_rex64 = 691,
  CODE_FOR_eh_return_internal = 692,
  CODE_FOR_leave = 693,
  CODE_FOR_leave_rex64 = 694,
  CODE_FOR_split_stack_return = 695,
  CODE_FOR_ffssi2_no_cmove = 696,
  CODE_FOR_ctzsi2 = 703,
  CODE_FOR_ctzdi2 = 704,
  CODE_FOR_bsr_rex64 = 707,
  CODE_FOR_bsr = 708,
  CODE_FOR_clzsi2_lzcnt = 710,
  CODE_FOR_clzdi2_lzcnt = 711,
  CODE_FOR_tzcnt_si = 714,
  CODE_FOR_lzcnt_si = 715,
  CODE_FOR_tzcnt_di = 716,
  CODE_FOR_lzcnt_di = 717,
  CODE_FOR_tzcnt_hi = 722,
  CODE_FOR_lzcnt_hi = 723,
  CODE_FOR_bmi_bextr_si = 724,
  CODE_FOR_bmi_bextr_di = 725,
  CODE_FOR_bmi2_pdep_si3 = 740,
  CODE_FOR_bmi2_pdep_di3 = 741,
  CODE_FOR_bmi2_pext_si3 = 742,
  CODE_FOR_bmi2_pext_di3 = 743,
  CODE_FOR_tbm_bextri_si = 744,
  CODE_FOR_tbm_bextri_di = 745,
  CODE_FOR_popcountsi2 = 764,
  CODE_FOR_popcountdi2 = 765,
  CODE_FOR_popcounthi2 = 769,
  CODE_FOR_bswaphi_lowpart = 775,
  CODE_FOR_paritydi2_cmp = 776,
  CODE_FOR_paritysi2_cmp = 777,
   CODE_FOR_tls_initial_exec_64_sun = CODE_FOR_nothing,
  CODE_FOR_truncxfsf2_i387_noop_unspec = 830,
  CODE_FOR_truncxfdf2_i387_noop_unspec = 831,
  CODE_FOR_sqrtxf2 = 832,
  CODE_FOR_sqrt_extendsfxf2_i387 = 833,
  CODE_FOR_sqrt_extenddfxf2_i387 = 834,
  CODE_FOR_fpremxf4_i387 = 838,
  CODE_FOR_fprem1xf4_i387 = 839,
  CODE_FOR_sincosxf3 = 846,
  CODE_FOR_sincos_extendsfxf3_i387 = 847,
  CODE_FOR_sincos_extenddfxf3_i387 = 848,
  CODE_FOR_fptanxf4_i387 = 849,
  CODE_FOR_fptan_extendsfxf4_i387 = 850,
  CODE_FOR_fptan_extenddfxf4_i387 = 851,
  CODE_FOR_fpatan_extendsfxf3_i387 = 853,
  CODE_FOR_fpatan_extenddfxf3_i387 = 854,
  CODE_FOR_fyl2xxf3_i387 = 855,
  CODE_FOR_fyl2x_extendsfxf3_i387 = 856,
  CODE_FOR_fyl2x_extenddfxf3_i387 = 857,
  CODE_FOR_fyl2xp1xf3_i387 = 858,
  CODE_FOR_fyl2xp1_extendsfxf3_i387 = 859,
  CODE_FOR_fyl2xp1_extenddfxf3_i387 = 860,
  CODE_FOR_fxtractxf3_i387 = 861,
  CODE_FOR_fxtract_extendsfxf3_i387 = 862,
  CODE_FOR_fxtract_extenddfxf3_i387 = 863,
  CODE_FOR_fscalexf4_i387 = 865,
  CODE_FOR_sse4_1_roundsf2 = 866,
  CODE_FOR_sse4_1_rounddf2 = 867,
  CODE_FOR_rintxf2 = 868,
  CODE_FOR_rintsf2_frndint = 869,
  CODE_FOR_rintdf2_frndint = 870,
  CODE_FOR_fistdi2 = 872,
  CODE_FOR_fistdi2_with_temp = 873,
  CODE_FOR_fisthi2 = 876,
  CODE_FOR_fistsi2 = 877,
  CODE_FOR_fisthi2_with_temp = 878,
  CODE_FOR_fistsi2_with_temp = 879,
  CODE_FOR_frndintsf2_floor = 880,
  CODE_FOR_frndintsf2_ceil = 881,
  CODE_FOR_frndintsf2_trunc = 882,
  CODE_FOR_frndintdf2_floor = 883,
  CODE_FOR_frndintdf2_ceil = 884,
  CODE_FOR_frndintdf2_trunc = 885,
  CODE_FOR_frndintxf2_floor = 886,
  CODE_FOR_frndintxf2_ceil = 887,
  CODE_FOR_frndintxf2_trunc = 888,
  CODE_FOR_frndintsf2_floor_i387 = 889,
  CODE_FOR_frndintsf2_ceil_i387 = 890,
  CODE_FOR_frndintsf2_trunc_i387 = 891,
  CODE_FOR_frndintdf2_floor_i387 = 892,
  CODE_FOR_frndintdf2_ceil_i387 = 893,
  CODE_FOR_frndintdf2_trunc_i387 = 894,
  CODE_FOR_frndintxf2_floor_i387 = 895,
  CODE_FOR_frndintxf2_ceil_i387 = 896,
  CODE_FOR_frndintxf2_trunc_i387 = 897,
  CODE_FOR_frndintxf2_mask_pm = 898,
  CODE_FOR_frndintxf2_mask_pm_i387 = 899,
  CODE_FOR_fistdi2_floor = 906,
  CODE_FOR_fistdi2_ceil = 907,
  CODE_FOR_fistdi2_floor_with_temp = 908,
  CODE_FOR_fistdi2_ceil_with_temp = 909,
  CODE_FOR_fisthi2_floor = 910,
  CODE_FOR_fisthi2_ceil = 911,
  CODE_FOR_fistsi2_floor = 912,
  CODE_FOR_fistsi2_ceil = 913,
  CODE_FOR_fisthi2_floor_with_temp = 914,
  CODE_FOR_fisthi2_ceil_with_temp = 915,
  CODE_FOR_fistsi2_floor_with_temp = 916,
  CODE_FOR_fistsi2_ceil_with_temp = 917,
  CODE_FOR_fxamsf2_i387 = 918,
  CODE_FOR_fxamdf2_i387 = 919,
  CODE_FOR_fxamxf2_i387 = 920,
  CODE_FOR_fxamsf2_i387_with_temp = 921,
  CODE_FOR_fxamdf2_i387_with_temp = 922,
  CODE_FOR_movmsk_df = 923,
  CODE_FOR_cld = 924,
  CODE_FOR_smaxsf3 = 975,
  CODE_FOR_sminsf3 = 976,
  CODE_FOR_smaxdf3 = 977,
  CODE_FOR_smindf3 = 978,
  CODE_FOR_pro_epilogue_adjust_stack_si_add = 983,
  CODE_FOR_pro_epilogue_adjust_stack_di_add = 984,
  CODE_FOR_pro_epilogue_adjust_stack_si_sub = 985,
  CODE_FOR_pro_epilogue_adjust_stack_di_sub = 986,
  CODE_FOR_allocate_stack_worker_probe_si = 987,
  CODE_FOR_allocate_stack_worker_probe_di = 988,
  CODE_FOR_probe_stack_si = 989,
  CODE_FOR_probe_stack_di = 990,
  CODE_FOR_adjust_stack_and_probesi = 991,
  CODE_FOR_adjust_stack_and_probedi = 992,
  CODE_FOR_probe_stack_rangesi = 993,
  CODE_FOR_probe_stack_rangedi = 994,
  CODE_FOR_trap = 995,
  CODE_FOR_stack_protect_set_si = 999,
  CODE_FOR_stack_protect_set_di = 1000,
  CODE_FOR_stack_tls_protect_set_si = 1001,
  CODE_FOR_stack_tls_protect_set_di = 1002,
  CODE_FOR_stack_protect_test_si = 1003,
  CODE_FOR_stack_protect_test_di = 1004,
  CODE_FOR_stack_tls_protect_test_si = 1005,
  CODE_FOR_stack_tls_protect_test_di = 1006,
  CODE_FOR_sse4_2_crc32qi = 1007,
  CODE_FOR_sse4_2_crc32hi = 1008,
  CODE_FOR_sse4_2_crc32si = 1009,
  CODE_FOR_sse4_2_crc32di = 1010,
  CODE_FOR_rdpmc = 1011,
  CODE_FOR_rdpmc_rex64 = 1012,
  CODE_FOR_rdtsc = 1013,
  CODE_FOR_rdtsc_rex64 = 1014,
  CODE_FOR_rdtscp = 1015,
  CODE_FOR_rdtscp_rex64 = 1016,
  CODE_FOR_fxsave = 1017,
  CODE_FOR_fxsave64 = 1018,
  CODE_FOR_fxrstor = 1019,
  CODE_FOR_fxrstor64 = 1020,
  CODE_FOR_xsave = 1021,
  CODE_FOR_xsaveopt = 1022,
  CODE_FOR_xsavec = 1023,
  CODE_FOR_xsaves = 1024,
  CODE_FOR_xsave_rex64 = 1025,
  CODE_FOR_xsaveopt_rex64 = 1026,
  CODE_FOR_xsavec_rex64 = 1027,
  CODE_FOR_xsaves_rex64 = 1028,
  CODE_FOR_xsave64 = 1029,
  CODE_FOR_xsaveopt64 = 1030,
  CODE_FOR_xsavec64 = 1031,
  CODE_FOR_xsaves64 = 1032,
  CODE_FOR_xrstor = 1033,
  CODE_FOR_xrstors = 1034,
  CODE_FOR_xrstor_rex64 = 1035,
  CODE_FOR_xrstors_rex64 = 1036,
  CODE_FOR_xrstor64 = 1037,
  CODE_FOR_xrstors64 = 1038,
  CODE_FOR_fnstenv = 1039,
  CODE_FOR_fldenv = 1040,
  CODE_FOR_fnstsw = 1041,
  CODE_FOR_fnclex = 1042,
  CODE_FOR_lwp_slwpcbsi = 1045,
  CODE_FOR_lwp_slwpcbdi = 1046,
  CODE_FOR_rdfsbasesi = 1051,
  CODE_FOR_rdgsbasesi = 1052,
  CODE_FOR_rdfsbasedi = 1053,
  CODE_FOR_rdgsbasedi = 1054,
  CODE_FOR_wrfsbasesi = 1055,
  CODE_FOR_wrgsbasesi = 1056,
  CODE_FOR_wrfsbasedi = 1057,
  CODE_FOR_wrgsbasedi = 1058,
  CODE_FOR_rdrandhi_1 = 1059,
  CODE_FOR_rdrandsi_1 = 1060,
  CODE_FOR_rdranddi_1 = 1061,
  CODE_FOR_rdseedhi_1 = 1062,
  CODE_FOR_rdseedsi_1 = 1063,
  CODE_FOR_rdseeddi_1 = 1064,
  CODE_FOR_xbegin_1 = 1066,
  CODE_FOR_xend = 1067,
  CODE_FOR_xabort = 1068,
  CODE_FOR_xtest_1 = 1069,
  CODE_FOR_clwb = 1070,
  CODE_FOR_clflushopt = 1071,
  CODE_FOR_mwaitx = 1072,
  CODE_FOR_monitorx_si = 1073,
  CODE_FOR_monitorx_di = 1074,
  CODE_FOR_clzero_si = 1075,
  CODE_FOR_clzero_di = 1076,
  CODE_FOR_move_size_reloc_si = 1091,
  CODE_FOR_move_size_reloc_di = 1092,
  CODE_FOR_rdpid = 1095,
  CODE_FOR_sse_movntq = 1101,
  CODE_FOR_mmx_ieee_maxv2sf3 = 1107,
  CODE_FOR_mmx_ieee_minv2sf3 = 1108,
  CODE_FOR_mmx_rcpv2sf2 = 1109,
  CODE_FOR_mmx_rcpit1v2sf3 = 1110,
  CODE_FOR_mmx_rcpit2v2sf3 = 1111,
  CODE_FOR_mmx_rsqrtv2sf2 = 1112,
  CODE_FOR_mmx_rsqit1v2sf3 = 1113,
  CODE_FOR_mmx_haddv2sf3 = 1114,
  CODE_FOR_mmx_hsubv2sf3 = 1115,
  CODE_FOR_mmx_addsubv2sf3 = 1116,
  CODE_FOR_mmx_gtv2sf3 = 1118,
  CODE_FOR_mmx_gev2sf3 = 1119,
  CODE_FOR_mmx_pf2id = 1120,
  CODE_FOR_mmx_pf2iw = 1121,
  CODE_FOR_mmx_pi2fw = 1122,
  CODE_FOR_mmx_floatv2si2 = 1123,
  CODE_FOR_mmx_pswapdv2sf2 = 1124,
  CODE_FOR_mmx_ashrv4hi3 = 1155,
  CODE_FOR_mmx_ashrv2si3 = 1156,
  CODE_FOR_mmx_ashlv4hi3 = 1157,
  CODE_FOR_mmx_lshrv4hi3 = 1158,
  CODE_FOR_mmx_ashlv2si3 = 1159,
  CODE_FOR_mmx_lshrv2si3 = 1160,
  CODE_FOR_mmx_ashlv1di3 = 1161,
  CODE_FOR_mmx_lshrv1di3 = 1162,
  CODE_FOR_mmx_gtv8qi3 = 1166,
  CODE_FOR_mmx_gtv4hi3 = 1167,
  CODE_FOR_mmx_gtv2si3 = 1168,
  CODE_FOR_mmx_andnotv8qi3 = 1169,
  CODE_FOR_mmx_andnotv4hi3 = 1170,
  CODE_FOR_mmx_andnotv2si3 = 1171,
  CODE_FOR_mmx_packsswb = 1181,
  CODE_FOR_mmx_packssdw = 1182,
  CODE_FOR_mmx_packuswb = 1183,
  CODE_FOR_mmx_punpckhbw = 1184,
  CODE_FOR_mmx_punpcklbw = 1185,
  CODE_FOR_mmx_punpckhwd = 1186,
  CODE_FOR_mmx_punpcklwd = 1187,
  CODE_FOR_mmx_punpckhdq = 1188,
  CODE_FOR_mmx_punpckldq = 1189,
  CODE_FOR_mmx_pextrw = 1191,
  CODE_FOR_mmx_pshufw_1 = 1192,
  CODE_FOR_mmx_pswapdv2si2 = 1193,
  CODE_FOR_mmx_psadbw = 1202,
  CODE_FOR_mmx_pmovmskb = 1203,
  CODE_FOR_movv64qi_internal = 1208,
  CODE_FOR_movv32qi_internal = 1209,
  CODE_FOR_movv16qi_internal = 1210,
  CODE_FOR_movv32hi_internal = 1211,
  CODE_FOR_movv16hi_internal = 1212,
  CODE_FOR_movv8hi_internal = 1213,
  CODE_FOR_movv16si_internal = 1214,
  CODE_FOR_movv8si_internal = 1215,
  CODE_FOR_movv4si_internal = 1216,
  CODE_FOR_movv8di_internal = 1217,
  CODE_FOR_movv4di_internal = 1218,
  CODE_FOR_movv2di_internal = 1219,
  CODE_FOR_movv4ti_internal = 1220,
  CODE_FOR_movv2ti_internal = 1221,
  CODE_FOR_movv1ti_internal = 1222,
  CODE_FOR_movv16sf_internal = 1223,
  CODE_FOR_movv8sf_internal = 1224,
  CODE_FOR_movv4sf_internal = 1225,
  CODE_FOR_movv8df_internal = 1226,
  CODE_FOR_movv4df_internal = 1227,
  CODE_FOR_movv2df_internal = 1228,
  CODE_FOR_avx512f_loadv16si_mask = 1229,
  CODE_FOR_avx512vl_loadv8si_mask = 1230,
  CODE_FOR_avx512vl_loadv4si_mask = 1231,
  CODE_FOR_avx512f_loadv8di_mask = 1232,
  CODE_FOR_avx512vl_loadv4di_mask = 1233,
  CODE_FOR_avx512vl_loadv2di_mask = 1234,
  CODE_FOR_avx512f_loadv16sf_mask = 1235,
  CODE_FOR_avx512vl_loadv8sf_mask = 1236,
  CODE_FOR_avx512vl_loadv4sf_mask = 1237,
  CODE_FOR_avx512f_loadv8df_mask = 1238,
  CODE_FOR_avx512vl_loadv4df_mask = 1239,
  CODE_FOR_avx512vl_loadv2df_mask = 1240,
  CODE_FOR_avx512bw_loadv64qi_mask = 1241,
  CODE_FOR_avx512vl_loadv16qi_mask = 1242,
  CODE_FOR_avx512vl_loadv32qi_mask = 1243,
  CODE_FOR_avx512bw_loadv32hi_mask = 1244,
  CODE_FOR_avx512vl_loadv16hi_mask = 1245,
  CODE_FOR_avx512vl_loadv8hi_mask = 1246,
  CODE_FOR_avx512f_blendmv16si = 1247,
  CODE_FOR_avx512vl_blendmv8si = 1248,
  CODE_FOR_avx512vl_blendmv4si = 1249,
  CODE_FOR_avx512f_blendmv8di = 1250,
  CODE_FOR_avx512vl_blendmv4di = 1251,
  CODE_FOR_avx512vl_blendmv2di = 1252,
  CODE_FOR_avx512f_blendmv16sf = 1253,
  CODE_FOR_avx512vl_blendmv8sf = 1254,
  CODE_FOR_avx512vl_blendmv4sf = 1255,
  CODE_FOR_avx512f_blendmv8df = 1256,
  CODE_FOR_avx512vl_blendmv4df = 1257,
  CODE_FOR_avx512vl_blendmv2df = 1258,
  CODE_FOR_avx512bw_blendmv64qi = 1259,
  CODE_FOR_avx512vl_blendmv16qi = 1260,
  CODE_FOR_avx512vl_blendmv32qi = 1261,
  CODE_FOR_avx512bw_blendmv32hi = 1262,
  CODE_FOR_avx512vl_blendmv16hi = 1263,
  CODE_FOR_avx512vl_blendmv8hi = 1264,
  CODE_FOR_avx512f_storev16si_mask = 1265,
  CODE_FOR_avx512vl_storev8si_mask = 1266,
  CODE_FOR_avx512vl_storev4si_mask = 1267,
  CODE_FOR_avx512f_storev8di_mask = 1268,
  CODE_FOR_avx512vl_storev4di_mask = 1269,
  CODE_FOR_avx512vl_storev2di_mask = 1270,
  CODE_FOR_avx512f_storev16sf_mask = 1271,
  CODE_FOR_avx512vl_storev8sf_mask = 1272,
  CODE_FOR_avx512vl_storev4sf_mask = 1273,
  CODE_FOR_avx512f_storev8df_mask = 1274,
  CODE_FOR_avx512vl_storev4df_mask = 1275,
  CODE_FOR_avx512vl_storev2df_mask = 1276,
  CODE_FOR_avx512bw_storev64qi_mask = 1277,
  CODE_FOR_avx512vl_storev16qi_mask = 1278,
  CODE_FOR_avx512vl_storev32qi_mask = 1279,
  CODE_FOR_avx512bw_storev32hi_mask = 1280,
  CODE_FOR_avx512vl_storev16hi_mask = 1281,
  CODE_FOR_avx512vl_storev8hi_mask = 1282,
  CODE_FOR_sse2_movq128 = 1283,
  CODE_FOR_movdi_to_sse = 1284,
  CODE_FOR_avx_lddqu256 = 1285,
  CODE_FOR_sse3_lddqu = 1286,
  CODE_FOR_sse2_movntisi = 1287,
  CODE_FOR_sse2_movntidi = 1288,
  CODE_FOR_avx512f_movntv16sf = 1289,
  CODE_FOR_avx_movntv8sf = 1290,
  CODE_FOR_sse_movntv4sf = 1291,
  CODE_FOR_avx512f_movntv8df = 1292,
  CODE_FOR_avx_movntv4df = 1293,
  CODE_FOR_sse2_movntv2df = 1294,
  CODE_FOR_avx512f_movntv8di = 1295,
  CODE_FOR_avx_movntv4di = 1296,
  CODE_FOR_sse2_movntv2di = 1297,
  CODE_FOR_kandqi = 1298,
  CODE_FOR_kiorqi = 1299,
  CODE_FOR_kxorqi = 1300,
  CODE_FOR_kandhi = 1301,
  CODE_FOR_kiorhi = 1302,
  CODE_FOR_kxorhi = 1303,
  CODE_FOR_kandsi = 1304,
  CODE_FOR_kiorsi = 1305,
  CODE_FOR_kxorsi = 1306,
  CODE_FOR_kanddi = 1307,
  CODE_FOR_kiordi = 1308,
  CODE_FOR_kxordi = 1309,
  CODE_FOR_kandnqi = 1310,
  CODE_FOR_kandnhi = 1311,
  CODE_FOR_kandnsi = 1312,
  CODE_FOR_kandndi = 1313,
  CODE_FOR_kxnorqi = 1314,
  CODE_FOR_kxnorhi = 1315,
  CODE_FOR_kxnorsi = 1316,
  CODE_FOR_kxnordi = 1317,
  CODE_FOR_knotqi = 1318,
  CODE_FOR_knothi = 1319,
  CODE_FOR_knotsi = 1320,
  CODE_FOR_knotdi = 1321,
  CODE_FOR_kaddqi = 1322,
  CODE_FOR_kaddhi = 1323,
  CODE_FOR_kaddsi = 1324,
  CODE_FOR_kadddi = 1325,
  CODE_FOR_kashiftqi = 1326,
  CODE_FOR_klshiftrtqi = 1327,
  CODE_FOR_kashifthi = 1328,
  CODE_FOR_klshiftrthi = 1329,
  CODE_FOR_kashiftsi = 1330,
  CODE_FOR_klshiftrtsi = 1331,
  CODE_FOR_kashiftdi = 1332,
  CODE_FOR_klshiftrtdi = 1333,
  CODE_FOR_ktestqi = 1334,
  CODE_FOR_ktesthi = 1335,
  CODE_FOR_ktestsi = 1336,
  CODE_FOR_ktestdi = 1337,
  CODE_FOR_kortestqi = 1338,
  CODE_FOR_kortesthi = 1339,
  CODE_FOR_kortestsi = 1340,
  CODE_FOR_kortestdi = 1341,
  CODE_FOR_kunpckhi = 1342,
  CODE_FOR_kunpcksi = 1343,
  CODE_FOR_kunpckdi = 1344,
  CODE_FOR_sse_vmaddv4sf3 = 1399,
  CODE_FOR_sse_vmaddv4sf3_round = 1400,
  CODE_FOR_sse_vmsubv4sf3 = 1401,
  CODE_FOR_sse_vmsubv4sf3_round = 1402,
  CODE_FOR_sse2_vmaddv2df3 = 1403,
  CODE_FOR_sse2_vmaddv2df3_round = 1404,
  CODE_FOR_sse2_vmsubv2df3 = 1405,
  CODE_FOR_sse2_vmsubv2df3_round = 1406,
  CODE_FOR_sse_vmmulv4sf3 = 1431,
  CODE_FOR_sse_vmmulv4sf3_round = 1432,
  CODE_FOR_sse_vmdivv4sf3 = 1433,
  CODE_FOR_sse_vmdivv4sf3_round = 1434,
  CODE_FOR_sse2_vmmulv2df3 = 1435,
  CODE_FOR_sse2_vmmulv2df3_round = 1436,
  CODE_FOR_sse2_vmdivv2df3 = 1437,
  CODE_FOR_sse2_vmdivv2df3_round = 1438,
  CODE_FOR_avx512f_divv16sf3 = 1439,
  CODE_FOR_avx512f_divv16sf3_round = 1440,
  CODE_FOR_avx512f_divv16sf3_mask = 1441,
  CODE_FOR_avx512f_divv16sf3_mask_round = 1442,
  CODE_FOR_avx_divv8sf3 = 1443,
   CODE_FOR_avx_divv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv8sf3_mask = 1444,
   CODE_FOR_avx_divv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3 = 1445,
   CODE_FOR_sse_divv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sse_divv4sf3_mask = 1446,
   CODE_FOR_sse_divv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_divv8df3 = 1447,
  CODE_FOR_avx512f_divv8df3_round = 1448,
  CODE_FOR_avx512f_divv8df3_mask = 1449,
  CODE_FOR_avx512f_divv8df3_mask_round = 1450,
  CODE_FOR_avx_divv4df3 = 1451,
   CODE_FOR_avx_divv4df3_round = CODE_FOR_nothing,
  CODE_FOR_avx_divv4df3_mask = 1452,
   CODE_FOR_avx_divv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3 = 1453,
   CODE_FOR_sse2_divv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sse2_divv2df3_mask = 1454,
   CODE_FOR_sse2_divv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx_rcpv8sf2 = 1455,
  CODE_FOR_sse_rcpv4sf2 = 1456,
  CODE_FOR_sse_vmrcpv4sf2 = 1457,
  CODE_FOR_rcp14v16sf_mask = 1459,
  CODE_FOR_rcp14v8sf_mask = 1461,
  CODE_FOR_rcp14v4sf_mask = 1463,
  CODE_FOR_rcp14v8df_mask = 1465,
  CODE_FOR_rcp14v4df_mask = 1467,
  CODE_FOR_rcp14v2df_mask = 1469,
  CODE_FOR_srcp14v4sf = 1470,
  CODE_FOR_srcp14v2df = 1471,
  CODE_FOR_avx512f_sqrtv16sf2 = 1472,
  CODE_FOR_avx512f_sqrtv16sf2_round = 1473,
  CODE_FOR_avx512f_sqrtv16sf2_mask = 1474,
  CODE_FOR_avx512f_sqrtv16sf2_mask_round = 1475,
  CODE_FOR_avx_sqrtv8sf2 = 1476,
   CODE_FOR_avx_sqrtv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv8sf2_mask = 1477,
   CODE_FOR_avx_sqrtv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2 = 1478,
   CODE_FOR_sse_sqrtv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_sse_sqrtv4sf2_mask = 1479,
   CODE_FOR_sse_sqrtv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_sqrtv8df2 = 1480,
  CODE_FOR_avx512f_sqrtv8df2_round = 1481,
  CODE_FOR_avx512f_sqrtv8df2_mask = 1482,
  CODE_FOR_avx512f_sqrtv8df2_mask_round = 1483,
  CODE_FOR_avx_sqrtv4df2 = 1484,
   CODE_FOR_avx_sqrtv4df2_round = CODE_FOR_nothing,
  CODE_FOR_avx_sqrtv4df2_mask = 1485,
   CODE_FOR_avx_sqrtv4df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2 = 1486,
   CODE_FOR_sse2_sqrtv2df2_round = CODE_FOR_nothing,
  CODE_FOR_sse2_sqrtv2df2_mask = 1487,
   CODE_FOR_sse2_sqrtv2df2_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsqrtv4sf2 = 1488,
  CODE_FOR_sse_vmsqrtv4sf2_round = 1489,
  CODE_FOR_sse2_vmsqrtv2df2 = 1490,
  CODE_FOR_sse2_vmsqrtv2df2_round = 1491,
  CODE_FOR_avx_rsqrtv8sf2 = 1492,
  CODE_FOR_sse_rsqrtv4sf2 = 1493,
  CODE_FOR_rsqrt14v16sf_mask = 1495,
  CODE_FOR_rsqrt14v8sf_mask = 1497,
  CODE_FOR_rsqrt14v4sf_mask = 1499,
  CODE_FOR_rsqrt14v8df_mask = 1501,
  CODE_FOR_rsqrt14v4df_mask = 1503,
  CODE_FOR_rsqrt14v2df_mask = 1505,
  CODE_FOR_rsqrt14v4sf = 1506,
  CODE_FOR_rsqrt14v2df = 1507,
  CODE_FOR_sse_vmrsqrtv4sf2 = 1508,
  CODE_FOR_ieee_maxv16sf3 = 1557,
  CODE_FOR_ieee_maxv16sf3_round = 1558,
  CODE_FOR_ieee_maxv16sf3_mask = 1559,
  CODE_FOR_ieee_maxv16sf3_mask_round = 1560,
  CODE_FOR_ieee_minv16sf3 = 1561,
  CODE_FOR_ieee_minv16sf3_round = 1562,
  CODE_FOR_ieee_minv16sf3_mask = 1563,
  CODE_FOR_ieee_minv16sf3_mask_round = 1564,
  CODE_FOR_ieee_maxv8sf3 = 1565,
   CODE_FOR_ieee_maxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8sf3_mask = 1566,
   CODE_FOR_ieee_maxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3 = 1567,
   CODE_FOR_ieee_minv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv8sf3_mask = 1568,
   CODE_FOR_ieee_minv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3 = 1569,
   CODE_FOR_ieee_maxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4sf3_mask = 1570,
   CODE_FOR_ieee_maxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3 = 1571,
   CODE_FOR_ieee_minv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4sf3_mask = 1572,
   CODE_FOR_ieee_minv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv8df3 = 1573,
  CODE_FOR_ieee_maxv8df3_round = 1574,
  CODE_FOR_ieee_maxv8df3_mask = 1575,
  CODE_FOR_ieee_maxv8df3_mask_round = 1576,
  CODE_FOR_ieee_minv8df3 = 1577,
  CODE_FOR_ieee_minv8df3_round = 1578,
  CODE_FOR_ieee_minv8df3_mask = 1579,
  CODE_FOR_ieee_minv8df3_mask_round = 1580,
  CODE_FOR_ieee_maxv4df3 = 1581,
   CODE_FOR_ieee_maxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv4df3_mask = 1582,
   CODE_FOR_ieee_maxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3 = 1583,
   CODE_FOR_ieee_minv4df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv4df3_mask = 1584,
   CODE_FOR_ieee_minv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3 = 1585,
   CODE_FOR_ieee_maxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_maxv2df3_mask = 1586,
   CODE_FOR_ieee_maxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3 = 1587,
   CODE_FOR_ieee_minv2df3_round = CODE_FOR_nothing,
  CODE_FOR_ieee_minv2df3_mask = 1588,
   CODE_FOR_ieee_minv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse_vmsmaxv4sf3 = 1589,
  CODE_FOR_sse_vmsmaxv4sf3_round = 1590,
  CODE_FOR_sse_vmsminv4sf3 = 1591,
  CODE_FOR_sse_vmsminv4sf3_round = 1592,
  CODE_FOR_sse2_vmsmaxv2df3 = 1593,
  CODE_FOR_sse2_vmsmaxv2df3_round = 1594,
  CODE_FOR_sse2_vmsminv2df3 = 1595,
  CODE_FOR_sse2_vmsminv2df3_round = 1596,
  CODE_FOR_avx_addsubv4df3 = 1597,
  CODE_FOR_sse3_addsubv2df3 = 1598,
  CODE_FOR_avx_addsubv8sf3 = 1599,
  CODE_FOR_sse3_addsubv4sf3 = 1600,
  CODE_FOR_avx_haddv4df3 = 1601,
  CODE_FOR_avx_hsubv4df3 = 1602,
  CODE_FOR_sse3_hsubv2df3 = 1604,
  CODE_FOR_avx_haddv8sf3 = 1607,
  CODE_FOR_avx_hsubv8sf3 = 1608,
  CODE_FOR_sse3_haddv4sf3 = 1609,
  CODE_FOR_sse3_hsubv4sf3 = 1610,
  CODE_FOR_reducepv16sf_mask = 1612,
  CODE_FOR_reducepv8sf_mask = 1614,
  CODE_FOR_reducepv4sf_mask = 1616,
  CODE_FOR_reducepv8df_mask = 1618,
  CODE_FOR_reducepv4df_mask = 1620,
  CODE_FOR_reducepv2df_mask = 1622,
  CODE_FOR_reducesv4sf = 1623,
  CODE_FOR_reducesv2df = 1624,
  CODE_FOR_avx_cmpv8sf3 = 1625,
  CODE_FOR_avx_cmpv4sf3 = 1626,
  CODE_FOR_avx_cmpv4df3 = 1627,
  CODE_FOR_avx_cmpv2df3 = 1628,
  CODE_FOR_avx_vmcmpv4sf3 = 1629,
  CODE_FOR_avx_vmcmpv2df3 = 1630,
  CODE_FOR_avx_maskcmpv8sf3 = 1635,
  CODE_FOR_sse_maskcmpv4sf3 = 1636,
  CODE_FOR_avx_maskcmpv4df3 = 1637,
  CODE_FOR_sse2_maskcmpv2df3 = 1638,
  CODE_FOR_sse_vmmaskcmpv4sf3 = 1639,
  CODE_FOR_sse2_vmmaskcmpv2df3 = 1640,
  CODE_FOR_avx512f_cmpv16si3 = 1641,
  CODE_FOR_avx512f_cmpv16si3_mask = 1642,
  CODE_FOR_avx512f_cmpv16si3_round = 1643,
  CODE_FOR_avx512f_cmpv16si3_mask_round = 1644,
  CODE_FOR_avx512vl_cmpv8si3 = 1645,
  CODE_FOR_avx512vl_cmpv8si3_mask = 1646,
   CODE_FOR_avx512vl_cmpv8si3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv8si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4si3 = 1647,
  CODE_FOR_avx512vl_cmpv4si3_mask = 1648,
   CODE_FOR_avx512vl_cmpv4si3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv4si3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8di3 = 1649,
  CODE_FOR_avx512f_cmpv8di3_mask = 1650,
  CODE_FOR_avx512f_cmpv8di3_round = 1651,
  CODE_FOR_avx512f_cmpv8di3_mask_round = 1652,
  CODE_FOR_avx512vl_cmpv4di3 = 1653,
  CODE_FOR_avx512vl_cmpv4di3_mask = 1654,
   CODE_FOR_avx512vl_cmpv4di3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv4di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2di3 = 1655,
  CODE_FOR_avx512vl_cmpv2di3_mask = 1656,
   CODE_FOR_avx512vl_cmpv2di3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv2di3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv16sf3 = 1657,
  CODE_FOR_avx512f_cmpv16sf3_mask = 1658,
  CODE_FOR_avx512f_cmpv16sf3_round = 1659,
  CODE_FOR_avx512f_cmpv16sf3_mask_round = 1660,
  CODE_FOR_avx512vl_cmpv8sf3 = 1661,
  CODE_FOR_avx512vl_cmpv8sf3_mask = 1662,
   CODE_FOR_avx512vl_cmpv8sf3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv4sf3 = 1663,
  CODE_FOR_avx512vl_cmpv4sf3_mask = 1664,
   CODE_FOR_avx512vl_cmpv4sf3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_cmpv8df3 = 1665,
  CODE_FOR_avx512f_cmpv8df3_mask = 1666,
  CODE_FOR_avx512f_cmpv8df3_round = 1667,
  CODE_FOR_avx512f_cmpv8df3_mask_round = 1668,
  CODE_FOR_avx512vl_cmpv4df3 = 1669,
  CODE_FOR_avx512vl_cmpv4df3_mask = 1670,
   CODE_FOR_avx512vl_cmpv4df3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_cmpv2df3 = 1671,
  CODE_FOR_avx512vl_cmpv2df3_mask = 1672,
   CODE_FOR_avx512vl_cmpv2df3_round = CODE_FOR_nothing,
   CODE_FOR_avx512vl_cmpv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512bw_cmpv64qi3 = 1673,
  CODE_FOR_avx512bw_cmpv64qi3_mask = 1674,
  CODE_FOR_avx512vl_cmpv16qi3 = 1675,
  CODE_FOR_avx512vl_cmpv16qi3_mask = 1676,
  CODE_FOR_avx512vl_cmpv32qi3 = 1677,
  CODE_FOR_avx512vl_cmpv32qi3_mask = 1678,
  CODE_FOR_avx512bw_cmpv32hi3 = 1679,
  CODE_FOR_avx512bw_cmpv32hi3_mask = 1680,
  CODE_FOR_avx512vl_cmpv16hi3 = 1681,
  CODE_FOR_avx512vl_cmpv16hi3_mask = 1682,
  CODE_FOR_avx512vl_cmpv8hi3 = 1683,
  CODE_FOR_avx512vl_cmpv8hi3_mask = 1684,
  CODE_FOR_avx512bw_ucmpv64qi3 = 1685,
  CODE_FOR_avx512bw_ucmpv64qi3_mask = 1686,
  CODE_FOR_avx512vl_ucmpv16qi3 = 1687,
  CODE_FOR_avx512vl_ucmpv16qi3_mask = 1688,
  CODE_FOR_avx512vl_ucmpv32qi3 = 1689,
  CODE_FOR_avx512vl_ucmpv32qi3_mask = 1690,
  CODE_FOR_avx512bw_ucmpv32hi3 = 1691,
  CODE_FOR_avx512bw_ucmpv32hi3_mask = 1692,
  CODE_FOR_avx512vl_ucmpv16hi3 = 1693,
  CODE_FOR_avx512vl_ucmpv16hi3_mask = 1694,
  CODE_FOR_avx512vl_ucmpv8hi3 = 1695,
  CODE_FOR_avx512vl_ucmpv8hi3_mask = 1696,
  CODE_FOR_avx512f_ucmpv16si3 = 1697,
  CODE_FOR_avx512f_ucmpv16si3_mask = 1698,
  CODE_FOR_avx512vl_ucmpv8si3 = 1699,
  CODE_FOR_avx512vl_ucmpv8si3_mask = 1700,
  CODE_FOR_avx512vl_ucmpv4si3 = 1701,
  CODE_FOR_avx512vl_ucmpv4si3_mask = 1702,
  CODE_FOR_avx512f_ucmpv8di3 = 1703,
  CODE_FOR_avx512f_ucmpv8di3_mask = 1704,
  CODE_FOR_avx512vl_ucmpv4di3 = 1705,
  CODE_FOR_avx512vl_ucmpv4di3_mask = 1706,
  CODE_FOR_avx512vl_ucmpv2di3 = 1707,
  CODE_FOR_avx512vl_ucmpv2di3_mask = 1708,
  CODE_FOR_avx512f_vmcmpv4sf3 = 1709,
  CODE_FOR_avx512f_vmcmpv4sf3_round = 1710,
  CODE_FOR_avx512f_vmcmpv2df3 = 1711,
  CODE_FOR_avx512f_vmcmpv2df3_round = 1712,
  CODE_FOR_avx512f_vmcmpv4sf3_mask = 1713,
  CODE_FOR_avx512f_vmcmpv4sf3_mask_round = 1714,
  CODE_FOR_avx512f_vmcmpv2df3_mask = 1715,
  CODE_FOR_avx512f_vmcmpv2df3_mask_round = 1716,
  CODE_FOR_avx512f_maskcmpv16sf3 = 1717,
  CODE_FOR_avx512f_maskcmpv8sf3 = 1718,
  CODE_FOR_avx512f_maskcmpv4sf3 = 1719,
  CODE_FOR_avx512f_maskcmpv8df3 = 1720,
  CODE_FOR_avx512f_maskcmpv4df3 = 1721,
  CODE_FOR_avx512f_maskcmpv2df3 = 1722,
  CODE_FOR_sse_comi = 1723,
  CODE_FOR_sse_comi_round = 1724,
  CODE_FOR_sse2_comi = 1725,
  CODE_FOR_sse2_comi_round = 1726,
  CODE_FOR_sse_ucomi = 1727,
  CODE_FOR_sse_ucomi_round = 1728,
  CODE_FOR_sse2_ucomi = 1729,
  CODE_FOR_sse2_ucomi_round = 1730,
  CODE_FOR_avx_andnotv8sf3 = 1731,
  CODE_FOR_avx_andnotv8sf3_mask = 1732,
  CODE_FOR_sse_andnotv4sf3 = 1733,
  CODE_FOR_sse_andnotv4sf3_mask = 1734,
  CODE_FOR_avx_andnotv4df3 = 1735,
  CODE_FOR_avx_andnotv4df3_mask = 1736,
  CODE_FOR_sse2_andnotv2df3 = 1737,
  CODE_FOR_sse2_andnotv2df3_mask = 1738,
  CODE_FOR_avx512f_andnotv16sf3 = 1739,
  CODE_FOR_avx512f_andnotv16sf3_mask = 1740,
  CODE_FOR_avx512f_andnotv8df3 = 1741,
  CODE_FOR_avx512f_andnotv8df3_mask = 1742,
  CODE_FOR_fma_fmadd_v16sf_maskz_1 = 1800,
  CODE_FOR_fma_fmadd_v16sf_maskz_1_round = 1801,
  CODE_FOR_fma_fmadd_v8sf_maskz_1 = 1803,
   CODE_FOR_fma_fmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v4sf_maskz_1 = 1805,
   CODE_FOR_fma_fmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v8df_maskz_1 = 1809,
  CODE_FOR_fma_fmadd_v8df_maskz_1_round = 1810,
  CODE_FOR_fma_fmadd_v4df_maskz_1 = 1812,
   CODE_FOR_fma_fmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmadd_v2df_maskz_1 = 1814,
   CODE_FOR_fma_fmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask = 1815,
  CODE_FOR_avx512f_fmadd_v16sf_mask_round = 1816,
  CODE_FOR_avx512vl_fmadd_v8sf_mask = 1817,
   CODE_FOR_avx512vl_fmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v4sf_mask = 1818,
   CODE_FOR_avx512vl_fmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v8df_mask = 1819,
  CODE_FOR_avx512f_fmadd_v8df_mask_round = 1820,
  CODE_FOR_avx512vl_fmadd_v4df_mask = 1821,
   CODE_FOR_avx512vl_fmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmadd_v2df_mask = 1822,
   CODE_FOR_avx512vl_fmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmadd_v16sf_mask3 = 1823,
  CODE_FOR_avx512f_fmadd_v16sf_mask3_round = 1824,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3 = 1825,
  CODE_FOR_avx512vl_fmadd_v8sf_mask3_round = 1826,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3 = 1827,
  CODE_FOR_avx512vl_fmadd_v4sf_mask3_round = 1828,
  CODE_FOR_avx512f_fmadd_v8df_mask3 = 1829,
  CODE_FOR_avx512f_fmadd_v8df_mask3_round = 1830,
  CODE_FOR_avx512vl_fmadd_v4df_mask3 = 1831,
  CODE_FOR_avx512vl_fmadd_v4df_mask3_round = 1832,
  CODE_FOR_avx512vl_fmadd_v2df_mask3 = 1833,
  CODE_FOR_avx512vl_fmadd_v2df_mask3_round = 1834,
  CODE_FOR_fma_fmsub_v16sf_maskz_1 = 1844,
  CODE_FOR_fma_fmsub_v16sf_maskz_1_round = 1845,
  CODE_FOR_fma_fmsub_v8sf_maskz_1 = 1847,
   CODE_FOR_fma_fmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v4sf_maskz_1 = 1849,
   CODE_FOR_fma_fmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v8df_maskz_1 = 1853,
  CODE_FOR_fma_fmsub_v8df_maskz_1_round = 1854,
  CODE_FOR_fma_fmsub_v4df_maskz_1 = 1856,
   CODE_FOR_fma_fmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsub_v2df_maskz_1 = 1858,
   CODE_FOR_fma_fmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v16sf_mask = 1859,
  CODE_FOR_avx512f_fmsub_v16sf_mask_round = 1860,
  CODE_FOR_avx512vl_fmsub_v8sf_mask = 1861,
  CODE_FOR_avx512vl_fmsub_v8sf_mask_round = 1862,
  CODE_FOR_avx512vl_fmsub_v4sf_mask = 1863,
  CODE_FOR_avx512vl_fmsub_v4sf_mask_round = 1864,
  CODE_FOR_avx512f_fmsub_v8df_mask = 1865,
  CODE_FOR_avx512f_fmsub_v8df_mask_round = 1866,
  CODE_FOR_avx512vl_fmsub_v4df_mask = 1867,
  CODE_FOR_avx512vl_fmsub_v4df_mask_round = 1868,
  CODE_FOR_avx512vl_fmsub_v2df_mask = 1869,
  CODE_FOR_avx512vl_fmsub_v2df_mask_round = 1870,
  CODE_FOR_avx512f_fmsub_v16sf_mask3 = 1871,
  CODE_FOR_avx512f_fmsub_v16sf_mask3_round = 1872,
  CODE_FOR_avx512vl_fmsub_v8sf_mask3 = 1873,
   CODE_FOR_avx512vl_fmsub_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v4sf_mask3 = 1874,
   CODE_FOR_avx512vl_fmsub_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsub_v8df_mask3 = 1875,
  CODE_FOR_avx512f_fmsub_v8df_mask3_round = 1876,
  CODE_FOR_avx512vl_fmsub_v4df_mask3 = 1877,
   CODE_FOR_avx512vl_fmsub_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fmsub_v2df_mask3 = 1878,
   CODE_FOR_avx512vl_fmsub_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1 = 1888,
  CODE_FOR_fma_fnmadd_v16sf_maskz_1_round = 1889,
  CODE_FOR_fma_fnmadd_v8sf_maskz_1 = 1891,
   CODE_FOR_fma_fnmadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v4sf_maskz_1 = 1893,
   CODE_FOR_fma_fnmadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v8df_maskz_1 = 1897,
  CODE_FOR_fma_fnmadd_v8df_maskz_1_round = 1898,
  CODE_FOR_fma_fnmadd_v4df_maskz_1 = 1900,
   CODE_FOR_fma_fnmadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmadd_v2df_maskz_1 = 1902,
   CODE_FOR_fma_fnmadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask = 1903,
  CODE_FOR_avx512f_fnmadd_v16sf_mask_round = 1904,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask = 1905,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask = 1906,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask = 1907,
  CODE_FOR_avx512f_fnmadd_v8df_mask_round = 1908,
  CODE_FOR_avx512vl_fnmadd_v4df_mask = 1909,
   CODE_FOR_avx512vl_fnmadd_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask = 1910,
   CODE_FOR_avx512vl_fnmadd_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3 = 1911,
  CODE_FOR_avx512f_fnmadd_v16sf_mask3_round = 1912,
  CODE_FOR_avx512vl_fnmadd_v8sf_mask3 = 1913,
   CODE_FOR_avx512vl_fnmadd_v8sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v4sf_mask3 = 1914,
   CODE_FOR_avx512vl_fnmadd_v4sf_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmadd_v8df_mask3 = 1915,
  CODE_FOR_avx512f_fnmadd_v8df_mask3_round = 1916,
  CODE_FOR_avx512vl_fnmadd_v4df_mask3 = 1917,
   CODE_FOR_avx512vl_fnmadd_v4df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmadd_v2df_mask3 = 1918,
   CODE_FOR_avx512vl_fnmadd_v2df_mask3_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1 = 1942,
  CODE_FOR_fma_fnmsub_v16sf_maskz_1_round = 1943,
  CODE_FOR_fma_fnmsub_v8sf_maskz_1 = 1945,
   CODE_FOR_fma_fnmsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v4sf_maskz_1 = 1947,
   CODE_FOR_fma_fnmsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v8df_maskz_1 = 1951,
  CODE_FOR_fma_fnmsub_v8df_maskz_1_round = 1952,
  CODE_FOR_fma_fnmsub_v4df_maskz_1 = 1954,
   CODE_FOR_fma_fnmsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fnmsub_v2df_maskz_1 = 1956,
   CODE_FOR_fma_fnmsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask = 1957,
  CODE_FOR_avx512f_fnmsub_v16sf_mask_round = 1958,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask = 1959,
   CODE_FOR_avx512vl_fnmsub_v8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask = 1960,
   CODE_FOR_avx512vl_fnmsub_v4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v8df_mask = 1961,
  CODE_FOR_avx512f_fnmsub_v8df_mask_round = 1962,
  CODE_FOR_avx512vl_fnmsub_v4df_mask = 1963,
   CODE_FOR_avx512vl_fnmsub_v4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512vl_fnmsub_v2df_mask = 1964,
   CODE_FOR_avx512vl_fnmsub_v2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3 = 1965,
  CODE_FOR_avx512f_fnmsub_v16sf_mask3_round = 1966,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3 = 1967,
  CODE_FOR_avx512vl_fnmsub_v8sf_mask3_round = 1968,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3 = 1969,
  CODE_FOR_avx512vl_fnmsub_v4sf_mask3_round = 1970,
  CODE_FOR_avx512f_fnmsub_v8df_mask3 = 1971,
  CODE_FOR_avx512f_fnmsub_v8df_mask3_round = 1972,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3 = 1973,
  CODE_FOR_avx512vl_fnmsub_v4df_mask3_round = 1974,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3 = 1975,
  CODE_FOR_avx512vl_fnmsub_v2df_mask3_round = 1976,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1 = 1984,
  CODE_FOR_fma_fmaddsub_v16sf_maskz_1_round = 1985,
  CODE_FOR_fma_fmaddsub_v8sf_maskz_1 = 1987,
   CODE_FOR_fma_fmaddsub_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v4sf_maskz_1 = 1989,
   CODE_FOR_fma_fmaddsub_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1 = 1993,
  CODE_FOR_fma_fmaddsub_v8df_maskz_1_round = 1994,
  CODE_FOR_fma_fmaddsub_v4df_maskz_1 = 1996,
   CODE_FOR_fma_fmaddsub_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmaddsub_v2df_maskz_1 = 1998,
   CODE_FOR_fma_fmaddsub_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask = 1999,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask_round = 2000,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask = 2001,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask_round = 2002,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask = 2003,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask_round = 2004,
  CODE_FOR_avx512f_fmaddsub_v8df_mask = 2005,
  CODE_FOR_avx512f_fmaddsub_v8df_mask_round = 2006,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask = 2007,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask_round = 2008,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask = 2009,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask_round = 2010,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3 = 2011,
  CODE_FOR_avx512f_fmaddsub_v16sf_mask3_round = 2012,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3 = 2013,
  CODE_FOR_avx512vl_fmaddsub_v8sf_mask3_round = 2014,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3 = 2015,
  CODE_FOR_avx512vl_fmaddsub_v4sf_mask3_round = 2016,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3 = 2017,
  CODE_FOR_avx512f_fmaddsub_v8df_mask3_round = 2018,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3 = 2019,
  CODE_FOR_avx512vl_fmaddsub_v4df_mask3_round = 2020,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3 = 2021,
  CODE_FOR_avx512vl_fmaddsub_v2df_mask3_round = 2022,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1 = 2030,
  CODE_FOR_fma_fmsubadd_v16sf_maskz_1_round = 2031,
  CODE_FOR_fma_fmsubadd_v8sf_maskz_1 = 2033,
   CODE_FOR_fma_fmsubadd_v8sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v4sf_maskz_1 = 2035,
   CODE_FOR_fma_fmsubadd_v4sf_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1 = 2039,
  CODE_FOR_fma_fmsubadd_v8df_maskz_1_round = 2040,
  CODE_FOR_fma_fmsubadd_v4df_maskz_1 = 2042,
   CODE_FOR_fma_fmsubadd_v4df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_fma_fmsubadd_v2df_maskz_1 = 2044,
   CODE_FOR_fma_fmsubadd_v2df_maskz_1_round = CODE_FOR_nothing,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask = 2045,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask_round = 2046,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask = 2047,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask_round = 2048,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask = 2049,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask_round = 2050,
  CODE_FOR_avx512f_fmsubadd_v8df_mask = 2051,
  CODE_FOR_avx512f_fmsubadd_v8df_mask_round = 2052,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask = 2053,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask_round = 2054,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask = 2055,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask_round = 2056,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3 = 2057,
  CODE_FOR_avx512f_fmsubadd_v16sf_mask3_round = 2058,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3 = 2059,
  CODE_FOR_avx512vl_fmsubadd_v8sf_mask3_round = 2060,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3 = 2061,
  CODE_FOR_avx512vl_fmsubadd_v4sf_mask3_round = 2062,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3 = 2063,
  CODE_FOR_avx512f_fmsubadd_v8df_mask3_round = 2064,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3 = 2065,
  CODE_FOR_avx512vl_fmsubadd_v4df_mask3_round = 2066,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3 = 2067,
  CODE_FOR_avx512vl_fmsubadd_v2df_mask3_round = 2068,
  CODE_FOR_sse_cvtpi2ps = 2093,
  CODE_FOR_sse_cvtps2pi = 2094,
  CODE_FOR_sse_cvttps2pi = 2095,
  CODE_FOR_sse_cvtsi2ss = 2096,
  CODE_FOR_sse_cvtsi2ss_round = 2097,
  CODE_FOR_sse_cvtsi2ssq = 2098,
  CODE_FOR_sse_cvtsi2ssq_round = 2099,
  CODE_FOR_sse_cvtss2si = 2100,
  CODE_FOR_sse_cvtss2si_round = 2101,
  CODE_FOR_sse_cvtss2si_2 = 2102,
  CODE_FOR_sse_cvtss2siq = 2103,
  CODE_FOR_sse_cvtss2siq_round = 2104,
  CODE_FOR_sse_cvtss2siq_2 = 2105,
  CODE_FOR_sse_cvttss2si = 2106,
  CODE_FOR_sse_cvttss2si_round = 2107,
  CODE_FOR_sse_cvttss2siq = 2108,
  CODE_FOR_sse_cvttss2siq_round = 2109,
  CODE_FOR_cvtusi2ss32 = 2110,
  CODE_FOR_cvtusi2ss32_round = 2111,
  CODE_FOR_cvtusi2sd32 = 2112,
   CODE_FOR_cvtusi2sd32_round = CODE_FOR_nothing,
  CODE_FOR_cvtusi2ss64 = 2113,
  CODE_FOR_cvtusi2ss64_round = 2114,
  CODE_FOR_cvtusi2sd64 = 2115,
  CODE_FOR_cvtusi2sd64_round = 2116,
  CODE_FOR_floatv16siv16sf2 = 2117,
  CODE_FOR_floatv16siv16sf2_round = 2118,
  CODE_FOR_floatv16siv16sf2_mask = 2119,
  CODE_FOR_floatv16siv16sf2_mask_round = 2120,
  CODE_FOR_floatv8siv8sf2 = 2121,
   CODE_FOR_floatv8siv8sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv8siv8sf2_mask = 2122,
   CODE_FOR_floatv8siv8sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2 = 2123,
   CODE_FOR_floatv4siv4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4siv4sf2_mask = 2124,
   CODE_FOR_floatv4siv4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv16siv16sf2 = 2125,
  CODE_FOR_ufloatv16siv16sf2_round = 2126,
  CODE_FOR_ufloatv16siv16sf2_mask = 2127,
  CODE_FOR_ufloatv16siv16sf2_mask_round = 2128,
  CODE_FOR_ufloatv8siv8sf2 = 2129,
  CODE_FOR_ufloatv8siv8sf2_round = 2130,
  CODE_FOR_ufloatv8siv8sf2_mask = 2131,
  CODE_FOR_ufloatv8siv8sf2_mask_round = 2132,
  CODE_FOR_ufloatv4siv4sf2 = 2133,
  CODE_FOR_ufloatv4siv4sf2_round = 2134,
  CODE_FOR_ufloatv4siv4sf2_mask = 2135,
  CODE_FOR_ufloatv4siv4sf2_mask_round = 2136,
  CODE_FOR_avx_fix_notruncv8sfv8si = 2137,
  CODE_FOR_avx_fix_notruncv8sfv8si_mask = 2138,
  CODE_FOR_sse2_fix_notruncv4sfv4si = 2139,
  CODE_FOR_sse2_fix_notruncv4sfv4si_mask = 2140,
  CODE_FOR_avx512f_fix_notruncv16sfv16si = 2141,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_round = 2142,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask = 2143,
  CODE_FOR_avx512f_fix_notruncv16sfv16si_mask_round = 2144,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask = 2147,
  CODE_FOR_avx512f_ufix_notruncv16sfv16si_mask_round = 2148,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask = 2151,
  CODE_FOR_avx512vl_ufix_notruncv8sfv8si_mask_round = 2152,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask = 2155,
  CODE_FOR_avx512vl_ufix_notruncv4sfv4si_mask_round = 2156,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask = 2159,
  CODE_FOR_avx512dq_cvtps2qqv8di_mask_round = 2160,
  CODE_FOR_avx512dq_cvtps2qqv4di_mask = 2162,
   CODE_FOR_avx512dq_cvtps2qqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2qqv2di_mask = 2164,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask = 2167,
  CODE_FOR_avx512dq_cvtps2uqqv8di_mask_round = 2168,
  CODE_FOR_avx512dq_cvtps2uqqv4di_mask = 2170,
   CODE_FOR_avx512dq_cvtps2uqqv4di_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_cvtps2uqqv2di_mask = 2172,
  CODE_FOR_fix_truncv16sfv16si2 = 2173,
  CODE_FOR_fix_truncv16sfv16si2_round = 2174,
  CODE_FOR_fix_truncv16sfv16si2_mask = 2175,
  CODE_FOR_fix_truncv16sfv16si2_mask_round = 2176,
  CODE_FOR_ufix_truncv16sfv16si2 = 2177,
  CODE_FOR_ufix_truncv16sfv16si2_round = 2178,
  CODE_FOR_ufix_truncv16sfv16si2_mask = 2179,
  CODE_FOR_ufix_truncv16sfv16si2_mask_round = 2180,
  CODE_FOR_fix_truncv8sfv8si2 = 2181,
  CODE_FOR_fix_truncv8sfv8si2_mask = 2182,
  CODE_FOR_fix_truncv4sfv4si2 = 2183,
  CODE_FOR_fix_truncv4sfv4si2_mask = 2184,
  CODE_FOR_sse2_cvtpi2pd = 2185,
  CODE_FOR_sse2_cvtpd2pi = 2186,
  CODE_FOR_sse2_cvttpd2pi = 2187,
  CODE_FOR_sse2_cvtsi2sd = 2188,
  CODE_FOR_sse2_cvtsi2sdq = 2189,
  CODE_FOR_sse2_cvtsi2sdq_round = 2190,
  CODE_FOR_avx512f_vcvtss2usi = 2191,
  CODE_FOR_avx512f_vcvtss2usi_round = 2192,
  CODE_FOR_avx512f_vcvtss2usiq = 2193,
  CODE_FOR_avx512f_vcvtss2usiq_round = 2194,
  CODE_FOR_avx512f_vcvttss2usi = 2195,
  CODE_FOR_avx512f_vcvttss2usi_round = 2196,
  CODE_FOR_avx512f_vcvttss2usiq = 2197,
  CODE_FOR_avx512f_vcvttss2usiq_round = 2198,
  CODE_FOR_avx512f_vcvtsd2usi = 2199,
  CODE_FOR_avx512f_vcvtsd2usi_round = 2200,
  CODE_FOR_avx512f_vcvtsd2usiq = 2201,
  CODE_FOR_avx512f_vcvtsd2usiq_round = 2202,
  CODE_FOR_avx512f_vcvttsd2usi = 2203,
  CODE_FOR_avx512f_vcvttsd2usi_round = 2204,
  CODE_FOR_avx512f_vcvttsd2usiq = 2205,
  CODE_FOR_avx512f_vcvttsd2usiq_round = 2206,
  CODE_FOR_sse2_cvtsd2si = 2207,
  CODE_FOR_sse2_cvtsd2si_round = 2208,
  CODE_FOR_sse2_cvtsd2si_2 = 2209,
  CODE_FOR_sse2_cvtsd2siq = 2210,
  CODE_FOR_sse2_cvtsd2siq_round = 2211,
  CODE_FOR_sse2_cvtsd2siq_2 = 2212,
  CODE_FOR_sse2_cvttsd2si = 2213,
  CODE_FOR_sse2_cvttsd2si_round = 2214,
  CODE_FOR_sse2_cvttsd2siq = 2215,
  CODE_FOR_sse2_cvttsd2siq_round = 2216,
  CODE_FOR_floatv8siv8df2 = 2217,
  CODE_FOR_floatv8siv8df2_mask = 2218,
  CODE_FOR_floatv4siv4df2 = 2219,
  CODE_FOR_floatv4siv4df2_mask = 2220,
  CODE_FOR_floatv8div8df2 = 2221,
  CODE_FOR_floatv8div8df2_round = 2222,
  CODE_FOR_floatv8div8df2_mask = 2223,
  CODE_FOR_floatv8div8df2_mask_round = 2224,
  CODE_FOR_ufloatv8div8df2 = 2225,
  CODE_FOR_ufloatv8div8df2_round = 2226,
  CODE_FOR_ufloatv8div8df2_mask = 2227,
  CODE_FOR_ufloatv8div8df2_mask_round = 2228,
  CODE_FOR_floatv4div4df2 = 2229,
  CODE_FOR_floatv4div4df2_round = 2230,
  CODE_FOR_floatv4div4df2_mask = 2231,
  CODE_FOR_floatv4div4df2_mask_round = 2232,
  CODE_FOR_ufloatv4div4df2 = 2233,
  CODE_FOR_ufloatv4div4df2_round = 2234,
  CODE_FOR_ufloatv4div4df2_mask = 2235,
  CODE_FOR_ufloatv4div4df2_mask_round = 2236,
  CODE_FOR_floatv2div2df2 = 2237,
  CODE_FOR_floatv2div2df2_round = 2238,
  CODE_FOR_floatv2div2df2_mask = 2239,
  CODE_FOR_floatv2div2df2_mask_round = 2240,
  CODE_FOR_ufloatv2div2df2 = 2241,
  CODE_FOR_ufloatv2div2df2_round = 2242,
  CODE_FOR_ufloatv2div2df2_mask = 2243,
  CODE_FOR_ufloatv2div2df2_mask_round = 2244,
  CODE_FOR_floatv8div8sf2 = 2245,
  CODE_FOR_floatv8div8sf2_round = 2246,
  CODE_FOR_floatv8div8sf2_mask = 2247,
  CODE_FOR_floatv8div8sf2_mask_round = 2248,
  CODE_FOR_ufloatv8div8sf2 = 2249,
  CODE_FOR_ufloatv8div8sf2_round = 2250,
  CODE_FOR_ufloatv8div8sf2_mask = 2251,
  CODE_FOR_ufloatv8div8sf2_mask_round = 2252,
  CODE_FOR_floatv4div4sf2 = 2253,
   CODE_FOR_floatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_floatv4div4sf2_mask = 2254,
   CODE_FOR_floatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv4div4sf2 = 2255,
   CODE_FOR_ufloatv4div4sf2_round = CODE_FOR_nothing,
  CODE_FOR_ufloatv4div4sf2_mask = 2256,
   CODE_FOR_ufloatv4div4sf2_mask_round = CODE_FOR_nothing,
  CODE_FOR_floatv2div2sf2_mask = 2259,
  CODE_FOR_ufloatv2div2sf2_mask = 2260,
  CODE_FOR_ufloatv8siv8df2 = 2263,
  CODE_FOR_ufloatv8siv8df2_mask = 2264,
  CODE_FOR_ufloatv4siv4df2 = 2265,
  CODE_FOR_ufloatv4siv4df2_mask = 2266,
  CODE_FOR_ufloatv2siv2df2 = 2267,
  CODE_FOR_ufloatv2siv2df2_mask = 2268,
  CODE_FOR_avx512f_cvtdq2pd512_2 = 2269,
  CODE_FOR_avx_cvtdq2pd256_2 = 2270,
  CODE_FOR_sse2_cvtdq2pd = 2271,
  CODE_FOR_sse2_cvtdq2pd_mask = 2272,
  CODE_FOR_avx512f_cvtpd2dq512 = 2273,
  CODE_FOR_avx512f_cvtpd2dq512_round = 2274,
  CODE_FOR_avx512f_cvtpd2dq512_mask = 2275,
  CODE_FOR_avx512f_cvtpd2dq512_mask_round = 2276,
  CODE_FOR_avx_cvtpd2dq256 = 2277,
  CODE_FOR_avx_cvtpd2dq256_mask = 2278,
  CODE_FOR_sse2_cvtpd2dq = 2280,
  CODE_FOR_sse2_cvtpd2dq_mask = 2281,
  CODE_FOR_ufix_notruncv8dfv8si2 = 2282,
  CODE_FOR_ufix_notruncv8dfv8si2_round = 2283,
  CODE_FOR_ufix_notruncv8dfv8si2_mask = 2284,
  CODE_FOR_ufix_notruncv8dfv8si2_mask_round = 2285,
  CODE_FOR_ufix_notruncv4dfv4si2 = 2286,
  CODE_FOR_ufix_notruncv4dfv4si2_round = 2287,
  CODE_FOR_ufix_notruncv4dfv4si2_mask = 2288,
  CODE_FOR_ufix_notruncv4dfv4si2_mask_round = 2289,
  CODE_FOR_ufix_notruncv2dfv2si2 = 2290,
  CODE_FOR_ufix_notruncv2dfv2si2_mask = 2291,
  CODE_FOR_fix_truncv8dfv8si2 = 2292,
  CODE_FOR_fix_truncv8dfv8si2_round = 2293,
  CODE_FOR_fix_truncv8dfv8si2_mask = 2294,
  CODE_FOR_fix_truncv8dfv8si2_mask_round = 2295,
  CODE_FOR_ufix_truncv8dfv8si2 = 2296,
  CODE_FOR_ufix_truncv8dfv8si2_round = 2297,
  CODE_FOR_ufix_truncv8dfv8si2_mask = 2298,
  CODE_FOR_ufix_truncv8dfv8si2_mask_round = 2299,
  CODE_FOR_ufix_truncv2dfv2si2 = 2300,
  CODE_FOR_ufix_truncv2dfv2si2_mask = 2301,
  CODE_FOR_fix_truncv4dfv4si2 = 2302,
  CODE_FOR_fix_truncv4dfv4si2_mask = 2303,
  CODE_FOR_ufix_truncv4dfv4si2 = 2304,
  CODE_FOR_ufix_truncv4dfv4si2_mask = 2305,
  CODE_FOR_fix_truncv8dfv8di2 = 2306,
  CODE_FOR_fix_truncv8dfv8di2_round = 2307,
  CODE_FOR_fix_truncv8dfv8di2_mask = 2308,
  CODE_FOR_fix_truncv8dfv8di2_mask_round = 2309,
  CODE_FOR_ufix_truncv8dfv8di2 = 2310,
  CODE_FOR_ufix_truncv8dfv8di2_round = 2311,
  CODE_FOR_ufix_truncv8dfv8di2_mask = 2312,
  CODE_FOR_ufix_truncv8dfv8di2_mask_round = 2313,
  CODE_FOR_fix_truncv4dfv4di2 = 2314,
   CODE_FOR_fix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4dfv4di2_mask = 2315,
   CODE_FOR_fix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4dfv4di2 = 2316,
   CODE_FOR_ufix_truncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4dfv4di2_mask = 2317,
   CODE_FOR_ufix_truncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2 = 2318,
   CODE_FOR_fix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2dfv2di2_mask = 2319,
   CODE_FOR_fix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv2dfv2di2 = 2320,
   CODE_FOR_ufix_truncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv2dfv2di2_mask = 2321,
   CODE_FOR_ufix_truncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv8dfv8di2 = 2322,
  CODE_FOR_fix_notruncv8dfv8di2_round = 2323,
  CODE_FOR_fix_notruncv8dfv8di2_mask = 2324,
  CODE_FOR_fix_notruncv8dfv8di2_mask_round = 2325,
  CODE_FOR_fix_notruncv4dfv4di2 = 2326,
   CODE_FOR_fix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv4dfv4di2_mask = 2327,
   CODE_FOR_fix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2 = 2328,
   CODE_FOR_fix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_notruncv2dfv2di2_mask = 2329,
   CODE_FOR_fix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv8dfv8di2 = 2330,
  CODE_FOR_ufix_notruncv8dfv8di2_round = 2331,
  CODE_FOR_ufix_notruncv8dfv8di2_mask = 2332,
  CODE_FOR_ufix_notruncv8dfv8di2_mask_round = 2333,
  CODE_FOR_ufix_notruncv4dfv4di2 = 2334,
   CODE_FOR_ufix_notruncv4dfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv4dfv4di2_mask = 2335,
   CODE_FOR_ufix_notruncv4dfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2 = 2336,
   CODE_FOR_ufix_notruncv2dfv2di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_notruncv2dfv2di2_mask = 2337,
   CODE_FOR_ufix_notruncv2dfv2di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv8sfv8di2 = 2338,
  CODE_FOR_fix_truncv8sfv8di2_round = 2339,
  CODE_FOR_fix_truncv8sfv8di2_mask = 2340,
  CODE_FOR_fix_truncv8sfv8di2_mask_round = 2341,
  CODE_FOR_ufix_truncv8sfv8di2 = 2342,
  CODE_FOR_ufix_truncv8sfv8di2_round = 2343,
  CODE_FOR_ufix_truncv8sfv8di2_mask = 2344,
  CODE_FOR_ufix_truncv8sfv8di2_mask_round = 2345,
  CODE_FOR_fix_truncv4sfv4di2 = 2346,
   CODE_FOR_fix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv4sfv4di2_mask = 2347,
   CODE_FOR_fix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4sfv4di2 = 2348,
   CODE_FOR_ufix_truncv4sfv4di2_round = CODE_FOR_nothing,
  CODE_FOR_ufix_truncv4sfv4di2_mask = 2349,
   CODE_FOR_ufix_truncv4sfv4di2_mask_round = CODE_FOR_nothing,
  CODE_FOR_fix_truncv2sfv2di2 = 2350,
  CODE_FOR_fix_truncv2sfv2di2_mask = 2351,
  CODE_FOR_ufix_truncv2sfv2di2 = 2352,
  CODE_FOR_ufix_truncv2sfv2di2_mask = 2353,
  CODE_FOR_ufix_truncv8sfv8si2 = 2354,
  CODE_FOR_ufix_truncv8sfv8si2_mask = 2355,
  CODE_FOR_ufix_truncv4sfv4si2 = 2356,
  CODE_FOR_ufix_truncv4sfv4si2_mask = 2357,
  CODE_FOR_sse2_cvttpd2dq = 2358,
  CODE_FOR_sse2_cvttpd2dq_mask = 2359,
  CODE_FOR_sse2_cvtsd2ss = 2360,
  CODE_FOR_sse2_cvtsd2ss_round = 2361,
  CODE_FOR_sse2_cvtss2sd = 2363,
  CODE_FOR_sse2_cvtss2sd_round = 2364,
  CODE_FOR_avx512f_cvtpd2ps512_mask = 2368,
  CODE_FOR_avx512f_cvtpd2ps512_mask_round = 2369,
  CODE_FOR_avx_cvtpd2ps256 = 2370,
  CODE_FOR_avx_cvtpd2ps256_mask = 2371,
  CODE_FOR_avx512f_cvtps2pd512 = 2374,
  CODE_FOR_avx512f_cvtps2pd512_round = 2375,
  CODE_FOR_avx512f_cvtps2pd512_mask = 2376,
  CODE_FOR_avx512f_cvtps2pd512_mask_round = 2377,
  CODE_FOR_avx_cvtps2pd256 = 2378,
   CODE_FOR_avx_cvtps2pd256_round = CODE_FOR_nothing,
  CODE_FOR_avx_cvtps2pd256_mask = 2379,
   CODE_FOR_avx_cvtps2pd256_mask_round = CODE_FOR_nothing,
  CODE_FOR_vec_unpacks_lo_v16sf = 2381,
  CODE_FOR_avx512bw_cvtb2maskv64qi = 2382,
  CODE_FOR_avx512vl_cvtb2maskv16qi = 2383,
  CODE_FOR_avx512vl_cvtb2maskv32qi = 2384,
  CODE_FOR_avx512bw_cvtw2maskv32hi = 2385,
  CODE_FOR_avx512vl_cvtw2maskv16hi = 2386,
  CODE_FOR_avx512vl_cvtw2maskv8hi = 2387,
  CODE_FOR_avx512f_cvtd2maskv16si = 2388,
  CODE_FOR_avx512vl_cvtd2maskv8si = 2389,
  CODE_FOR_avx512vl_cvtd2maskv4si = 2390,
  CODE_FOR_avx512f_cvtq2maskv8di = 2391,
  CODE_FOR_avx512vl_cvtq2maskv4di = 2392,
  CODE_FOR_avx512vl_cvtq2maskv2di = 2393,
  CODE_FOR_sse2_cvtps2pd = 2406,
  CODE_FOR_sse2_cvtps2pd_mask = 2407,
  CODE_FOR_sse_movhlps = 2408,
  CODE_FOR_sse_movlhps = 2409,
  CODE_FOR_avx512f_unpckhps512_mask = 2411,
  CODE_FOR_avx_unpckhps256 = 2412,
  CODE_FOR_avx_unpckhps256_mask = 2413,
  CODE_FOR_vec_interleave_highv4sf = 2414,
  CODE_FOR_vec_interleave_highv4sf_mask = 2415,
  CODE_FOR_avx512f_unpcklps512_mask = 2417,
  CODE_FOR_avx_unpcklps256 = 2418,
  CODE_FOR_avx_unpcklps256_mask = 2419,
  CODE_FOR_unpcklps128_mask = 2420,
  CODE_FOR_vec_interleave_lowv4sf = 2421,
  CODE_FOR_avx_movshdup256 = 2422,
  CODE_FOR_avx_movshdup256_mask = 2423,
  CODE_FOR_sse3_movshdup = 2424,
  CODE_FOR_sse3_movshdup_mask = 2425,
  CODE_FOR_avx512f_movshdup512_mask = 2427,
  CODE_FOR_avx_movsldup256 = 2428,
  CODE_FOR_avx_movsldup256_mask = 2429,
  CODE_FOR_sse3_movsldup = 2430,
  CODE_FOR_sse3_movsldup_mask = 2431,
  CODE_FOR_avx512f_movsldup512_mask = 2433,
  CODE_FOR_avx_shufps256_1 = 2434,
  CODE_FOR_avx_shufps256_1_mask = 2435,
  CODE_FOR_sse_shufps_v4sf_mask = 2436,
  CODE_FOR_sse_shufps_v4si = 2437,
  CODE_FOR_sse_shufps_v4sf = 2438,
  CODE_FOR_sse_storehps = 2439,
  CODE_FOR_sse_loadhps = 2440,
  CODE_FOR_sse_storelps = 2441,
  CODE_FOR_sse_loadlps = 2442,
  CODE_FOR_sse_movss = 2443,
  CODE_FOR_avx2_vec_dupv8sf = 2444,
  CODE_FOR_avx2_vec_dupv4sf = 2445,
  CODE_FOR_avx2_vec_dupv8sf_1 = 2446,
  CODE_FOR_avx512f_vec_dupv16sf_1 = 2447,
  CODE_FOR_avx512f_vec_dupv8df_1 = 2448,
  CODE_FOR_vec_setv4si_0 = 2452,
  CODE_FOR_vec_setv4sf_0 = 2453,
  CODE_FOR_sse4_1_insertps = 2455,
  CODE_FOR_avx512dq_vextractf64x2_1_maskm = 2459,
  CODE_FOR_avx512dq_vextracti64x2_1_maskm = 2460,
  CODE_FOR_avx512f_vextractf32x4_1_maskm = 2461,
  CODE_FOR_avx512f_vextracti32x4_1_maskm = 2462,
  CODE_FOR_avx512dq_vextractf64x2_1_mask = 2464,
  CODE_FOR_avx512dq_vextracti64x2_1_mask = 2466,
  CODE_FOR_avx512f_vextractf32x4_1_mask = 2468,
  CODE_FOR_avx512f_vextracti32x4_1_mask = 2470,
  CODE_FOR_vec_extract_lo_v8df_maskm = 2471,
  CODE_FOR_vec_extract_lo_v8di_maskm = 2472,
  CODE_FOR_vec_extract_lo_v8df = 2473,
  CODE_FOR_vec_extract_lo_v8df_mask = 2474,
  CODE_FOR_vec_extract_lo_v8di = 2475,
  CODE_FOR_vec_extract_lo_v8di_mask = 2476,
  CODE_FOR_vec_extract_hi_v8df_maskm = 2477,
  CODE_FOR_vec_extract_hi_v8di_maskm = 2478,
  CODE_FOR_vec_extract_hi_v8df = 2479,
  CODE_FOR_vec_extract_hi_v8df_mask = 2480,
  CODE_FOR_vec_extract_hi_v8di = 2481,
  CODE_FOR_vec_extract_hi_v8di_mask = 2482,
  CODE_FOR_vec_extract_hi_v16sf_maskm = 2483,
  CODE_FOR_vec_extract_hi_v16si_maskm = 2484,
  CODE_FOR_vec_extract_hi_v16sf = 2485,
  CODE_FOR_vec_extract_hi_v16sf_mask = 2486,
  CODE_FOR_vec_extract_hi_v16si = 2487,
  CODE_FOR_vec_extract_hi_v16si_mask = 2488,
  CODE_FOR_vec_extract_lo_v16sf = 2489,
  CODE_FOR_vec_extract_lo_v16sf_mask = 2490,
  CODE_FOR_vec_extract_lo_v16si = 2491,
  CODE_FOR_vec_extract_lo_v16si_mask = 2492,
  CODE_FOR_vec_extract_lo_v4di = 2493,
  CODE_FOR_vec_extract_lo_v4di_mask = 2494,
  CODE_FOR_vec_extract_lo_v4df = 2495,
  CODE_FOR_vec_extract_lo_v4df_mask = 2496,
  CODE_FOR_vec_extract_hi_v4di = 2497,
  CODE_FOR_vec_extract_hi_v4di_mask = 2498,
  CODE_FOR_vec_extract_hi_v4df = 2499,
  CODE_FOR_vec_extract_hi_v4df_mask = 2500,
  CODE_FOR_vec_extract_lo_v8si = 2501,
  CODE_FOR_vec_extract_lo_v8si_mask = 2502,
  CODE_FOR_vec_extract_lo_v8sf = 2503,
  CODE_FOR_vec_extract_lo_v8sf_mask = 2504,
  CODE_FOR_vec_extract_lo_v8si_maskm = 2505,
  CODE_FOR_vec_extract_lo_v8sf_maskm = 2506,
  CODE_FOR_vec_extract_hi_v8si_maskm = 2507,
  CODE_FOR_vec_extract_hi_v8sf_maskm = 2508,
  CODE_FOR_vec_extract_hi_v8si_mask = 2509,
  CODE_FOR_vec_extract_hi_v8sf_mask = 2510,
  CODE_FOR_vec_extract_hi_v8si = 2511,
  CODE_FOR_vec_extract_hi_v8sf = 2512,
  CODE_FOR_vec_extract_lo_v32hi = 2513,
  CODE_FOR_vec_extract_hi_v32hi = 2514,
  CODE_FOR_vec_extract_lo_v16hi = 2515,
  CODE_FOR_vec_extract_hi_v16hi = 2516,
  CODE_FOR_vec_extract_lo_v64qi = 2517,
  CODE_FOR_vec_extract_hi_v64qi = 2518,
  CODE_FOR_vec_extract_lo_v32qi = 2519,
  CODE_FOR_vec_extract_hi_v32qi = 2520,
  CODE_FOR_avx512f_unpckhpd512_mask = 2522,
  CODE_FOR_avx_unpckhpd256 = 2523,
  CODE_FOR_avx_unpckhpd256_mask = 2524,
  CODE_FOR_avx512vl_unpckhpd128_mask = 2525,
  CODE_FOR_avx512vl_unpcklpd128_mask = 2531,
  CODE_FOR_avx512f_vmscalefv4sf = 2533,
  CODE_FOR_avx512f_vmscalefv4sf_round = 2534,
  CODE_FOR_avx512f_vmscalefv2df = 2535,
  CODE_FOR_avx512f_vmscalefv2df_round = 2536,
  CODE_FOR_avx512f_scalefv16sf = 2537,
  CODE_FOR_avx512f_scalefv16sf_round = 2538,
  CODE_FOR_avx512f_scalefv16sf_mask = 2539,
  CODE_FOR_avx512f_scalefv16sf_mask_round = 2540,
  CODE_FOR_avx512vl_scalefv8sf = 2541,
  CODE_FOR_avx512vl_scalefv8sf_round = 2542,
  CODE_FOR_avx512vl_scalefv8sf_mask = 2543,
  CODE_FOR_avx512vl_scalefv8sf_mask_round = 2544,
  CODE_FOR_avx512vl_scalefv4sf = 2545,
  CODE_FOR_avx512vl_scalefv4sf_round = 2546,
  CODE_FOR_avx512vl_scalefv4sf_mask = 2547,
  CODE_FOR_avx512vl_scalefv4sf_mask_round = 2548,
  CODE_FOR_avx512f_scalefv8df = 2549,
  CODE_FOR_avx512f_scalefv8df_round = 2550,
  CODE_FOR_avx512f_scalefv8df_mask = 2551,
  CODE_FOR_avx512f_scalefv8df_mask_round = 2552,
  CODE_FOR_avx512vl_scalefv4df = 2553,
  CODE_FOR_avx512vl_scalefv4df_round = 2554,
  CODE_FOR_avx512vl_scalefv4df_mask = 2555,
  CODE_FOR_avx512vl_scalefv4df_mask_round = 2556,
  CODE_FOR_avx512vl_scalefv2df = 2557,
  CODE_FOR_avx512vl_scalefv2df_round = 2558,
  CODE_FOR_avx512vl_scalefv2df_mask = 2559,
  CODE_FOR_avx512vl_scalefv2df_mask_round = 2560,
  CODE_FOR_avx512f_vternlogv16si = 2561,
  CODE_FOR_avx512f_vternlogv16si_maskz_1 = 2562,
  CODE_FOR_avx512vl_vternlogv8si = 2563,
  CODE_FOR_avx512vl_vternlogv8si_maskz_1 = 2564,
  CODE_FOR_avx512vl_vternlogv4si = 2565,
  CODE_FOR_avx512vl_vternlogv4si_maskz_1 = 2566,
  CODE_FOR_avx512f_vternlogv8di = 2567,
  CODE_FOR_avx512f_vternlogv8di_maskz_1 = 2568,
  CODE_FOR_avx512vl_vternlogv4di = 2569,
  CODE_FOR_avx512vl_vternlogv4di_maskz_1 = 2570,
  CODE_FOR_avx512vl_vternlogv2di = 2571,
  CODE_FOR_avx512vl_vternlogv2di_maskz_1 = 2572,
  CODE_FOR_avx512f_vternlogv16si_mask = 2573,
  CODE_FOR_avx512vl_vternlogv8si_mask = 2574,
  CODE_FOR_avx512vl_vternlogv4si_mask = 2575,
  CODE_FOR_avx512f_vternlogv8di_mask = 2576,
  CODE_FOR_avx512vl_vternlogv4di_mask = 2577,
  CODE_FOR_avx512vl_vternlogv2di_mask = 2578,
  CODE_FOR_avx512f_getexpv16sf = 2579,
  CODE_FOR_avx512f_getexpv16sf_round = 2580,
  CODE_FOR_avx512f_getexpv16sf_mask = 2581,
  CODE_FOR_avx512f_getexpv16sf_mask_round = 2582,
  CODE_FOR_avx512vl_getexpv8sf = 2583,
  CODE_FOR_avx512vl_getexpv8sf_round = 2584,
  CODE_FOR_avx512vl_getexpv8sf_mask = 2585,
  CODE_FOR_avx512vl_getexpv8sf_mask_round = 2586,
  CODE_FOR_avx512vl_getexpv4sf = 2587,
  CODE_FOR_avx512vl_getexpv4sf_round = 2588,
  CODE_FOR_avx512vl_getexpv4sf_mask = 2589,
  CODE_FOR_avx512vl_getexpv4sf_mask_round = 2590,
  CODE_FOR_avx512f_getexpv8df = 2591,
  CODE_FOR_avx512f_getexpv8df_round = 2592,
  CODE_FOR_avx512f_getexpv8df_mask = 2593,
  CODE_FOR_avx512f_getexpv8df_mask_round = 2594,
  CODE_FOR_avx512vl_getexpv4df = 2595,
  CODE_FOR_avx512vl_getexpv4df_round = 2596,
  CODE_FOR_avx512vl_getexpv4df_mask = 2597,
  CODE_FOR_avx512vl_getexpv4df_mask_round = 2598,
  CODE_FOR_avx512vl_getexpv2df = 2599,
  CODE_FOR_avx512vl_getexpv2df_round = 2600,
  CODE_FOR_avx512vl_getexpv2df_mask = 2601,
  CODE_FOR_avx512vl_getexpv2df_mask_round = 2602,
  CODE_FOR_avx512f_sgetexpv4sf = 2603,
  CODE_FOR_avx512f_sgetexpv4sf_round = 2604,
  CODE_FOR_avx512f_sgetexpv2df = 2605,
  CODE_FOR_avx512f_sgetexpv2df_round = 2606,
  CODE_FOR_avx512f_alignv16si_mask = 2608,
  CODE_FOR_avx512vl_alignv8si_mask = 2610,
  CODE_FOR_avx512vl_alignv4si_mask = 2612,
  CODE_FOR_avx512f_alignv8di_mask = 2614,
  CODE_FOR_avx512vl_alignv4di_mask = 2616,
  CODE_FOR_avx512vl_alignv2di_mask = 2618,
  CODE_FOR_avx512f_fixupimmv16sf = 2619,
  CODE_FOR_avx512f_fixupimmv16sf_round = 2620,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1 = 2621,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_1_round = 2622,
  CODE_FOR_avx512vl_fixupimmv8sf = 2623,
  CODE_FOR_avx512vl_fixupimmv8sf_round = 2624,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1 = 2625,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_1_round = 2626,
  CODE_FOR_avx512vl_fixupimmv4sf = 2627,
  CODE_FOR_avx512vl_fixupimmv4sf_round = 2628,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1 = 2629,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_1_round = 2630,
  CODE_FOR_avx512f_fixupimmv8df = 2631,
  CODE_FOR_avx512f_fixupimmv8df_round = 2632,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1 = 2633,
  CODE_FOR_avx512f_fixupimmv8df_maskz_1_round = 2634,
  CODE_FOR_avx512vl_fixupimmv4df = 2635,
  CODE_FOR_avx512vl_fixupimmv4df_round = 2636,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1 = 2637,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_1_round = 2638,
  CODE_FOR_avx512vl_fixupimmv2df = 2639,
  CODE_FOR_avx512vl_fixupimmv2df_round = 2640,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1 = 2641,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_1_round = 2642,
  CODE_FOR_avx512f_fixupimmv16sf_mask = 2643,
  CODE_FOR_avx512f_fixupimmv16sf_mask_round = 2644,
  CODE_FOR_avx512vl_fixupimmv8sf_mask = 2645,
  CODE_FOR_avx512vl_fixupimmv8sf_mask_round = 2646,
  CODE_FOR_avx512vl_fixupimmv4sf_mask = 2647,
  CODE_FOR_avx512vl_fixupimmv4sf_mask_round = 2648,
  CODE_FOR_avx512f_fixupimmv8df_mask = 2649,
  CODE_FOR_avx512f_fixupimmv8df_mask_round = 2650,
  CODE_FOR_avx512vl_fixupimmv4df_mask = 2651,
  CODE_FOR_avx512vl_fixupimmv4df_mask_round = 2652,
  CODE_FOR_avx512vl_fixupimmv2df_mask = 2653,
  CODE_FOR_avx512vl_fixupimmv2df_mask_round = 2654,
  CODE_FOR_avx512f_sfixupimmv4sf = 2655,
  CODE_FOR_avx512f_sfixupimmv4sf_round = 2656,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1 = 2657,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_1_round = 2658,
  CODE_FOR_avx512f_sfixupimmv2df = 2659,
  CODE_FOR_avx512f_sfixupimmv2df_round = 2660,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1 = 2661,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_1_round = 2662,
  CODE_FOR_avx512f_sfixupimmv4sf_mask = 2663,
  CODE_FOR_avx512f_sfixupimmv4sf_mask_round = 2664,
  CODE_FOR_avx512f_sfixupimmv2df_mask = 2665,
  CODE_FOR_avx512f_sfixupimmv2df_mask_round = 2666,
  CODE_FOR_avx512f_rndscalev16sf = 2667,
  CODE_FOR_avx512f_rndscalev16sf_round = 2668,
  CODE_FOR_avx512f_rndscalev16sf_mask = 2669,
  CODE_FOR_avx512f_rndscalev16sf_mask_round = 2670,
  CODE_FOR_avx512vl_rndscalev8sf = 2671,
  CODE_FOR_avx512vl_rndscalev8sf_round = 2672,
  CODE_FOR_avx512vl_rndscalev8sf_mask = 2673,
  CODE_FOR_avx512vl_rndscalev8sf_mask_round = 2674,
  CODE_FOR_avx512vl_rndscalev4sf = 2675,
  CODE_FOR_avx512vl_rndscalev4sf_round = 2676,
  CODE_FOR_avx512vl_rndscalev4sf_mask = 2677,
  CODE_FOR_avx512vl_rndscalev4sf_mask_round = 2678,
  CODE_FOR_avx512f_rndscalev8df = 2679,
  CODE_FOR_avx512f_rndscalev8df_round = 2680,
  CODE_FOR_avx512f_rndscalev8df_mask = 2681,
  CODE_FOR_avx512f_rndscalev8df_mask_round = 2682,
  CODE_FOR_avx512vl_rndscalev4df = 2683,
  CODE_FOR_avx512vl_rndscalev4df_round = 2684,
  CODE_FOR_avx512vl_rndscalev4df_mask = 2685,
  CODE_FOR_avx512vl_rndscalev4df_mask_round = 2686,
  CODE_FOR_avx512vl_rndscalev2df = 2687,
  CODE_FOR_avx512vl_rndscalev2df_round = 2688,
  CODE_FOR_avx512vl_rndscalev2df_mask = 2689,
  CODE_FOR_avx512vl_rndscalev2df_mask_round = 2690,
  CODE_FOR_avx512f_rndscalev4sf = 2691,
  CODE_FOR_avx512f_rndscalev4sf_round = 2692,
  CODE_FOR_avx512f_rndscalev2df = 2693,
  CODE_FOR_avx512f_rndscalev2df_round = 2694,
  CODE_FOR_avx512f_shufps512_1 = 2695,
  CODE_FOR_avx512f_shufps512_1_mask = 2696,
  CODE_FOR_avx512f_shufpd512_1 = 2697,
  CODE_FOR_avx512f_shufpd512_1_mask = 2698,
  CODE_FOR_avx_shufpd256_1 = 2699,
  CODE_FOR_avx_shufpd256_1_mask = 2700,
  CODE_FOR_sse2_shufpd_v2df_mask = 2701,
  CODE_FOR_avx2_interleave_highv4di = 2702,
  CODE_FOR_avx2_interleave_highv4di_mask = 2703,
  CODE_FOR_avx512f_interleave_highv8di_mask = 2705,
  CODE_FOR_vec_interleave_highv2di = 2706,
  CODE_FOR_vec_interleave_highv2di_mask = 2707,
  CODE_FOR_avx2_interleave_lowv4di = 2708,
  CODE_FOR_avx2_interleave_lowv4di_mask = 2709,
  CODE_FOR_avx512f_interleave_lowv8di_mask = 2711,
  CODE_FOR_vec_interleave_lowv2di = 2712,
  CODE_FOR_vec_interleave_lowv2di_mask = 2713,
  CODE_FOR_sse2_shufpd_v2di = 2714,
  CODE_FOR_sse2_shufpd_v2df = 2715,
  CODE_FOR_sse2_storehpd = 2716,
  CODE_FOR_sse2_storelpd = 2718,
  CODE_FOR_sse2_loadhpd = 2720,
  CODE_FOR_sse2_loadlpd = 2721,
  CODE_FOR_sse2_movsd = 2722,
  CODE_FOR_vec_dupv2df = 2723,
  CODE_FOR_vec_dupv2df_mask = 2724,
  CODE_FOR_vec_concatv2df = 2725,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask = 2738,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask = 2739,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask = 2740,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask = 2741,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask = 2742,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask = 2743,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask = 2744,
  CODE_FOR_avx512f_truncatev8div8si2_mask = 2745,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask = 2746,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask = 2747,
  CODE_FOR_avx512f_truncatev8div8hi2_mask = 2748,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask = 2749,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2 = 2750,
  CODE_FOR_avx512bw_truncatev32hiv32qi2 = 2751,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2 = 2752,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask = 2753,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask = 2754,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask = 2755,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask = 2765,
  CODE_FOR_avx512vl_truncatev4div4si2_mask = 2766,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask = 2767,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask = 2768,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask = 2769,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask = 2770,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask = 2771,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask = 2772,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask = 2773,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask = 2792,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask = 2793,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask = 2794,
  CODE_FOR_avx512vl_ss_truncatev2div2qi2_mask_store = 2798,
  CODE_FOR_avx512vl_truncatev2div2qi2_mask_store = 2799,
  CODE_FOR_avx512vl_us_truncatev2div2qi2_mask_store = 2800,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask = 2807,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask = 2808,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask = 2809,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask = 2810,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask = 2811,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask = 2812,
  CODE_FOR_avx512vl_ss_truncatev4siv4qi2_mask_store = 2819,
  CODE_FOR_avx512vl_truncatev4siv4qi2_mask_store = 2820,
  CODE_FOR_avx512vl_us_truncatev4siv4qi2_mask_store = 2821,
  CODE_FOR_avx512vl_ss_truncatev4div4qi2_mask_store = 2822,
  CODE_FOR_avx512vl_truncatev4div4qi2_mask_store = 2823,
  CODE_FOR_avx512vl_us_truncatev4div4qi2_mask_store = 2824,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask = 2831,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask = 2832,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask = 2833,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask = 2834,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask = 2835,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask = 2836,
  CODE_FOR_avx512vl_ss_truncatev8hiv8qi2_mask_store = 2843,
  CODE_FOR_avx512vl_truncatev8hiv8qi2_mask_store = 2844,
  CODE_FOR_avx512vl_us_truncatev8hiv8qi2_mask_store = 2845,
  CODE_FOR_avx512vl_ss_truncatev8siv8qi2_mask_store = 2846,
  CODE_FOR_avx512vl_truncatev8siv8qi2_mask_store = 2847,
  CODE_FOR_avx512vl_us_truncatev8siv8qi2_mask_store = 2848,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask = 2864,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask = 2865,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask = 2866,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask = 2867,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask = 2868,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask = 2869,
  CODE_FOR_avx512vl_ss_truncatev4siv4hi2_mask_store = 2876,
  CODE_FOR_avx512vl_truncatev4siv4hi2_mask_store = 2877,
  CODE_FOR_avx512vl_us_truncatev4siv4hi2_mask_store = 2878,
  CODE_FOR_avx512vl_ss_truncatev4div4hi2_mask_store = 2879,
  CODE_FOR_avx512vl_truncatev4div4hi2_mask_store = 2880,
  CODE_FOR_avx512vl_us_truncatev4div4hi2_mask_store = 2881,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask = 2885,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask = 2886,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask = 2887,
  CODE_FOR_avx512vl_ss_truncatev2div2hi2_mask_store = 2891,
  CODE_FOR_avx512vl_truncatev2div2hi2_mask_store = 2892,
  CODE_FOR_avx512vl_us_truncatev2div2hi2_mask_store = 2893,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask = 2900,
  CODE_FOR_avx512vl_truncatev2div2si2_mask = 2901,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask = 2902,
  CODE_FOR_avx512vl_ss_truncatev2div2si2_mask_store = 2906,
  CODE_FOR_avx512vl_truncatev2div2si2_mask_store = 2907,
  CODE_FOR_avx512vl_us_truncatev2div2si2_mask_store = 2908,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask = 2915,
  CODE_FOR_avx512f_truncatev8div16qi2_mask = 2916,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask = 2917,
  CODE_FOR_avx512f_ss_truncatev8div16qi2_mask_store = 2921,
  CODE_FOR_avx512f_truncatev8div16qi2_mask_store = 2922,
  CODE_FOR_avx512f_us_truncatev8div16qi2_mask_store = 2923,
  CODE_FOR_avx512bw_pmaddwd512v32hi = 3074,
  CODE_FOR_avx512bw_pmaddwd512v32hi_mask = 3075,
  CODE_FOR_avx512bw_pmaddwd512v16hi = 3076,
  CODE_FOR_avx512bw_pmaddwd512v16hi_mask = 3077,
  CODE_FOR_avx512bw_pmaddwd512v8hi = 3078,
  CODE_FOR_avx512bw_pmaddwd512v8hi_mask = 3079,
  CODE_FOR_avx512dq_mulv8di3 = 3082,
  CODE_FOR_avx512dq_mulv8di3_mask = 3083,
  CODE_FOR_avx512dq_mulv4di3 = 3084,
  CODE_FOR_avx512dq_mulv4di3_mask = 3085,
  CODE_FOR_avx512dq_mulv2di3 = 3086,
  CODE_FOR_avx512dq_mulv2di3_mask = 3087,
  CODE_FOR_ashrv16hi3_mask = 3095,
  CODE_FOR_ashrv8hi3_mask = 3097,
  CODE_FOR_ashrv8si3_mask = 3099,
  CODE_FOR_ashrv4si3_mask = 3101,
  CODE_FOR_ashrv16hi3 = 3102,
  CODE_FOR_ashrv8hi3 = 3103,
  CODE_FOR_ashrv8si3 = 3104,
  CODE_FOR_ashrv4si3 = 3105,
  CODE_FOR_ashrv2di3_mask = 3107,
  CODE_FOR_ashrv32hi3 = 3108,
  CODE_FOR_ashrv32hi3_mask = 3109,
  CODE_FOR_ashrv4di3 = 3110,
  CODE_FOR_ashrv4di3_mask = 3111,
  CODE_FOR_ashrv16si3 = 3112,
  CODE_FOR_ashrv16si3_mask = 3113,
  CODE_FOR_ashrv8di3 = 3114,
  CODE_FOR_ashrv8di3_mask = 3115,
  CODE_FOR_ashlv32hi3 = 3116,
  CODE_FOR_ashlv32hi3_mask = 3117,
  CODE_FOR_lshrv32hi3 = 3118,
  CODE_FOR_lshrv32hi3_mask = 3119,
  CODE_FOR_ashlv16hi3 = 3120,
  CODE_FOR_ashlv16hi3_mask = 3121,
  CODE_FOR_lshrv16hi3 = 3122,
  CODE_FOR_lshrv16hi3_mask = 3123,
  CODE_FOR_ashlv8hi3 = 3124,
  CODE_FOR_ashlv8hi3_mask = 3125,
  CODE_FOR_lshrv8hi3 = 3126,
  CODE_FOR_lshrv8hi3_mask = 3127,
  CODE_FOR_ashlv8si3 = 3128,
  CODE_FOR_ashlv8si3_mask = 3129,
  CODE_FOR_lshrv8si3 = 3130,
  CODE_FOR_lshrv8si3_mask = 3131,
  CODE_FOR_ashlv4si3 = 3132,
  CODE_FOR_ashlv4si3_mask = 3133,
  CODE_FOR_lshrv4si3 = 3134,
  CODE_FOR_lshrv4si3_mask = 3135,
  CODE_FOR_ashlv4di3 = 3136,
  CODE_FOR_ashlv4di3_mask = 3137,
  CODE_FOR_lshrv4di3 = 3138,
  CODE_FOR_lshrv4di3_mask = 3139,
  CODE_FOR_ashlv2di3 = 3140,
  CODE_FOR_ashlv2di3_mask = 3141,
  CODE_FOR_lshrv2di3 = 3142,
  CODE_FOR_lshrv2di3_mask = 3143,
  CODE_FOR_ashlv16si3 = 3144,
  CODE_FOR_ashlv16si3_mask = 3145,
  CODE_FOR_lshrv16si3 = 3146,
  CODE_FOR_lshrv16si3_mask = 3147,
  CODE_FOR_ashlv8di3 = 3148,
  CODE_FOR_ashlv8di3_mask = 3149,
  CODE_FOR_lshrv8di3 = 3150,
  CODE_FOR_lshrv8di3_mask = 3151,
  CODE_FOR_avx512bw_ashlv4ti3 = 3152,
  CODE_FOR_avx2_ashlv2ti3 = 3153,
  CODE_FOR_sse2_ashlv1ti3 = 3154,
  CODE_FOR_avx512bw_lshrv4ti3 = 3155,
  CODE_FOR_avx2_lshrv2ti3 = 3156,
  CODE_FOR_sse2_lshrv1ti3 = 3157,
  CODE_FOR_avx512f_rolvv16si = 3158,
  CODE_FOR_avx512f_rolvv16si_mask = 3159,
  CODE_FOR_avx512f_rorvv16si = 3160,
  CODE_FOR_avx512f_rorvv16si_mask = 3161,
  CODE_FOR_avx512vl_rolvv8si = 3162,
  CODE_FOR_avx512vl_rolvv8si_mask = 3163,
  CODE_FOR_avx512vl_rorvv8si = 3164,
  CODE_FOR_avx512vl_rorvv8si_mask = 3165,
  CODE_FOR_avx512vl_rolvv4si = 3166,
  CODE_FOR_avx512vl_rolvv4si_mask = 3167,
  CODE_FOR_avx512vl_rorvv4si = 3168,
  CODE_FOR_avx512vl_rorvv4si_mask = 3169,
  CODE_FOR_avx512f_rolvv8di = 3170,
  CODE_FOR_avx512f_rolvv8di_mask = 3171,
  CODE_FOR_avx512f_rorvv8di = 3172,
  CODE_FOR_avx512f_rorvv8di_mask = 3173,
  CODE_FOR_avx512vl_rolvv4di = 3174,
  CODE_FOR_avx512vl_rolvv4di_mask = 3175,
  CODE_FOR_avx512vl_rorvv4di = 3176,
  CODE_FOR_avx512vl_rorvv4di_mask = 3177,
  CODE_FOR_avx512vl_rolvv2di = 3178,
  CODE_FOR_avx512vl_rolvv2di_mask = 3179,
  CODE_FOR_avx512vl_rorvv2di = 3180,
  CODE_FOR_avx512vl_rorvv2di_mask = 3181,
  CODE_FOR_avx512f_rolv16si = 3182,
  CODE_FOR_avx512f_rolv16si_mask = 3183,
  CODE_FOR_avx512f_rorv16si = 3184,
  CODE_FOR_avx512f_rorv16si_mask = 3185,
  CODE_FOR_avx512vl_rolv8si = 3186,
  CODE_FOR_avx512vl_rolv8si_mask = 3187,
  CODE_FOR_avx512vl_rorv8si = 3188,
  CODE_FOR_avx512vl_rorv8si_mask = 3189,
  CODE_FOR_avx512vl_rolv4si = 3190,
  CODE_FOR_avx512vl_rolv4si_mask = 3191,
  CODE_FOR_avx512vl_rorv4si = 3192,
  CODE_FOR_avx512vl_rorv4si_mask = 3193,
  CODE_FOR_avx512f_rolv8di = 3194,
  CODE_FOR_avx512f_rolv8di_mask = 3195,
  CODE_FOR_avx512f_rorv8di = 3196,
  CODE_FOR_avx512f_rorv8di_mask = 3197,
  CODE_FOR_avx512vl_rolv4di = 3198,
  CODE_FOR_avx512vl_rolv4di_mask = 3199,
  CODE_FOR_avx512vl_rorv4di = 3200,
  CODE_FOR_avx512vl_rorv4di_mask = 3201,
  CODE_FOR_avx512vl_rolv2di = 3202,
  CODE_FOR_avx512vl_rolv2di_mask = 3203,
  CODE_FOR_avx512vl_rorv2di = 3204,
  CODE_FOR_avx512vl_rorv2di_mask = 3205,
  CODE_FOR_smaxv64qi3_mask = 3267,
  CODE_FOR_sminv64qi3_mask = 3269,
  CODE_FOR_umaxv64qi3_mask = 3271,
  CODE_FOR_uminv64qi3_mask = 3273,
  CODE_FOR_smaxv16qi3_mask = 3275,
  CODE_FOR_sminv16qi3_mask = 3277,
  CODE_FOR_umaxv16qi3_mask = 3279,
  CODE_FOR_uminv16qi3_mask = 3281,
  CODE_FOR_smaxv32qi3_mask = 3283,
  CODE_FOR_sminv32qi3_mask = 3285,
  CODE_FOR_umaxv32qi3_mask = 3287,
  CODE_FOR_uminv32qi3_mask = 3289,
  CODE_FOR_smaxv32hi3_mask = 3291,
  CODE_FOR_sminv32hi3_mask = 3293,
  CODE_FOR_umaxv32hi3_mask = 3295,
  CODE_FOR_uminv32hi3_mask = 3297,
  CODE_FOR_smaxv16hi3_mask = 3299,
  CODE_FOR_sminv16hi3_mask = 3301,
  CODE_FOR_umaxv16hi3_mask = 3303,
  CODE_FOR_uminv16hi3_mask = 3305,
  CODE_FOR_smaxv8hi3_mask = 3307,
  CODE_FOR_sminv8hi3_mask = 3309,
  CODE_FOR_umaxv8hi3_mask = 3311,
  CODE_FOR_uminv8hi3_mask = 3313,
  CODE_FOR_avx512bw_eqv64qi3_1 = 3338,
  CODE_FOR_avx512bw_eqv64qi3_mask_1 = 3339,
  CODE_FOR_avx512vl_eqv16qi3_1 = 3340,
  CODE_FOR_avx512vl_eqv16qi3_mask_1 = 3341,
  CODE_FOR_avx512vl_eqv32qi3_1 = 3342,
  CODE_FOR_avx512vl_eqv32qi3_mask_1 = 3343,
  CODE_FOR_avx512bw_eqv32hi3_1 = 3344,
  CODE_FOR_avx512bw_eqv32hi3_mask_1 = 3345,
  CODE_FOR_avx512vl_eqv16hi3_1 = 3346,
  CODE_FOR_avx512vl_eqv16hi3_mask_1 = 3347,
  CODE_FOR_avx512vl_eqv8hi3_1 = 3348,
  CODE_FOR_avx512vl_eqv8hi3_mask_1 = 3349,
  CODE_FOR_avx512f_eqv16si3_1 = 3350,
  CODE_FOR_avx512f_eqv16si3_mask_1 = 3351,
  CODE_FOR_avx512vl_eqv8si3_1 = 3352,
  CODE_FOR_avx512vl_eqv8si3_mask_1 = 3353,
  CODE_FOR_avx512vl_eqv4si3_1 = 3354,
  CODE_FOR_avx512vl_eqv4si3_mask_1 = 3355,
  CODE_FOR_avx512f_eqv8di3_1 = 3356,
  CODE_FOR_avx512f_eqv8di3_mask_1 = 3357,
  CODE_FOR_avx512vl_eqv4di3_1 = 3358,
  CODE_FOR_avx512vl_eqv4di3_mask_1 = 3359,
  CODE_FOR_avx512vl_eqv2di3_1 = 3360,
  CODE_FOR_avx512vl_eqv2di3_mask_1 = 3361,
  CODE_FOR_sse4_2_gtv2di3 = 3366,
  CODE_FOR_avx2_gtv32qi3 = 3367,
  CODE_FOR_avx2_gtv16hi3 = 3368,
  CODE_FOR_avx2_gtv8si3 = 3369,
  CODE_FOR_avx2_gtv4di3 = 3370,
  CODE_FOR_avx512f_gtv16si3 = 3371,
  CODE_FOR_avx512f_gtv16si3_mask = 3372,
  CODE_FOR_avx512vl_gtv8si3 = 3373,
  CODE_FOR_avx512vl_gtv8si3_mask = 3374,
  CODE_FOR_avx512vl_gtv4si3 = 3375,
  CODE_FOR_avx512vl_gtv4si3_mask = 3376,
  CODE_FOR_avx512f_gtv8di3 = 3377,
  CODE_FOR_avx512f_gtv8di3_mask = 3378,
  CODE_FOR_avx512vl_gtv4di3 = 3379,
  CODE_FOR_avx512vl_gtv4di3_mask = 3380,
  CODE_FOR_avx512vl_gtv2di3 = 3381,
  CODE_FOR_avx512vl_gtv2di3_mask = 3382,
  CODE_FOR_avx512bw_gtv64qi3 = 3383,
  CODE_FOR_avx512bw_gtv64qi3_mask = 3384,
  CODE_FOR_avx512vl_gtv16qi3 = 3385,
  CODE_FOR_avx512vl_gtv16qi3_mask = 3386,
  CODE_FOR_avx512vl_gtv32qi3 = 3387,
  CODE_FOR_avx512vl_gtv32qi3_mask = 3388,
  CODE_FOR_avx512bw_gtv32hi3 = 3389,
  CODE_FOR_avx512bw_gtv32hi3_mask = 3390,
  CODE_FOR_avx512vl_gtv16hi3 = 3391,
  CODE_FOR_avx512vl_gtv16hi3_mask = 3392,
  CODE_FOR_avx512vl_gtv8hi3 = 3393,
  CODE_FOR_avx512vl_gtv8hi3_mask = 3394,
  CODE_FOR_sse2_gtv16qi3 = 3395,
  CODE_FOR_sse2_gtv8hi3 = 3396,
  CODE_FOR_sse2_gtv4si3 = 3397,
  CODE_FOR_andv16si3_mask = 3417,
  CODE_FOR_iorv16si3_mask = 3419,
  CODE_FOR_xorv16si3_mask = 3421,
  CODE_FOR_andv8si3_mask = 3423,
  CODE_FOR_iorv8si3_mask = 3425,
  CODE_FOR_xorv8si3_mask = 3427,
  CODE_FOR_andv4si3_mask = 3429,
  CODE_FOR_iorv4si3_mask = 3431,
  CODE_FOR_xorv4si3_mask = 3433,
  CODE_FOR_andv8di3_mask = 3435,
  CODE_FOR_iorv8di3_mask = 3437,
  CODE_FOR_xorv8di3_mask = 3439,
  CODE_FOR_andv4di3_mask = 3441,
  CODE_FOR_iorv4di3_mask = 3443,
  CODE_FOR_xorv4di3_mask = 3445,
  CODE_FOR_andv2di3_mask = 3447,
  CODE_FOR_iorv2di3_mask = 3449,
  CODE_FOR_xorv2di3_mask = 3451,
  CODE_FOR_avx512bw_testmv64qi3 = 3488,
  CODE_FOR_avx512bw_testmv64qi3_mask = 3489,
  CODE_FOR_avx512vl_testmv16qi3 = 3490,
  CODE_FOR_avx512vl_testmv16qi3_mask = 3491,
  CODE_FOR_avx512vl_testmv32qi3 = 3492,
  CODE_FOR_avx512vl_testmv32qi3_mask = 3493,
  CODE_FOR_avx512bw_testmv32hi3 = 3494,
  CODE_FOR_avx512bw_testmv32hi3_mask = 3495,
  CODE_FOR_avx512vl_testmv16hi3 = 3496,
  CODE_FOR_avx512vl_testmv16hi3_mask = 3497,
  CODE_FOR_avx512vl_testmv8hi3 = 3498,
  CODE_FOR_avx512vl_testmv8hi3_mask = 3499,
  CODE_FOR_avx512f_testmv16si3 = 3500,
  CODE_FOR_avx512f_testmv16si3_mask = 3501,
  CODE_FOR_avx512vl_testmv8si3 = 3502,
  CODE_FOR_avx512vl_testmv8si3_mask = 3503,
  CODE_FOR_avx512vl_testmv4si3 = 3504,
  CODE_FOR_avx512vl_testmv4si3_mask = 3505,
  CODE_FOR_avx512f_testmv8di3 = 3506,
  CODE_FOR_avx512f_testmv8di3_mask = 3507,
  CODE_FOR_avx512vl_testmv4di3 = 3508,
  CODE_FOR_avx512vl_testmv4di3_mask = 3509,
  CODE_FOR_avx512vl_testmv2di3 = 3510,
  CODE_FOR_avx512vl_testmv2di3_mask = 3511,
  CODE_FOR_avx512bw_testnmv64qi3 = 3512,
  CODE_FOR_avx512bw_testnmv64qi3_mask = 3513,
  CODE_FOR_avx512vl_testnmv16qi3 = 3514,
  CODE_FOR_avx512vl_testnmv16qi3_mask = 3515,
  CODE_FOR_avx512vl_testnmv32qi3 = 3516,
  CODE_FOR_avx512vl_testnmv32qi3_mask = 3517,
  CODE_FOR_avx512bw_testnmv32hi3 = 3518,
  CODE_FOR_avx512bw_testnmv32hi3_mask = 3519,
  CODE_FOR_avx512vl_testnmv16hi3 = 3520,
  CODE_FOR_avx512vl_testnmv16hi3_mask = 3521,
  CODE_FOR_avx512vl_testnmv8hi3 = 3522,
  CODE_FOR_avx512vl_testnmv8hi3_mask = 3523,
  CODE_FOR_avx512f_testnmv16si3 = 3524,
  CODE_FOR_avx512f_testnmv16si3_mask = 3525,
  CODE_FOR_avx512vl_testnmv8si3 = 3526,
  CODE_FOR_avx512vl_testnmv8si3_mask = 3527,
  CODE_FOR_avx512vl_testnmv4si3 = 3528,
  CODE_FOR_avx512vl_testnmv4si3_mask = 3529,
  CODE_FOR_avx512f_testnmv8di3 = 3530,
  CODE_FOR_avx512f_testnmv8di3_mask = 3531,
  CODE_FOR_avx512vl_testnmv4di3 = 3532,
  CODE_FOR_avx512vl_testnmv4di3_mask = 3533,
  CODE_FOR_avx512vl_testnmv2di3 = 3534,
  CODE_FOR_avx512vl_testnmv2di3_mask = 3535,
  CODE_FOR_avx512bw_packsswb = 3536,
  CODE_FOR_avx512bw_packsswb_mask = 3537,
  CODE_FOR_avx2_packsswb = 3538,
  CODE_FOR_avx2_packsswb_mask = 3539,
  CODE_FOR_sse2_packsswb = 3540,
  CODE_FOR_sse2_packsswb_mask = 3541,
  CODE_FOR_avx512bw_packssdw = 3542,
  CODE_FOR_avx512bw_packssdw_mask = 3543,
  CODE_FOR_avx2_packssdw = 3544,
  CODE_FOR_avx2_packssdw_mask = 3545,
  CODE_FOR_sse2_packssdw = 3546,
  CODE_FOR_sse2_packssdw_mask = 3547,
  CODE_FOR_avx512bw_packuswb = 3548,
  CODE_FOR_avx512bw_packuswb_mask = 3549,
  CODE_FOR_avx2_packuswb = 3550,
  CODE_FOR_avx2_packuswb_mask = 3551,
  CODE_FOR_sse2_packuswb = 3552,
  CODE_FOR_sse2_packuswb_mask = 3553,
  CODE_FOR_avx512bw_interleave_highv64qi = 3554,
  CODE_FOR_avx512bw_interleave_highv64qi_mask = 3555,
  CODE_FOR_avx2_interleave_highv32qi = 3556,
  CODE_FOR_avx2_interleave_highv32qi_mask = 3557,
  CODE_FOR_vec_interleave_highv16qi = 3558,
  CODE_FOR_vec_interleave_highv16qi_mask = 3559,
  CODE_FOR_avx512bw_interleave_lowv64qi = 3560,
  CODE_FOR_avx512bw_interleave_lowv64qi_mask = 3561,
  CODE_FOR_avx2_interleave_lowv32qi = 3562,
  CODE_FOR_avx2_interleave_lowv32qi_mask = 3563,
  CODE_FOR_vec_interleave_lowv16qi = 3564,
  CODE_FOR_vec_interleave_lowv16qi_mask = 3565,
  CODE_FOR_avx512bw_interleave_highv32hi = 3566,
  CODE_FOR_avx512bw_interleave_highv32hi_mask = 3567,
  CODE_FOR_avx2_interleave_highv16hi = 3568,
  CODE_FOR_avx2_interleave_highv16hi_mask = 3569,
  CODE_FOR_vec_interleave_highv8hi = 3570,
  CODE_FOR_vec_interleave_highv8hi_mask = 3571,
  CODE_FOR_avx512bw_interleave_lowv32hi_mask = 3573,
  CODE_FOR_avx2_interleave_lowv16hi = 3574,
  CODE_FOR_avx2_interleave_lowv16hi_mask = 3575,
  CODE_FOR_vec_interleave_lowv8hi = 3576,
  CODE_FOR_vec_interleave_lowv8hi_mask = 3577,
  CODE_FOR_avx2_interleave_highv8si = 3578,
  CODE_FOR_avx2_interleave_highv8si_mask = 3579,
  CODE_FOR_avx512f_interleave_highv16si_mask = 3581,
  CODE_FOR_vec_interleave_highv4si = 3582,
  CODE_FOR_vec_interleave_highv4si_mask = 3583,
  CODE_FOR_avx2_interleave_lowv8si = 3584,
  CODE_FOR_avx2_interleave_lowv8si_mask = 3585,
  CODE_FOR_avx512f_interleave_lowv16si_mask = 3587,
  CODE_FOR_vec_interleave_lowv4si = 3588,
  CODE_FOR_vec_interleave_lowv4si_mask = 3589,
  CODE_FOR_sse4_1_pinsrb = 3590,
  CODE_FOR_sse2_pinsrw = 3591,
  CODE_FOR_sse4_1_pinsrd = 3592,
  CODE_FOR_sse4_1_pinsrq = 3593,
  CODE_FOR_avx512dq_vinsertf64x2_1_mask = 3595,
  CODE_FOR_avx512dq_vinserti64x2_1_mask = 3597,
  CODE_FOR_avx512f_vinsertf32x4_1_mask = 3599,
  CODE_FOR_avx512f_vinserti32x4_1_mask = 3601,
  CODE_FOR_vec_set_lo_v16sf = 3602,
  CODE_FOR_vec_set_lo_v16sf_mask = 3603,
  CODE_FOR_vec_set_lo_v16si = 3604,
  CODE_FOR_vec_set_lo_v16si_mask = 3605,
  CODE_FOR_vec_set_hi_v16sf = 3606,
  CODE_FOR_vec_set_hi_v16sf_mask = 3607,
  CODE_FOR_vec_set_hi_v16si = 3608,
  CODE_FOR_vec_set_hi_v16si_mask = 3609,
  CODE_FOR_vec_set_lo_v8df = 3610,
  CODE_FOR_vec_set_lo_v8df_mask = 3611,
  CODE_FOR_vec_set_lo_v8di = 3612,
  CODE_FOR_vec_set_lo_v8di_mask = 3613,
  CODE_FOR_vec_set_hi_v8df = 3614,
  CODE_FOR_vec_set_hi_v8df_mask = 3615,
  CODE_FOR_vec_set_hi_v8di = 3616,
  CODE_FOR_vec_set_hi_v8di_mask = 3617,
  CODE_FOR_avx512dq_shuf_i64x2_1_mask = 3619,
  CODE_FOR_avx512dq_shuf_f64x2_1_mask = 3621,
  CODE_FOR_avx512f_shuf_f64x2_1 = 3622,
  CODE_FOR_avx512f_shuf_f64x2_1_mask = 3623,
  CODE_FOR_avx512f_shuf_i64x2_1 = 3624,
  CODE_FOR_avx512f_shuf_i64x2_1_mask = 3625,
  CODE_FOR_avx512vl_shuf_i32x4_1 = 3626,
  CODE_FOR_avx512vl_shuf_i32x4_1_mask = 3627,
  CODE_FOR_avx512vl_shuf_f32x4_1 = 3628,
  CODE_FOR_avx512vl_shuf_f32x4_1_mask = 3629,
  CODE_FOR_avx512f_shuf_f32x4_1 = 3630,
  CODE_FOR_avx512f_shuf_f32x4_1_mask = 3631,
  CODE_FOR_avx512f_shuf_i32x4_1 = 3632,
  CODE_FOR_avx512f_shuf_i32x4_1_mask = 3633,
  CODE_FOR_avx512f_pshufd_1 = 3634,
  CODE_FOR_avx512f_pshufd_1_mask = 3635,
  CODE_FOR_avx2_pshufd_1 = 3636,
  CODE_FOR_avx2_pshufd_1_mask = 3637,
  CODE_FOR_sse2_pshufd_1 = 3638,
  CODE_FOR_sse2_pshufd_1_mask = 3639,
  CODE_FOR_avx512bw_pshuflwv32hi_mask = 3641,
  CODE_FOR_avx2_pshuflw_1 = 3642,
  CODE_FOR_avx2_pshuflw_1_mask = 3643,
  CODE_FOR_sse2_pshuflw_1 = 3644,
  CODE_FOR_sse2_pshuflw_1_mask = 3645,
  CODE_FOR_avx512bw_pshufhwv32hi_mask = 3647,
  CODE_FOR_avx2_pshufhw_1 = 3648,
  CODE_FOR_avx2_pshufhw_1_mask = 3649,
  CODE_FOR_sse2_pshufhw_1 = 3650,
  CODE_FOR_sse2_pshufhw_1_mask = 3651,
  CODE_FOR_sse2_loadld = 3652,
  CODE_FOR_vec_concatv2di = 3674,
  CODE_FOR_avx512f_psadbw = 3687,
  CODE_FOR_avx2_psadbw = 3688,
  CODE_FOR_sse2_psadbw = 3689,
  CODE_FOR_avx_movmskps256 = 3690,
  CODE_FOR_sse_movmskps = 3691,
  CODE_FOR_avx_movmskpd256 = 3692,
  CODE_FOR_sse2_movmskpd = 3693,
  CODE_FOR_avx2_pmovmskb = 3698,
  CODE_FOR_sse2_pmovmskb = 3699,
  CODE_FOR_sse_ldmxcsr = 3704,
  CODE_FOR_sse_stmxcsr = 3705,
  CODE_FOR_sse2_clflush = 3706,
  CODE_FOR_sse3_mwait = 3707,
  CODE_FOR_sse3_monitor_si = 3708,
  CODE_FOR_sse3_monitor_di = 3709,
  CODE_FOR_avx2_phaddwv16hi3 = 3710,
  CODE_FOR_avx2_phaddswv16hi3 = 3711,
  CODE_FOR_avx2_phsubwv16hi3 = 3712,
  CODE_FOR_avx2_phsubswv16hi3 = 3713,
  CODE_FOR_ssse3_phaddwv8hi3 = 3714,
  CODE_FOR_ssse3_phaddswv8hi3 = 3715,
  CODE_FOR_ssse3_phsubwv8hi3 = 3716,
  CODE_FOR_ssse3_phsubswv8hi3 = 3717,
  CODE_FOR_ssse3_phaddwv4hi3 = 3718,
  CODE_FOR_ssse3_phaddswv4hi3 = 3719,
  CODE_FOR_ssse3_phsubwv4hi3 = 3720,
  CODE_FOR_ssse3_phsubswv4hi3 = 3721,
  CODE_FOR_avx2_phadddv8si3 = 3722,
  CODE_FOR_avx2_phsubdv8si3 = 3723,
  CODE_FOR_ssse3_phadddv4si3 = 3724,
  CODE_FOR_ssse3_phsubdv4si3 = 3725,
  CODE_FOR_ssse3_phadddv2si3 = 3726,
  CODE_FOR_ssse3_phsubdv2si3 = 3727,
  CODE_FOR_avx2_pmaddubsw256 = 3728,
  CODE_FOR_avx512bw_pmaddubsw512v8hi = 3729,
  CODE_FOR_avx512bw_pmaddubsw512v8hi_mask = 3730,
  CODE_FOR_avx512bw_pmaddubsw512v16hi = 3731,
  CODE_FOR_avx512bw_pmaddubsw512v16hi_mask = 3732,
  CODE_FOR_avx512bw_pmaddubsw512v32hi = 3733,
  CODE_FOR_avx512bw_pmaddubsw512v32hi_mask = 3734,
  CODE_FOR_avx512bw_umulhrswv32hi3 = 3735,
  CODE_FOR_avx512bw_umulhrswv32hi3_mask = 3736,
  CODE_FOR_ssse3_pmaddubsw128 = 3737,
  CODE_FOR_ssse3_pmaddubsw = 3738,
  CODE_FOR_avx512bw_pshufbv64qi3 = 3746,
  CODE_FOR_avx512bw_pshufbv64qi3_mask = 3747,
  CODE_FOR_avx2_pshufbv32qi3 = 3748,
  CODE_FOR_avx2_pshufbv32qi3_mask = 3749,
  CODE_FOR_ssse3_pshufbv16qi3 = 3750,
  CODE_FOR_ssse3_pshufbv16qi3_mask = 3751,
  CODE_FOR_ssse3_pshufbv8qi3 = 3752,
  CODE_FOR_avx2_psignv32qi3 = 3753,
  CODE_FOR_ssse3_psignv16qi3 = 3754,
  CODE_FOR_avx2_psignv16hi3 = 3755,
  CODE_FOR_ssse3_psignv8hi3 = 3756,
  CODE_FOR_avx2_psignv8si3 = 3757,
  CODE_FOR_ssse3_psignv4si3 = 3758,
  CODE_FOR_ssse3_psignv8qi3 = 3759,
  CODE_FOR_ssse3_psignv4hi3 = 3760,
  CODE_FOR_ssse3_psignv2si3 = 3761,
  CODE_FOR_avx512bw_palignrv64qi_mask = 3762,
  CODE_FOR_avx2_palignrv32qi_mask = 3763,
  CODE_FOR_ssse3_palignrv16qi_mask = 3764,
  CODE_FOR_avx512bw_palignrv4ti = 3765,
  CODE_FOR_avx2_palignrv2ti = 3766,
  CODE_FOR_ssse3_palignrti = 3767,
  CODE_FOR_ssse3_palignrdi = 3768,
  CODE_FOR_absv16si2_mask = 3781,
  CODE_FOR_absv8si2_mask = 3782,
  CODE_FOR_absv4si2_mask = 3783,
  CODE_FOR_absv8di2_mask = 3784,
  CODE_FOR_absv4di2_mask = 3785,
  CODE_FOR_absv2di2_mask = 3786,
  CODE_FOR_absv64qi2_mask = 3787,
  CODE_FOR_absv16qi2_mask = 3788,
  CODE_FOR_absv32qi2_mask = 3789,
  CODE_FOR_absv32hi2_mask = 3790,
  CODE_FOR_absv16hi2_mask = 3791,
  CODE_FOR_absv8hi2_mask = 3792,
  CODE_FOR_absv8qi2 = 3793,
  CODE_FOR_absv4hi2 = 3794,
  CODE_FOR_absv2si2 = 3795,
  CODE_FOR_sse4a_movntsf = 3796,
  CODE_FOR_sse4a_movntdf = 3797,
  CODE_FOR_sse4a_vmmovntv4sf = 3798,
  CODE_FOR_sse4a_vmmovntv2df = 3799,
  CODE_FOR_sse4a_extrqi = 3800,
  CODE_FOR_sse4a_extrq = 3801,
  CODE_FOR_sse4a_insertqi = 3802,
  CODE_FOR_sse4a_insertq = 3803,
  CODE_FOR_avx_blendps256 = 3804,
  CODE_FOR_sse4_1_blendps = 3805,
  CODE_FOR_avx_blendpd256 = 3806,
  CODE_FOR_sse4_1_blendpd = 3807,
  CODE_FOR_avx_blendvps256 = 3808,
  CODE_FOR_sse4_1_blendvps = 3809,
  CODE_FOR_avx_blendvpd256 = 3810,
  CODE_FOR_sse4_1_blendvpd = 3811,
  CODE_FOR_avx_dpps256 = 3812,
  CODE_FOR_sse4_1_dpps = 3813,
  CODE_FOR_avx_dppd256 = 3814,
  CODE_FOR_sse4_1_dppd = 3815,
  CODE_FOR_avx512f_movntdqa = 3816,
  CODE_FOR_avx2_movntdqa = 3817,
  CODE_FOR_sse4_1_movntdqa = 3818,
  CODE_FOR_avx2_mpsadbw = 3819,
  CODE_FOR_sse4_1_mpsadbw = 3820,
  CODE_FOR_avx512bw_packusdw = 3821,
  CODE_FOR_avx512bw_packusdw_mask = 3822,
  CODE_FOR_avx2_packusdw = 3823,
  CODE_FOR_avx2_packusdw_mask = 3824,
  CODE_FOR_sse4_1_packusdw = 3825,
  CODE_FOR_sse4_1_packusdw_mask = 3826,
  CODE_FOR_avx2_pblendvb = 3827,
  CODE_FOR_sse4_1_pblendvb = 3828,
  CODE_FOR_sse4_1_pblendw = 3829,
  CODE_FOR_avx2_pblenddv8si = 3831,
  CODE_FOR_avx2_pblenddv4si = 3832,
  CODE_FOR_sse4_1_phminposuw = 3833,
  CODE_FOR_avx2_sign_extendv16qiv16hi2 = 3834,
  CODE_FOR_avx2_sign_extendv16qiv16hi2_mask = 3835,
  CODE_FOR_avx2_zero_extendv16qiv16hi2 = 3836,
  CODE_FOR_avx2_zero_extendv16qiv16hi2_mask = 3837,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2 = 3838,
  CODE_FOR_avx512bw_sign_extendv32qiv32hi2_mask = 3839,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2 = 3840,
  CODE_FOR_avx512bw_zero_extendv32qiv32hi2_mask = 3841,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2 = 3842,
  CODE_FOR_sse4_1_sign_extendv8qiv8hi2_mask = 3843,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2 = 3844,
  CODE_FOR_sse4_1_zero_extendv8qiv8hi2_mask = 3845,
  CODE_FOR_avx512f_sign_extendv16qiv16si2_mask = 3847,
  CODE_FOR_avx512f_zero_extendv16qiv16si2_mask = 3849,
  CODE_FOR_avx2_sign_extendv8qiv8si2 = 3850,
  CODE_FOR_avx2_sign_extendv8qiv8si2_mask = 3851,
  CODE_FOR_avx2_zero_extendv8qiv8si2 = 3852,
  CODE_FOR_avx2_zero_extendv8qiv8si2_mask = 3853,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2 = 3854,
  CODE_FOR_sse4_1_sign_extendv4qiv4si2_mask = 3855,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2 = 3856,
  CODE_FOR_sse4_1_zero_extendv4qiv4si2_mask = 3857,
  CODE_FOR_avx512f_sign_extendv16hiv16si2 = 3858,
  CODE_FOR_avx512f_sign_extendv16hiv16si2_mask = 3859,
  CODE_FOR_avx512f_zero_extendv16hiv16si2 = 3860,
  CODE_FOR_avx512f_zero_extendv16hiv16si2_mask = 3861,
  CODE_FOR_avx2_sign_extendv8hiv8si2 = 3862,
  CODE_FOR_avx2_sign_extendv8hiv8si2_mask = 3863,
  CODE_FOR_avx2_zero_extendv8hiv8si2 = 3864,
  CODE_FOR_avx2_zero_extendv8hiv8si2_mask = 3865,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2 = 3866,
  CODE_FOR_sse4_1_sign_extendv4hiv4si2_mask = 3867,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2 = 3868,
  CODE_FOR_sse4_1_zero_extendv4hiv4si2_mask = 3869,
  CODE_FOR_avx512f_sign_extendv8qiv8di2 = 3870,
  CODE_FOR_avx512f_sign_extendv8qiv8di2_mask = 3871,
  CODE_FOR_avx512f_zero_extendv8qiv8di2 = 3872,
  CODE_FOR_avx512f_zero_extendv8qiv8di2_mask = 3873,
  CODE_FOR_avx2_sign_extendv4qiv4di2 = 3874,
  CODE_FOR_avx2_sign_extendv4qiv4di2_mask = 3875,
  CODE_FOR_avx2_zero_extendv4qiv4di2 = 3876,
  CODE_FOR_avx2_zero_extendv4qiv4di2_mask = 3877,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2 = 3878,
  CODE_FOR_sse4_1_sign_extendv2qiv2di2_mask = 3879,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2 = 3880,
  CODE_FOR_sse4_1_zero_extendv2qiv2di2_mask = 3881,
  CODE_FOR_avx512f_sign_extendv8hiv8di2 = 3882,
  CODE_FOR_avx512f_sign_extendv8hiv8di2_mask = 3883,
  CODE_FOR_avx512f_zero_extendv8hiv8di2 = 3884,
  CODE_FOR_avx512f_zero_extendv8hiv8di2_mask = 3885,
  CODE_FOR_avx2_sign_extendv4hiv4di2 = 3886,
  CODE_FOR_avx2_sign_extendv4hiv4di2_mask = 3887,
  CODE_FOR_avx2_zero_extendv4hiv4di2 = 3888,
  CODE_FOR_avx2_zero_extendv4hiv4di2_mask = 3889,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2 = 3890,
  CODE_FOR_sse4_1_sign_extendv2hiv2di2_mask = 3891,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2 = 3892,
  CODE_FOR_sse4_1_zero_extendv2hiv2di2_mask = 3893,
  CODE_FOR_avx512f_sign_extendv8siv8di2 = 3894,
  CODE_FOR_avx512f_sign_extendv8siv8di2_mask = 3895,
  CODE_FOR_avx512f_zero_extendv8siv8di2 = 3896,
  CODE_FOR_avx512f_zero_extendv8siv8di2_mask = 3897,
  CODE_FOR_avx2_sign_extendv4siv4di2 = 3898,
  CODE_FOR_avx2_sign_extendv4siv4di2_mask = 3899,
  CODE_FOR_avx2_zero_extendv4siv4di2 = 3900,
  CODE_FOR_avx2_zero_extendv4siv4di2_mask = 3901,
  CODE_FOR_sse4_1_sign_extendv2siv2di2 = 3902,
  CODE_FOR_sse4_1_sign_extendv2siv2di2_mask = 3903,
  CODE_FOR_sse4_1_zero_extendv2siv2di2 = 3904,
  CODE_FOR_sse4_1_zero_extendv2siv2di2_mask = 3905,
  CODE_FOR_avx_vtestps256 = 3906,
  CODE_FOR_avx_vtestps = 3907,
  CODE_FOR_avx_vtestpd256 = 3908,
  CODE_FOR_avx_vtestpd = 3909,
  CODE_FOR_sse4_1_ptestv16qi = 3910,
  CODE_FOR_sse4_1_ptestv8hi = 3911,
  CODE_FOR_sse4_1_ptestv4si = 3912,
  CODE_FOR_sse4_1_ptestv2di = 3913,
  CODE_FOR_sse4_1_ptestv4sf = 3914,
  CODE_FOR_sse4_1_ptestv2df = 3915,
  CODE_FOR_avx_ptestv32qi = 3916,
  CODE_FOR_avx_ptestv16hi = 3917,
  CODE_FOR_avx_ptestv8si = 3918,
  CODE_FOR_avx_ptestv4di = 3919,
  CODE_FOR_avx_ptestv8sf = 3920,
  CODE_FOR_avx_ptestv4df = 3921,
  CODE_FOR_ptesttf2 = 3922,
  CODE_FOR_avx_roundps256 = 3923,
  CODE_FOR_sse4_1_roundps = 3924,
  CODE_FOR_avx_roundpd256 = 3925,
  CODE_FOR_sse4_1_roundpd = 3926,
  CODE_FOR_sse4_1_roundss = 3927,
  CODE_FOR_sse4_1_roundsd = 3928,
  CODE_FOR_sse4_2_pcmpestr = 3929,
  CODE_FOR_sse4_2_pcmpestri = 3930,
  CODE_FOR_sse4_2_pcmpestrm = 3931,
  CODE_FOR_sse4_2_pcmpestr_cconly = 3932,
  CODE_FOR_sse4_2_pcmpistr = 3933,
  CODE_FOR_sse4_2_pcmpistri = 3934,
  CODE_FOR_sse4_2_pcmpistrm = 3935,
  CODE_FOR_sse4_2_pcmpistr_cconly = 3936,
  CODE_FOR_avx512er_exp2v16sf = 3953,
  CODE_FOR_avx512er_exp2v16sf_round = 3954,
  CODE_FOR_avx512er_exp2v16sf_mask = 3955,
  CODE_FOR_avx512er_exp2v16sf_mask_round = 3956,
  CODE_FOR_avx512er_exp2v8df = 3957,
  CODE_FOR_avx512er_exp2v8df_round = 3958,
  CODE_FOR_avx512er_exp2v8df_mask = 3959,
  CODE_FOR_avx512er_exp2v8df_mask_round = 3960,
  CODE_FOR_avx512er_rcp28v16sf_mask = 3963,
  CODE_FOR_avx512er_rcp28v16sf_mask_round = 3964,
  CODE_FOR_avx512er_rcp28v8df_mask = 3967,
  CODE_FOR_avx512er_rcp28v8df_mask_round = 3968,
  CODE_FOR_avx512er_vmrcp28v4sf = 3969,
  CODE_FOR_avx512er_vmrcp28v4sf_round = 3970,
  CODE_FOR_avx512er_vmrcp28v2df = 3971,
  CODE_FOR_avx512er_vmrcp28v2df_round = 3972,
  CODE_FOR_avx512er_rsqrt28v16sf_mask = 3975,
  CODE_FOR_avx512er_rsqrt28v16sf_mask_round = 3976,
  CODE_FOR_avx512er_rsqrt28v8df_mask = 3979,
  CODE_FOR_avx512er_rsqrt28v8df_mask_round = 3980,
  CODE_FOR_avx512er_vmrsqrt28v4sf = 3981,
  CODE_FOR_avx512er_vmrsqrt28v4sf_round = 3982,
  CODE_FOR_avx512er_vmrsqrt28v2df = 3983,
  CODE_FOR_avx512er_vmrsqrt28v2df_round = 3984,
  CODE_FOR_xop_pmacsww = 3985,
  CODE_FOR_xop_pmacssww = 3986,
  CODE_FOR_xop_pmacsdd = 3987,
  CODE_FOR_xop_pmacssdd = 3988,
  CODE_FOR_xop_pmacsdql = 3989,
  CODE_FOR_xop_pmacssdql = 3990,
  CODE_FOR_xop_pmacsdqh = 3991,
  CODE_FOR_xop_pmacssdqh = 3992,
  CODE_FOR_xop_pmacswd = 3993,
  CODE_FOR_xop_pmacsswd = 3994,
  CODE_FOR_xop_pmadcswd = 3995,
  CODE_FOR_xop_pmadcsswd = 3996,
  CODE_FOR_xop_pcmov_v32qi256 = 3997,
  CODE_FOR_xop_pcmov_v16qi = 3998,
  CODE_FOR_xop_pcmov_v16hi256 = 3999,
  CODE_FOR_xop_pcmov_v8hi = 4000,
  CODE_FOR_xop_pcmov_v16si512 = 4001,
  CODE_FOR_xop_pcmov_v8si256 = 4002,
  CODE_FOR_xop_pcmov_v4si = 4003,
  CODE_FOR_xop_pcmov_v8di512 = 4004,
  CODE_FOR_xop_pcmov_v4di256 = 4005,
  CODE_FOR_xop_pcmov_v2di = 4006,
  CODE_FOR_xop_pcmov_v16sf512 = 4007,
  CODE_FOR_xop_pcmov_v8sf256 = 4008,
  CODE_FOR_xop_pcmov_v4sf = 4009,
  CODE_FOR_xop_pcmov_v8df512 = 4010,
  CODE_FOR_xop_pcmov_v4df256 = 4011,
  CODE_FOR_xop_pcmov_v2df = 4012,
  CODE_FOR_xop_phaddbw = 4013,
  CODE_FOR_xop_phaddubw = 4014,
  CODE_FOR_xop_phaddbd = 4015,
  CODE_FOR_xop_phaddubd = 4016,
  CODE_FOR_xop_phaddbq = 4017,
  CODE_FOR_xop_phaddubq = 4018,
  CODE_FOR_xop_phaddwd = 4019,
  CODE_FOR_xop_phadduwd = 4020,
  CODE_FOR_xop_phaddwq = 4021,
  CODE_FOR_xop_phadduwq = 4022,
  CODE_FOR_xop_phadddq = 4023,
  CODE_FOR_xop_phaddudq = 4024,
  CODE_FOR_xop_phsubbw = 4025,
  CODE_FOR_xop_phsubwd = 4026,
  CODE_FOR_xop_phsubdq = 4027,
  CODE_FOR_xop_pperm = 4028,
  CODE_FOR_xop_pperm_pack_v2di_v4si = 4029,
  CODE_FOR_xop_pperm_pack_v4si_v8hi = 4030,
  CODE_FOR_xop_pperm_pack_v8hi_v16qi = 4031,
  CODE_FOR_xop_rotlv16qi3 = 4032,
  CODE_FOR_xop_rotlv8hi3 = 4033,
  CODE_FOR_xop_rotlv4si3 = 4034,
  CODE_FOR_xop_rotlv2di3 = 4035,
  CODE_FOR_xop_rotrv16qi3 = 4036,
  CODE_FOR_xop_rotrv8hi3 = 4037,
  CODE_FOR_xop_rotrv4si3 = 4038,
  CODE_FOR_xop_rotrv2di3 = 4039,
  CODE_FOR_xop_vrotlv16qi3 = 4040,
  CODE_FOR_xop_vrotlv8hi3 = 4041,
  CODE_FOR_xop_vrotlv4si3 = 4042,
  CODE_FOR_xop_vrotlv2di3 = 4043,
  CODE_FOR_xop_shav16qi3 = 4044,
  CODE_FOR_xop_shav8hi3 = 4045,
  CODE_FOR_xop_shav4si3 = 4046,
  CODE_FOR_xop_shav2di3 = 4047,
  CODE_FOR_xop_shlv16qi3 = 4048,
  CODE_FOR_xop_shlv8hi3 = 4049,
  CODE_FOR_xop_shlv4si3 = 4050,
  CODE_FOR_xop_shlv2di3 = 4051,
  CODE_FOR_xop_frczsf2 = 4052,
  CODE_FOR_xop_frczdf2 = 4053,
  CODE_FOR_xop_frczv4sf2 = 4054,
  CODE_FOR_xop_frczv2df2 = 4055,
  CODE_FOR_xop_frczv8sf2 = 4056,
  CODE_FOR_xop_frczv4df2 = 4057,
  CODE_FOR_xop_maskcmpv16qi3 = 4060,
  CODE_FOR_xop_maskcmpv8hi3 = 4061,
  CODE_FOR_xop_maskcmpv4si3 = 4062,
  CODE_FOR_xop_maskcmpv2di3 = 4063,
  CODE_FOR_xop_maskcmp_unsv16qi3 = 4064,
  CODE_FOR_xop_maskcmp_unsv8hi3 = 4065,
  CODE_FOR_xop_maskcmp_unsv4si3 = 4066,
  CODE_FOR_xop_maskcmp_unsv2di3 = 4067,
  CODE_FOR_xop_maskcmp_uns2v16qi3 = 4068,
  CODE_FOR_xop_maskcmp_uns2v8hi3 = 4069,
  CODE_FOR_xop_maskcmp_uns2v4si3 = 4070,
  CODE_FOR_xop_maskcmp_uns2v2di3 = 4071,
  CODE_FOR_xop_pcom_tfv16qi3 = 4072,
  CODE_FOR_xop_pcom_tfv8hi3 = 4073,
  CODE_FOR_xop_pcom_tfv4si3 = 4074,
  CODE_FOR_xop_pcom_tfv2di3 = 4075,
  CODE_FOR_xop_vpermil2v8sf3 = 4076,
  CODE_FOR_xop_vpermil2v4sf3 = 4077,
  CODE_FOR_xop_vpermil2v4df3 = 4078,
  CODE_FOR_xop_vpermil2v2df3 = 4079,
  CODE_FOR_aesenc = 4080,
  CODE_FOR_aesenclast = 4081,
  CODE_FOR_aesdec = 4082,
  CODE_FOR_aesdeclast = 4083,
  CODE_FOR_aesimc = 4084,
  CODE_FOR_aeskeygenassist = 4085,
  CODE_FOR_pclmulqdq = 4086,
  CODE_FOR_avx_vzeroupper = 4088,
  CODE_FOR_avx2_pbroadcastv16si = 4089,
  CODE_FOR_avx2_pbroadcastv8di = 4090,
  CODE_FOR_avx2_pbroadcastv64qi = 4091,
  CODE_FOR_avx2_pbroadcastv32qi = 4092,
  CODE_FOR_avx2_pbroadcastv16qi = 4093,
  CODE_FOR_avx2_pbroadcastv32hi = 4094,
  CODE_FOR_avx2_pbroadcastv16hi = 4095,
  CODE_FOR_avx2_pbroadcastv8hi = 4096,
  CODE_FOR_avx2_pbroadcastv8si = 4097,
  CODE_FOR_avx2_pbroadcastv4si = 4098,
  CODE_FOR_avx2_pbroadcastv4di = 4099,
  CODE_FOR_avx2_pbroadcastv2di = 4100,
  CODE_FOR_avx2_pbroadcastv32qi_1 = 4101,
  CODE_FOR_avx2_pbroadcastv16hi_1 = 4102,
  CODE_FOR_avx2_pbroadcastv8si_1 = 4103,
  CODE_FOR_avx2_pbroadcastv4di_1 = 4104,
  CODE_FOR_avx2_permvarv8si = 4105,
  CODE_FOR_avx2_permvarv8si_mask = 4106,
  CODE_FOR_avx2_permvarv8sf = 4107,
  CODE_FOR_avx2_permvarv8sf_mask = 4108,
  CODE_FOR_avx512f_permvarv16si = 4109,
  CODE_FOR_avx512f_permvarv16si_mask = 4110,
  CODE_FOR_avx512f_permvarv16sf = 4111,
  CODE_FOR_avx512f_permvarv16sf_mask = 4112,
  CODE_FOR_avx512f_permvarv8di = 4113,
  CODE_FOR_avx512f_permvarv8di_mask = 4114,
  CODE_FOR_avx512f_permvarv8df = 4115,
  CODE_FOR_avx512f_permvarv8df_mask = 4116,
  CODE_FOR_avx2_permvarv4di = 4117,
  CODE_FOR_avx2_permvarv4di_mask = 4118,
  CODE_FOR_avx2_permvarv4df = 4119,
  CODE_FOR_avx2_permvarv4df_mask = 4120,
  CODE_FOR_avx512bw_permvarv64qi = 4121,
  CODE_FOR_avx512bw_permvarv64qi_mask = 4122,
  CODE_FOR_avx512vl_permvarv16qi = 4123,
  CODE_FOR_avx512vl_permvarv16qi_mask = 4124,
  CODE_FOR_avx512vl_permvarv32qi = 4125,
  CODE_FOR_avx512vl_permvarv32qi_mask = 4126,
  CODE_FOR_avx512vl_permvarv8hi = 4127,
  CODE_FOR_avx512vl_permvarv8hi_mask = 4128,
  CODE_FOR_avx512vl_permvarv16hi = 4129,
  CODE_FOR_avx512vl_permvarv16hi_mask = 4130,
  CODE_FOR_avx512bw_permvarv32hi = 4131,
  CODE_FOR_avx512bw_permvarv32hi_mask = 4132,
  CODE_FOR_avx2_permv4di_1 = 4133,
  CODE_FOR_avx2_permv4di_1_mask = 4134,
  CODE_FOR_avx2_permv4df_1 = 4135,
  CODE_FOR_avx2_permv4df_1_mask = 4136,
  CODE_FOR_avx512f_permv8df_1 = 4137,
  CODE_FOR_avx512f_permv8df_1_mask = 4138,
  CODE_FOR_avx512f_permv8di_1 = 4139,
  CODE_FOR_avx512f_permv8di_1_mask = 4140,
  CODE_FOR_avx2_permv2ti = 4141,
  CODE_FOR_avx2_vec_dupv4df = 4142,
  CODE_FOR_avx512f_vec_dupv16si_1 = 4143,
  CODE_FOR_avx512f_vec_dupv8di_1 = 4144,
  CODE_FOR_avx512bw_vec_dupv32hi_1 = 4145,
  CODE_FOR_avx512bw_vec_dupv64qi_1 = 4146,
  CODE_FOR_avx512f_vec_dupv16si = 4147,
  CODE_FOR_avx512f_vec_dupv16si_mask = 4148,
  CODE_FOR_avx512vl_vec_dupv8si = 4149,
  CODE_FOR_avx512vl_vec_dupv8si_mask = 4150,
  CODE_FOR_avx512vl_vec_dupv4si = 4151,
  CODE_FOR_avx512vl_vec_dupv4si_mask = 4152,
  CODE_FOR_avx512f_vec_dupv8di = 4153,
  CODE_FOR_avx512f_vec_dupv8di_mask = 4154,
  CODE_FOR_avx512vl_vec_dupv4di = 4155,
  CODE_FOR_avx512vl_vec_dupv4di_mask = 4156,
  CODE_FOR_avx512vl_vec_dupv2di = 4157,
  CODE_FOR_avx512vl_vec_dupv2di_mask = 4158,
  CODE_FOR_avx512f_vec_dupv16sf = 4159,
  CODE_FOR_avx512f_vec_dupv16sf_mask = 4160,
  CODE_FOR_avx512vl_vec_dupv8sf = 4161,
  CODE_FOR_avx512vl_vec_dupv8sf_mask = 4162,
  CODE_FOR_avx512vl_vec_dupv4sf = 4163,
  CODE_FOR_avx512vl_vec_dupv4sf_mask = 4164,
  CODE_FOR_avx512f_vec_dupv8df = 4165,
  CODE_FOR_avx512f_vec_dupv8df_mask = 4166,
  CODE_FOR_avx512vl_vec_dupv4df = 4167,
  CODE_FOR_avx512vl_vec_dupv4df_mask = 4168,
  CODE_FOR_avx512vl_vec_dupv2df = 4169,
  CODE_FOR_avx512vl_vec_dupv2df_mask = 4170,
  CODE_FOR_avx512bw_vec_dupv64qi = 4171,
  CODE_FOR_avx512bw_vec_dupv64qi_mask = 4172,
  CODE_FOR_avx512vl_vec_dupv16qi = 4173,
  CODE_FOR_avx512vl_vec_dupv16qi_mask = 4174,
  CODE_FOR_avx512vl_vec_dupv32qi = 4175,
  CODE_FOR_avx512vl_vec_dupv32qi_mask = 4176,
  CODE_FOR_avx512bw_vec_dupv32hi = 4177,
  CODE_FOR_avx512bw_vec_dupv32hi_mask = 4178,
  CODE_FOR_avx512vl_vec_dupv16hi = 4179,
  CODE_FOR_avx512vl_vec_dupv16hi_mask = 4180,
  CODE_FOR_avx512vl_vec_dupv8hi = 4181,
  CODE_FOR_avx512vl_vec_dupv8hi_mask = 4182,
  CODE_FOR_avx512f_broadcastv16sf_mask = 4184,
  CODE_FOR_avx512f_broadcastv16si_mask = 4186,
  CODE_FOR_avx512f_broadcastv8df_mask = 4188,
  CODE_FOR_avx512f_broadcastv8di_mask = 4190,
  CODE_FOR_avx512bw_vec_dup_gprv64qi_mask = 4192,
  CODE_FOR_avx512vl_vec_dup_gprv16qi_mask = 4194,
  CODE_FOR_avx512vl_vec_dup_gprv32qi_mask = 4196,
  CODE_FOR_avx512bw_vec_dup_gprv32hi_mask = 4198,
  CODE_FOR_avx512vl_vec_dup_gprv16hi_mask = 4200,
  CODE_FOR_avx512vl_vec_dup_gprv8hi_mask = 4202,
  CODE_FOR_avx512f_vec_dup_gprv16si_mask = 4204,
  CODE_FOR_avx512vl_vec_dup_gprv8si_mask = 4206,
  CODE_FOR_avx512vl_vec_dup_gprv4si_mask = 4208,
  CODE_FOR_avx512f_vec_dup_gprv8di_mask = 4210,
  CODE_FOR_avx512vl_vec_dup_gprv4di_mask = 4212,
  CODE_FOR_avx512vl_vec_dup_gprv2di_mask = 4214,
  CODE_FOR_avx512f_vec_dup_gprv16sf_mask = 4216,
  CODE_FOR_avx512vl_vec_dup_gprv8sf_mask = 4218,
  CODE_FOR_avx512vl_vec_dup_gprv4sf_mask = 4220,
  CODE_FOR_avx512f_vec_dup_gprv8df_mask = 4222,
  CODE_FOR_avx512vl_vec_dup_gprv4df_mask = 4224,
  CODE_FOR_avx512vl_vec_dup_gprv2df_mask = 4226,
  CODE_FOR_vec_dupv4sf = 4227,
  CODE_FOR_avx2_vbroadcasti128_v32qi = 4230,
  CODE_FOR_avx2_vbroadcasti128_v16hi = 4231,
  CODE_FOR_avx2_vbroadcasti128_v8si = 4232,
  CODE_FOR_avx2_vbroadcasti128_v4di = 4233,
  CODE_FOR_vec_dupv8si = 4240,
  CODE_FOR_vec_dupv8sf = 4241,
  CODE_FOR_vec_dupv4di = 4242,
  CODE_FOR_vec_dupv4df = 4243,
  CODE_FOR_avx_vbroadcastf128_v32qi = 4244,
  CODE_FOR_avx_vbroadcastf128_v16hi = 4245,
  CODE_FOR_avx_vbroadcastf128_v8si = 4246,
  CODE_FOR_avx_vbroadcastf128_v4di = 4247,
  CODE_FOR_avx_vbroadcastf128_v8sf = 4248,
  CODE_FOR_avx_vbroadcastf128_v4df = 4249,
  CODE_FOR_avx512dq_broadcastv16si_mask = 4251,
  CODE_FOR_avx512dq_broadcastv8si_mask = 4253,
  CODE_FOR_avx512dq_broadcastv4si_mask = 4255,
  CODE_FOR_avx512dq_broadcastv16sf_mask = 4257,
  CODE_FOR_avx512dq_broadcastv8sf_mask = 4259,
  CODE_FOR_avx512vl_broadcastv8si_mask_1 = 4261,
  CODE_FOR_avx512vl_broadcastv8sf_mask_1 = 4263,
  CODE_FOR_avx512dq_broadcastv16sf_mask_1 = 4265,
  CODE_FOR_avx512dq_broadcastv16si_mask_1 = 4267,
  CODE_FOR_avx512dq_broadcastv8di_mask_1 = 4269,
  CODE_FOR_avx512dq_broadcastv8df_mask_1 = 4271,
  CODE_FOR_avx512dq_broadcastv4di_mask_1 = 4273,
  CODE_FOR_avx512dq_broadcastv4df_mask_1 = 4275,
  CODE_FOR_avx512cd_maskb_vec_dupv8di = 4276,
  CODE_FOR_avx512cd_maskb_vec_dupv4di = 4277,
  CODE_FOR_avx512cd_maskb_vec_dupv2di = 4278,
  CODE_FOR_avx512cd_maskw_vec_dupv16si = 4279,
  CODE_FOR_avx512cd_maskw_vec_dupv8si = 4280,
  CODE_FOR_avx512cd_maskw_vec_dupv4si = 4281,
  CODE_FOR_avx512f_vpermilvarv16sf3 = 4297,
  CODE_FOR_avx512f_vpermilvarv16sf3_mask = 4298,
  CODE_FOR_avx_vpermilvarv8sf3 = 4299,
  CODE_FOR_avx_vpermilvarv8sf3_mask = 4300,
  CODE_FOR_avx_vpermilvarv4sf3 = 4301,
  CODE_FOR_avx_vpermilvarv4sf3_mask = 4302,
  CODE_FOR_avx512f_vpermilvarv8df3 = 4303,
  CODE_FOR_avx512f_vpermilvarv8df3_mask = 4304,
  CODE_FOR_avx_vpermilvarv4df3 = 4305,
  CODE_FOR_avx_vpermilvarv4df3_mask = 4306,
  CODE_FOR_avx_vpermilvarv2df3 = 4307,
  CODE_FOR_avx_vpermilvarv2df3_mask = 4308,
  CODE_FOR_avx512f_vpermi2varv16si3 = 4309,
  CODE_FOR_avx512f_vpermi2varv16si3_maskz_1 = 4310,
  CODE_FOR_avx512f_vpermi2varv16sf3 = 4311,
  CODE_FOR_avx512f_vpermi2varv16sf3_maskz_1 = 4312,
  CODE_FOR_avx512f_vpermi2varv8di3 = 4313,
  CODE_FOR_avx512f_vpermi2varv8di3_maskz_1 = 4314,
  CODE_FOR_avx512f_vpermi2varv8df3 = 4315,
  CODE_FOR_avx512f_vpermi2varv8df3_maskz_1 = 4316,
  CODE_FOR_avx512vl_vpermi2varv8si3 = 4317,
  CODE_FOR_avx512vl_vpermi2varv8si3_maskz_1 = 4318,
  CODE_FOR_avx512vl_vpermi2varv8sf3 = 4319,
  CODE_FOR_avx512vl_vpermi2varv8sf3_maskz_1 = 4320,
  CODE_FOR_avx512vl_vpermi2varv4di3 = 4321,
  CODE_FOR_avx512vl_vpermi2varv4di3_maskz_1 = 4322,
  CODE_FOR_avx512vl_vpermi2varv4df3 = 4323,
  CODE_FOR_avx512vl_vpermi2varv4df3_maskz_1 = 4324,
  CODE_FOR_avx512vl_vpermi2varv4si3 = 4325,
  CODE_FOR_avx512vl_vpermi2varv4si3_maskz_1 = 4326,
  CODE_FOR_avx512vl_vpermi2varv4sf3 = 4327,
  CODE_FOR_avx512vl_vpermi2varv4sf3_maskz_1 = 4328,
  CODE_FOR_avx512vl_vpermi2varv2di3 = 4329,
  CODE_FOR_avx512vl_vpermi2varv2di3_maskz_1 = 4330,
  CODE_FOR_avx512vl_vpermi2varv2df3 = 4331,
  CODE_FOR_avx512vl_vpermi2varv2df3_maskz_1 = 4332,
  CODE_FOR_avx512bw_vpermi2varv64qi3 = 4333,
  CODE_FOR_avx512bw_vpermi2varv64qi3_maskz_1 = 4334,
  CODE_FOR_avx512vl_vpermi2varv16qi3 = 4335,
  CODE_FOR_avx512vl_vpermi2varv16qi3_maskz_1 = 4336,
  CODE_FOR_avx512vl_vpermi2varv32qi3 = 4337,
  CODE_FOR_avx512vl_vpermi2varv32qi3_maskz_1 = 4338,
  CODE_FOR_avx512vl_vpermi2varv8hi3 = 4339,
  CODE_FOR_avx512vl_vpermi2varv8hi3_maskz_1 = 4340,
  CODE_FOR_avx512vl_vpermi2varv16hi3 = 4341,
  CODE_FOR_avx512vl_vpermi2varv16hi3_maskz_1 = 4342,
  CODE_FOR_avx512bw_vpermi2varv32hi3 = 4343,
  CODE_FOR_avx512bw_vpermi2varv32hi3_maskz_1 = 4344,
  CODE_FOR_avx512f_vpermi2varv16si3_mask = 4345,
  CODE_FOR_avx512f_vpermi2varv16sf3_mask = 4346,
  CODE_FOR_avx512f_vpermi2varv8di3_mask = 4347,
  CODE_FOR_avx512f_vpermi2varv8df3_mask = 4348,
  CODE_FOR_avx512vl_vpermi2varv8si3_mask = 4349,
  CODE_FOR_avx512vl_vpermi2varv8sf3_mask = 4350,
  CODE_FOR_avx512vl_vpermi2varv4di3_mask = 4351,
  CODE_FOR_avx512vl_vpermi2varv4df3_mask = 4352,
  CODE_FOR_avx512vl_vpermi2varv4si3_mask = 4353,
  CODE_FOR_avx512vl_vpermi2varv4sf3_mask = 4354,
  CODE_FOR_avx512vl_vpermi2varv2di3_mask = 4355,
  CODE_FOR_avx512vl_vpermi2varv2df3_mask = 4356,
  CODE_FOR_avx512bw_vpermi2varv64qi3_mask = 4357,
  CODE_FOR_avx512vl_vpermi2varv16qi3_mask = 4358,
  CODE_FOR_avx512vl_vpermi2varv32qi3_mask = 4359,
  CODE_FOR_avx512vl_vpermi2varv8hi3_mask = 4360,
  CODE_FOR_avx512vl_vpermi2varv16hi3_mask = 4361,
  CODE_FOR_avx512bw_vpermi2varv32hi3_mask = 4362,
  CODE_FOR_avx512f_vpermt2varv16si3 = 4363,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz_1 = 4364,
  CODE_FOR_avx512f_vpermt2varv16sf3 = 4365,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz_1 = 4366,
  CODE_FOR_avx512f_vpermt2varv8di3 = 4367,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz_1 = 4368,
  CODE_FOR_avx512f_vpermt2varv8df3 = 4369,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz_1 = 4370,
  CODE_FOR_avx512vl_vpermt2varv8si3 = 4371,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz_1 = 4372,
  CODE_FOR_avx512vl_vpermt2varv8sf3 = 4373,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz_1 = 4374,
  CODE_FOR_avx512vl_vpermt2varv4di3 = 4375,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz_1 = 4376,
  CODE_FOR_avx512vl_vpermt2varv4df3 = 4377,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz_1 = 4378,
  CODE_FOR_avx512vl_vpermt2varv4si3 = 4379,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz_1 = 4380,
  CODE_FOR_avx512vl_vpermt2varv4sf3 = 4381,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz_1 = 4382,
  CODE_FOR_avx512vl_vpermt2varv2di3 = 4383,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz_1 = 4384,
  CODE_FOR_avx512vl_vpermt2varv2df3 = 4385,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz_1 = 4386,
  CODE_FOR_avx512bw_vpermt2varv64qi3 = 4387,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz_1 = 4388,
  CODE_FOR_avx512vl_vpermt2varv16qi3 = 4389,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz_1 = 4390,
  CODE_FOR_avx512vl_vpermt2varv32qi3 = 4391,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz_1 = 4392,
  CODE_FOR_avx512vl_vpermt2varv8hi3 = 4393,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz_1 = 4394,
  CODE_FOR_avx512vl_vpermt2varv16hi3 = 4395,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz_1 = 4396,
  CODE_FOR_avx512bw_vpermt2varv32hi3 = 4397,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz_1 = 4398,
  CODE_FOR_avx512f_vpermt2varv16si3_mask = 4399,
  CODE_FOR_avx512f_vpermt2varv16sf3_mask = 4400,
  CODE_FOR_avx512f_vpermt2varv8di3_mask = 4401,
  CODE_FOR_avx512f_vpermt2varv8df3_mask = 4402,
  CODE_FOR_avx512vl_vpermt2varv8si3_mask = 4403,
  CODE_FOR_avx512vl_vpermt2varv8sf3_mask = 4404,
  CODE_FOR_avx512vl_vpermt2varv4di3_mask = 4405,
  CODE_FOR_avx512vl_vpermt2varv4df3_mask = 4406,
  CODE_FOR_avx512vl_vpermt2varv4si3_mask = 4407,
  CODE_FOR_avx512vl_vpermt2varv4sf3_mask = 4408,
  CODE_FOR_avx512vl_vpermt2varv2di3_mask = 4409,
  CODE_FOR_avx512vl_vpermt2varv2df3_mask = 4410,
  CODE_FOR_avx512bw_vpermt2varv64qi3_mask = 4411,
  CODE_FOR_avx512vl_vpermt2varv16qi3_mask = 4412,
  CODE_FOR_avx512vl_vpermt2varv32qi3_mask = 4413,
  CODE_FOR_avx512vl_vpermt2varv8hi3_mask = 4414,
  CODE_FOR_avx512vl_vpermt2varv16hi3_mask = 4415,
  CODE_FOR_avx512bw_vpermt2varv32hi3_mask = 4416,
  CODE_FOR_vec_set_lo_v4di = 4429,
  CODE_FOR_vec_set_lo_v4di_mask = 4430,
  CODE_FOR_vec_set_lo_v4df = 4431,
  CODE_FOR_vec_set_lo_v4df_mask = 4432,
  CODE_FOR_vec_set_hi_v4di = 4433,
  CODE_FOR_vec_set_hi_v4di_mask = 4434,
  CODE_FOR_vec_set_hi_v4df = 4435,
  CODE_FOR_vec_set_hi_v4df_mask = 4436,
  CODE_FOR_vec_set_lo_v8si = 4437,
  CODE_FOR_vec_set_lo_v8si_mask = 4438,
  CODE_FOR_vec_set_lo_v8sf = 4439,
  CODE_FOR_vec_set_lo_v8sf_mask = 4440,
  CODE_FOR_vec_set_hi_v8si = 4441,
  CODE_FOR_vec_set_hi_v8si_mask = 4442,
  CODE_FOR_vec_set_hi_v8sf = 4443,
  CODE_FOR_vec_set_hi_v8sf_mask = 4444,
  CODE_FOR_vec_set_lo_v16hi = 4445,
  CODE_FOR_vec_set_hi_v16hi = 4446,
  CODE_FOR_vec_set_lo_v32qi = 4447,
  CODE_FOR_vec_set_hi_v32qi = 4448,
  CODE_FOR_avx_maskloadps = 4449,
  CODE_FOR_avx_maskloadpd = 4450,
  CODE_FOR_avx_maskloadps256 = 4451,
  CODE_FOR_avx_maskloadpd256 = 4452,
  CODE_FOR_avx2_maskloadd = 4453,
  CODE_FOR_avx2_maskloadq = 4454,
  CODE_FOR_avx2_maskloadd256 = 4455,
  CODE_FOR_avx2_maskloadq256 = 4456,
  CODE_FOR_avx_maskstoreps = 4457,
  CODE_FOR_avx_maskstorepd = 4458,
  CODE_FOR_avx_maskstoreps256 = 4459,
  CODE_FOR_avx_maskstorepd256 = 4460,
  CODE_FOR_avx2_maskstored = 4461,
  CODE_FOR_avx2_maskstoreq = 4462,
  CODE_FOR_avx2_maskstored256 = 4463,
  CODE_FOR_avx2_maskstoreq256 = 4464,
  CODE_FOR_avx_si256_si = 4465,
  CODE_FOR_avx_ps256_ps = 4466,
  CODE_FOR_avx_pd256_pd = 4467,
  CODE_FOR_avx2_ashrvv4si = 4468,
  CODE_FOR_avx2_ashrvv4si_mask = 4469,
  CODE_FOR_avx2_ashrvv8si = 4470,
  CODE_FOR_avx2_ashrvv8si_mask = 4471,
  CODE_FOR_avx512f_ashrvv16si = 4472,
  CODE_FOR_avx512f_ashrvv16si_mask = 4473,
  CODE_FOR_avx2_ashrvv2di = 4474,
  CODE_FOR_avx2_ashrvv2di_mask = 4475,
  CODE_FOR_avx2_ashrvv4di = 4476,
  CODE_FOR_avx2_ashrvv4di_mask = 4477,
  CODE_FOR_avx512f_ashrvv8di = 4478,
  CODE_FOR_avx512f_ashrvv8di_mask = 4479,
  CODE_FOR_avx512vl_ashrvv8hi = 4480,
  CODE_FOR_avx512vl_ashrvv8hi_mask = 4481,
  CODE_FOR_avx512vl_ashrvv16hi = 4482,
  CODE_FOR_avx512vl_ashrvv16hi_mask = 4483,
  CODE_FOR_avx512bw_ashrvv32hi = 4484,
  CODE_FOR_avx512bw_ashrvv32hi_mask = 4485,
  CODE_FOR_avx512f_ashlvv16si = 4486,
  CODE_FOR_avx512f_ashlvv16si_mask = 4487,
  CODE_FOR_avx512f_lshrvv16si = 4488,
  CODE_FOR_avx512f_lshrvv16si_mask = 4489,
  CODE_FOR_avx2_ashlvv8si = 4490,
  CODE_FOR_avx2_ashlvv8si_mask = 4491,
  CODE_FOR_avx2_lshrvv8si = 4492,
  CODE_FOR_avx2_lshrvv8si_mask = 4493,
  CODE_FOR_avx2_ashlvv4si = 4494,
  CODE_FOR_avx2_ashlvv4si_mask = 4495,
  CODE_FOR_avx2_lshrvv4si = 4496,
  CODE_FOR_avx2_lshrvv4si_mask = 4497,
  CODE_FOR_avx512f_ashlvv8di = 4498,
  CODE_FOR_avx512f_ashlvv8di_mask = 4499,
  CODE_FOR_avx512f_lshrvv8di = 4500,
  CODE_FOR_avx512f_lshrvv8di_mask = 4501,
  CODE_FOR_avx2_ashlvv4di = 4502,
  CODE_FOR_avx2_ashlvv4di_mask = 4503,
  CODE_FOR_avx2_lshrvv4di = 4504,
  CODE_FOR_avx2_lshrvv4di_mask = 4505,
  CODE_FOR_avx2_ashlvv2di = 4506,
  CODE_FOR_avx2_ashlvv2di_mask = 4507,
  CODE_FOR_avx2_lshrvv2di = 4508,
  CODE_FOR_avx2_lshrvv2di_mask = 4509,
  CODE_FOR_avx512vl_ashlvv8hi = 4510,
  CODE_FOR_avx512vl_ashlvv8hi_mask = 4511,
  CODE_FOR_avx512vl_lshrvv8hi = 4512,
  CODE_FOR_avx512vl_lshrvv8hi_mask = 4513,
  CODE_FOR_avx512vl_ashlvv16hi = 4514,
  CODE_FOR_avx512vl_ashlvv16hi_mask = 4515,
  CODE_FOR_avx512vl_lshrvv16hi = 4516,
  CODE_FOR_avx512vl_lshrvv16hi_mask = 4517,
  CODE_FOR_avx512bw_ashlvv32hi = 4518,
  CODE_FOR_avx512bw_ashlvv32hi_mask = 4519,
  CODE_FOR_avx512bw_lshrvv32hi = 4520,
  CODE_FOR_avx512bw_lshrvv32hi_mask = 4521,
  CODE_FOR_avx_vec_concatv32qi = 4522,
  CODE_FOR_avx_vec_concatv16hi = 4523,
  CODE_FOR_avx_vec_concatv8si = 4524,
  CODE_FOR_avx_vec_concatv4di = 4525,
  CODE_FOR_avx_vec_concatv8sf = 4526,
  CODE_FOR_avx_vec_concatv4df = 4527,
  CODE_FOR_avx_vec_concatv64qi = 4528,
  CODE_FOR_avx_vec_concatv32hi = 4529,
  CODE_FOR_avx_vec_concatv16si = 4530,
  CODE_FOR_avx_vec_concatv8di = 4531,
  CODE_FOR_avx_vec_concatv16sf = 4532,
  CODE_FOR_avx_vec_concatv8df = 4533,
  CODE_FOR_vcvtph2ps = 4534,
  CODE_FOR_vcvtph2ps_mask = 4535,
  CODE_FOR_vcvtph2ps256 = 4538,
  CODE_FOR_vcvtph2ps256_mask = 4539,
  CODE_FOR_avx512f_vcvtph2ps512_mask = 4542,
  CODE_FOR_avx512f_vcvtph2ps512_mask_round = 4543,
  CODE_FOR_vcvtps2ph256 = 4547,
  CODE_FOR_vcvtps2ph256_mask = 4548,
  CODE_FOR_avx512f_vcvtps2ph512_mask = 4550,
  CODE_FOR_avx512f_compressv16si_mask = 4767,
  CODE_FOR_avx512f_compressv16sf_mask = 4768,
  CODE_FOR_avx512f_compressv8di_mask = 4769,
  CODE_FOR_avx512f_compressv8df_mask = 4770,
  CODE_FOR_avx512vl_compressv8si_mask = 4771,
  CODE_FOR_avx512vl_compressv8sf_mask = 4772,
  CODE_FOR_avx512vl_compressv4di_mask = 4773,
  CODE_FOR_avx512vl_compressv4df_mask = 4774,
  CODE_FOR_avx512vl_compressv4si_mask = 4775,
  CODE_FOR_avx512vl_compressv4sf_mask = 4776,
  CODE_FOR_avx512vl_compressv2di_mask = 4777,
  CODE_FOR_avx512vl_compressv2df_mask = 4778,
  CODE_FOR_avx512f_compressstorev16si_mask = 4779,
  CODE_FOR_avx512f_compressstorev16sf_mask = 4780,
  CODE_FOR_avx512f_compressstorev8di_mask = 4781,
  CODE_FOR_avx512f_compressstorev8df_mask = 4782,
  CODE_FOR_avx512vl_compressstorev8si_mask = 4783,
  CODE_FOR_avx512vl_compressstorev8sf_mask = 4784,
  CODE_FOR_avx512vl_compressstorev4di_mask = 4785,
  CODE_FOR_avx512vl_compressstorev4df_mask = 4786,
  CODE_FOR_avx512vl_compressstorev4si_mask = 4787,
  CODE_FOR_avx512vl_compressstorev4sf_mask = 4788,
  CODE_FOR_avx512vl_compressstorev2di_mask = 4789,
  CODE_FOR_avx512vl_compressstorev2df_mask = 4790,
  CODE_FOR_avx512f_expandv16si_mask = 4791,
  CODE_FOR_avx512f_expandv16sf_mask = 4792,
  CODE_FOR_avx512f_expandv8di_mask = 4793,
  CODE_FOR_avx512f_expandv8df_mask = 4794,
  CODE_FOR_avx512vl_expandv8si_mask = 4795,
  CODE_FOR_avx512vl_expandv8sf_mask = 4796,
  CODE_FOR_avx512vl_expandv4di_mask = 4797,
  CODE_FOR_avx512vl_expandv4df_mask = 4798,
  CODE_FOR_avx512vl_expandv4si_mask = 4799,
  CODE_FOR_avx512vl_expandv4sf_mask = 4800,
  CODE_FOR_avx512vl_expandv2di_mask = 4801,
  CODE_FOR_avx512vl_expandv2df_mask = 4802,
  CODE_FOR_avx512dq_rangepv16sf = 4803,
  CODE_FOR_avx512dq_rangepv16sf_round = 4804,
  CODE_FOR_avx512dq_rangepv16sf_mask = 4805,
  CODE_FOR_avx512dq_rangepv16sf_mask_round = 4806,
  CODE_FOR_avx512dq_rangepv8sf = 4807,
   CODE_FOR_avx512dq_rangepv8sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8sf_mask = 4808,
   CODE_FOR_avx512dq_rangepv8sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf = 4809,
   CODE_FOR_avx512dq_rangepv4sf_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4sf_mask = 4810,
   CODE_FOR_avx512dq_rangepv4sf_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv8df = 4811,
  CODE_FOR_avx512dq_rangepv8df_round = 4812,
  CODE_FOR_avx512dq_rangepv8df_mask = 4813,
  CODE_FOR_avx512dq_rangepv8df_mask_round = 4814,
  CODE_FOR_avx512dq_rangepv4df = 4815,
   CODE_FOR_avx512dq_rangepv4df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv4df_mask = 4816,
   CODE_FOR_avx512dq_rangepv4df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df = 4817,
   CODE_FOR_avx512dq_rangepv2df_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangepv2df_mask = 4818,
   CODE_FOR_avx512dq_rangepv2df_mask_round = CODE_FOR_nothing,
  CODE_FOR_avx512dq_rangesv4sf = 4819,
  CODE_FOR_avx512dq_rangesv4sf_round = 4820,
  CODE_FOR_avx512dq_rangesv2df = 4821,
  CODE_FOR_avx512dq_rangesv2df_round = 4822,
  CODE_FOR_avx512dq_fpclassv16sf = 4823,
  CODE_FOR_avx512dq_fpclassv16sf_mask = 4824,
  CODE_FOR_avx512dq_fpclassv8sf = 4825,
  CODE_FOR_avx512dq_fpclassv8sf_mask = 4826,
  CODE_FOR_avx512dq_fpclassv4sf = 4827,
  CODE_FOR_avx512dq_fpclassv4sf_mask = 4828,
  CODE_FOR_avx512dq_fpclassv8df = 4829,
  CODE_FOR_avx512dq_fpclassv8df_mask = 4830,
  CODE_FOR_avx512dq_fpclassv4df = 4831,
  CODE_FOR_avx512dq_fpclassv4df_mask = 4832,
  CODE_FOR_avx512dq_fpclassv2df = 4833,
  CODE_FOR_avx512dq_fpclassv2df_mask = 4834,
  CODE_FOR_avx512dq_vmfpclassv4sf = 4835,
  CODE_FOR_avx512dq_vmfpclassv2df = 4836,
  CODE_FOR_avx512f_getmantv16sf = 4837,
  CODE_FOR_avx512f_getmantv16sf_round = 4838,
  CODE_FOR_avx512f_getmantv16sf_mask = 4839,
  CODE_FOR_avx512f_getmantv16sf_mask_round = 4840,
  CODE_FOR_avx512vl_getmantv8sf = 4841,
  CODE_FOR_avx512vl_getmantv8sf_round = 4842,
  CODE_FOR_avx512vl_getmantv8sf_mask = 4843,
  CODE_FOR_avx512vl_getmantv8sf_mask_round = 4844,
  CODE_FOR_avx512vl_getmantv4sf = 4845,
  CODE_FOR_avx512vl_getmantv4sf_round = 4846,
  CODE_FOR_avx512vl_getmantv4sf_mask = 4847,
  CODE_FOR_avx512vl_getmantv4sf_mask_round = 4848,
  CODE_FOR_avx512f_getmantv8df = 4849,
  CODE_FOR_avx512f_getmantv8df_round = 4850,
  CODE_FOR_avx512f_getmantv8df_mask = 4851,
  CODE_FOR_avx512f_getmantv8df_mask_round = 4852,
  CODE_FOR_avx512vl_getmantv4df = 4853,
  CODE_FOR_avx512vl_getmantv4df_round = 4854,
  CODE_FOR_avx512vl_getmantv4df_mask = 4855,
  CODE_FOR_avx512vl_getmantv4df_mask_round = 4856,
  CODE_FOR_avx512vl_getmantv2df = 4857,
  CODE_FOR_avx512vl_getmantv2df_round = 4858,
  CODE_FOR_avx512vl_getmantv2df_mask = 4859,
  CODE_FOR_avx512vl_getmantv2df_mask_round = 4860,
  CODE_FOR_avx512f_vgetmantv4sf = 4861,
  CODE_FOR_avx512f_vgetmantv4sf_round = 4862,
  CODE_FOR_avx512f_vgetmantv2df = 4863,
  CODE_FOR_avx512f_vgetmantv2df_round = 4864,
  CODE_FOR_avx512bw_dbpsadbwv8hi_mask = 4866,
  CODE_FOR_avx512bw_dbpsadbwv16hi_mask = 4868,
  CODE_FOR_avx512bw_dbpsadbwv32hi_mask = 4870,
  CODE_FOR_clzv16si2 = 4871,
  CODE_FOR_clzv16si2_mask = 4872,
  CODE_FOR_clzv8si2 = 4873,
  CODE_FOR_clzv8si2_mask = 4874,
  CODE_FOR_clzv4si2 = 4875,
  CODE_FOR_clzv4si2_mask = 4876,
  CODE_FOR_clzv8di2 = 4877,
  CODE_FOR_clzv8di2_mask = 4878,
  CODE_FOR_clzv4di2 = 4879,
  CODE_FOR_clzv4di2_mask = 4880,
  CODE_FOR_clzv2di2 = 4881,
  CODE_FOR_clzv2di2_mask = 4882,
  CODE_FOR_conflictv16si_mask = 4884,
  CODE_FOR_conflictv8si_mask = 4886,
  CODE_FOR_conflictv4si_mask = 4888,
  CODE_FOR_conflictv8di_mask = 4890,
  CODE_FOR_conflictv4di_mask = 4892,
  CODE_FOR_conflictv2di_mask = 4894,
  CODE_FOR_sha1msg1 = 4895,
  CODE_FOR_sha1msg2 = 4896,
  CODE_FOR_sha1nexte = 4897,
  CODE_FOR_sha1rnds4 = 4898,
  CODE_FOR_sha256msg1 = 4899,
  CODE_FOR_sha256msg2 = 4900,
  CODE_FOR_sha256rnds2 = 4901,
  CODE_FOR_avx512f_si512_si = 4902,
  CODE_FOR_avx512f_ps512_ps = 4903,
  CODE_FOR_avx512f_pd512_pd = 4904,
  CODE_FOR_avx512f_si512_256si = 4905,
  CODE_FOR_avx512f_ps512_256ps = 4906,
  CODE_FOR_avx512f_pd512_256pd = 4907,
  CODE_FOR_vpamdd52luqv8di = 4908,
  CODE_FOR_vpamdd52luqv8di_maskz_1 = 4909,
  CODE_FOR_vpamdd52huqv8di = 4910,
  CODE_FOR_vpamdd52huqv8di_maskz_1 = 4911,
  CODE_FOR_vpamdd52luqv4di = 4912,
  CODE_FOR_vpamdd52luqv4di_maskz_1 = 4913,
  CODE_FOR_vpamdd52huqv4di = 4914,
  CODE_FOR_vpamdd52huqv4di_maskz_1 = 4915,
  CODE_FOR_vpamdd52luqv2di = 4916,
  CODE_FOR_vpamdd52luqv2di_maskz_1 = 4917,
  CODE_FOR_vpamdd52huqv2di = 4918,
  CODE_FOR_vpamdd52huqv2di_maskz_1 = 4919,
  CODE_FOR_vpamdd52luqv8di_mask = 4920,
  CODE_FOR_vpamdd52huqv8di_mask = 4921,
  CODE_FOR_vpamdd52luqv4di_mask = 4922,
  CODE_FOR_vpamdd52huqv4di_mask = 4923,
  CODE_FOR_vpamdd52luqv2di_mask = 4924,
  CODE_FOR_vpamdd52huqv2di_mask = 4925,
  CODE_FOR_vpmultishiftqbv64qi = 4926,
  CODE_FOR_vpmultishiftqbv64qi_mask = 4927,
  CODE_FOR_vpmultishiftqbv16qi = 4928,
  CODE_FOR_vpmultishiftqbv16qi_mask = 4929,
  CODE_FOR_vpmultishiftqbv32qi = 4930,
  CODE_FOR_vpmultishiftqbv32qi_mask = 4931,
  CODE_FOR_avx5124fmaddps_4fmaddps = 4934,
  CODE_FOR_avx5124fmaddps_4fmaddps_mask = 4935,
  CODE_FOR_avx5124fmaddps_4fmaddps_maskz = 4936,
  CODE_FOR_avx5124fmaddps_4fmaddss = 4937,
  CODE_FOR_avx5124fmaddps_4fmaddss_mask = 4938,
  CODE_FOR_avx5124fmaddps_4fmaddss_maskz = 4939,
  CODE_FOR_avx5124fmaddps_4fnmaddps = 4940,
  CODE_FOR_avx5124fmaddps_4fnmaddps_mask = 4941,
  CODE_FOR_avx5124fmaddps_4fnmaddps_maskz = 4942,
  CODE_FOR_avx5124fmaddps_4fnmaddss = 4943,
  CODE_FOR_avx5124fmaddps_4fnmaddss_mask = 4944,
  CODE_FOR_avx5124fmaddps_4fnmaddss_maskz = 4945,
  CODE_FOR_avx5124vnniw_vp4dpwssd = 4946,
  CODE_FOR_avx5124vnniw_vp4dpwssd_mask = 4947,
  CODE_FOR_avx5124vnniw_vp4dpwssd_maskz = 4948,
  CODE_FOR_avx5124vnniw_vp4dpwssds = 4949,
  CODE_FOR_avx5124vnniw_vp4dpwssds_mask = 4950,
  CODE_FOR_avx5124vnniw_vp4dpwssds_maskz = 4951,
  CODE_FOR_vpopcountv16si = 4952,
  CODE_FOR_vpopcountv16si_mask = 4953,
  CODE_FOR_vpopcountv8di = 4954,
  CODE_FOR_vpopcountv8di_mask = 4955,
  CODE_FOR_mfence_sse2 = 4958,
  CODE_FOR_mfence_nosse = 4959,
  CODE_FOR_atomic_loaddi_fpu = 4960,
  CODE_FOR_atomic_storeqi_1 = 4961,
  CODE_FOR_atomic_storehi_1 = 4962,
  CODE_FOR_atomic_storesi_1 = 4963,
  CODE_FOR_atomic_storedi_1 = 4964,
  CODE_FOR_atomic_storedi_fpu = 4965,
  CODE_FOR_loaddi_via_fpu = 4966,
  CODE_FOR_storedi_via_fpu = 4967,
  CODE_FOR_loaddi_via_sse = 4968,
  CODE_FOR_storedi_via_sse = 4969,
  CODE_FOR_atomic_compare_and_swapdi_doubleword = 4970,
  CODE_FOR_atomic_compare_and_swapti_doubleword = 4971,
  CODE_FOR_atomic_compare_and_swapqi_1 = 4972,
  CODE_FOR_atomic_compare_and_swaphi_1 = 4973,
  CODE_FOR_atomic_compare_and_swapsi_1 = 4974,
  CODE_FOR_atomic_compare_and_swapdi_1 = 4975,
  CODE_FOR_atomic_fetch_addqi = 4976,
  CODE_FOR_atomic_fetch_addhi = 4977,
  CODE_FOR_atomic_fetch_addsi = 4978,
  CODE_FOR_atomic_fetch_adddi = 4979,
  CODE_FOR_atomic_exchangeqi = 4984,
  CODE_FOR_atomic_exchangehi = 4985,
  CODE_FOR_atomic_exchangesi = 4986,
  CODE_FOR_atomic_exchangedi = 4987,
  CODE_FOR_atomic_addqi = 4988,
  CODE_FOR_atomic_addhi = 4989,
  CODE_FOR_atomic_addsi = 4990,
  CODE_FOR_atomic_adddi = 4991,
  CODE_FOR_atomic_subqi = 4992,
  CODE_FOR_atomic_subhi = 4993,
  CODE_FOR_atomic_subsi = 4994,
  CODE_FOR_atomic_subdi = 4995,
  CODE_FOR_atomic_andqi = 4996,
  CODE_FOR_atomic_orqi = 4997,
  CODE_FOR_atomic_xorqi = 4998,
  CODE_FOR_atomic_andhi = 4999,
  CODE_FOR_atomic_orhi = 5000,
  CODE_FOR_atomic_xorhi = 5001,
  CODE_FOR_atomic_andsi = 5002,
  CODE_FOR_atomic_orsi = 5003,
  CODE_FOR_atomic_xorsi = 5004,
  CODE_FOR_atomic_anddi = 5005,
  CODE_FOR_atomic_ordi = 5006,
  CODE_FOR_atomic_xordi = 5007,
  CODE_FOR_atomic_bit_test_and_sethi_1 = 5008,
  CODE_FOR_atomic_bit_test_and_setsi_1 = 5009,
  CODE_FOR_atomic_bit_test_and_setdi_1 = 5010,
  CODE_FOR_atomic_bit_test_and_complementhi_1 = 5011,
  CODE_FOR_atomic_bit_test_and_complementsi_1 = 5012,
  CODE_FOR_atomic_bit_test_and_complementdi_1 = 5013,
  CODE_FOR_atomic_bit_test_and_resethi_1 = 5014,
  CODE_FOR_atomic_bit_test_and_resetsi_1 = 5015,
  CODE_FOR_atomic_bit_test_and_resetdi_1 = 5016,
  CODE_FOR_cbranchqi4 = 5017,
  CODE_FOR_cbranchhi4 = 5018,
  CODE_FOR_cbranchsi4 = 5019,
  CODE_FOR_cbranchdi4 = 5020,
  CODE_FOR_cbranchti4 = 5021,
  CODE_FOR_cstoreqi4 = 5022,
  CODE_FOR_cstorehi4 = 5023,
  CODE_FOR_cstoresi4 = 5024,
  CODE_FOR_cstoredi4 = 5025,
  CODE_FOR_cmpsi_1 = 5026,
  CODE_FOR_cmpdi_1 = 5027,
  CODE_FOR_cmpqi_ext_3 = 5028,
  CODE_FOR_cbranchxf4 = 5029,
  CODE_FOR_cstorexf4 = 5030,
  CODE_FOR_cbranchsf4 = 5031,
  CODE_FOR_cbranchdf4 = 5032,
  CODE_FOR_cstoresf4 = 5033,
  CODE_FOR_cstoredf4 = 5034,
  CODE_FOR_cbranchcc4 = 5035,
  CODE_FOR_cstorecc4 = 5036,
  CODE_FOR_reload_noff_store = 5037,
  CODE_FOR_reload_noff_load = 5038,
  CODE_FOR_movxi = 5039,
  CODE_FOR_movoi = 5040,
  CODE_FOR_movti = 5041,
  CODE_FOR_movcdi = 5042,
  CODE_FOR_movqi = 5043,
  CODE_FOR_movhi = 5044,
  CODE_FOR_movsi = 5045,
  CODE_FOR_movdi = 5046,
  CODE_FOR_movstrictqi = 5047,
  CODE_FOR_movstricthi = 5048,
  CODE_FOR_extvhi = 5049,
  CODE_FOR_extvsi = 5050,
  CODE_FOR_extzvhi = 5051,
  CODE_FOR_extzvsi = 5052,
  CODE_FOR_extzvdi = 5053,
  CODE_FOR_insvhi = 5054,
  CODE_FOR_insvsi = 5055,
  CODE_FOR_insvdi = 5056,
  CODE_FOR_movtf = 5057,
  CODE_FOR_movsf = 5058,
  CODE_FOR_movdf = 5059,
  CODE_FOR_movxf = 5060,
  CODE_FOR_zero_extendsidi2 = 5061,
  CODE_FOR_zero_extendqisi2 = 5062,
  CODE_FOR_zero_extendhisi2 = 5063,
  CODE_FOR_zero_extendqihi2 = 5064,
  CODE_FOR_extendsidi2 = 5065,
  CODE_FOR_extendsfdf2 = 5066,
  CODE_FOR_extendsfxf2 = 5067,
  CODE_FOR_extenddfxf2 = 5068,
  CODE_FOR_truncdfsf2 = 5069,
  CODE_FOR_truncdfsf2_with_temp = 5070,
  CODE_FOR_truncxfsf2 = 5071,
  CODE_FOR_truncxfdf2 = 5072,
  CODE_FOR_fix_truncxfdi2 = 5073,
  CODE_FOR_fix_truncsfdi2 = 5074,
  CODE_FOR_fix_truncdfdi2 = 5075,
  CODE_FOR_fix_truncxfsi2 = 5076,
  CODE_FOR_fix_truncsfsi2 = 5077,
  CODE_FOR_fix_truncdfsi2 = 5078,
  CODE_FOR_fix_truncsfhi2 = 5079,
  CODE_FOR_fix_truncdfhi2 = 5080,
  CODE_FOR_fix_truncxfhi2 = 5081,
  CODE_FOR_fixuns_truncsfsi2 = 5082,
  CODE_FOR_fixuns_truncdfsi2 = 5083,
  CODE_FOR_fixuns_truncsfhi2 = 5084,
  CODE_FOR_fixuns_truncdfhi2 = 5085,
  CODE_FOR_floatsisf2 = 5086,
  CODE_FOR_floatdisf2 = 5087,
  CODE_FOR_floatsidf2 = 5088,
  CODE_FOR_floatdidf2 = 5089,
  CODE_FOR_floatunsqisf2 = 5090,
  CODE_FOR_floatunshisf2 = 5091,
  CODE_FOR_floatunsqidf2 = 5092,
  CODE_FOR_floatunshidf2 = 5093,
  CODE_FOR_floatunssisf2 = 5094,
  CODE_FOR_floatunssidf2 = 5095,
  CODE_FOR_floatunssixf2 = 5096,
  CODE_FOR_floatunsdisf2 = 5097,
  CODE_FOR_floatunsdidf2 = 5098,
  CODE_FOR_addqi3 = 5099,
  CODE_FOR_addhi3 = 5100,
  CODE_FOR_addsi3 = 5101,
  CODE_FOR_adddi3 = 5102,
  CODE_FOR_addti3 = 5103,
  CODE_FOR_addvqi4 = 5104,
  CODE_FOR_addvhi4 = 5105,
  CODE_FOR_addvsi4 = 5106,
  CODE_FOR_addvdi4 = 5107,
  CODE_FOR_uaddvqi4 = 5108,
  CODE_FOR_uaddvhi4 = 5109,
  CODE_FOR_uaddvsi4 = 5110,
  CODE_FOR_uaddvdi4 = 5111,
  CODE_FOR_subqi3 = 5112,
  CODE_FOR_subhi3 = 5113,
  CODE_FOR_subsi3 = 5114,
  CODE_FOR_subdi3 = 5115,
  CODE_FOR_subti3 = 5116,
  CODE_FOR_subvqi4 = 5117,
  CODE_FOR_subvhi4 = 5118,
  CODE_FOR_subvsi4 = 5119,
  CODE_FOR_subvdi4 = 5120,
  CODE_FOR_usubvqi4 = 5121,
  CODE_FOR_usubvhi4 = 5122,
  CODE_FOR_usubvsi4 = 5123,
  CODE_FOR_usubvdi4 = 5124,
  CODE_FOR_addqi3_cconly_overflow = 5125,
  CODE_FOR_addxf3 = 5126,
  CODE_FOR_subxf3 = 5127,
  CODE_FOR_addsf3 = 5128,
  CODE_FOR_subsf3 = 5129,
  CODE_FOR_adddf3 = 5130,
  CODE_FOR_subdf3 = 5131,
  CODE_FOR_mulhi3 = 5132,
  CODE_FOR_mulsi3 = 5133,
  CODE_FOR_muldi3 = 5134,
  CODE_FOR_mulqi3 = 5135,
  CODE_FOR_mulvhi4 = 5136,
  CODE_FOR_mulvsi4 = 5137,
  CODE_FOR_mulvdi4 = 5138,
  CODE_FOR_umulvhi4 = 5139,
  CODE_FOR_umulvsi4 = 5140,
  CODE_FOR_umulvdi4 = 5141,
  CODE_FOR_mulvqi4 = 5142,
  CODE_FOR_umulvqi4 = 5143,
  CODE_FOR_mulsidi3 = 5144,
  CODE_FOR_umulsidi3 = 5145,
  CODE_FOR_mulditi3 = 5146,
  CODE_FOR_umulditi3 = 5147,
  CODE_FOR_mulqihi3 = 5148,
  CODE_FOR_umulqihi3 = 5149,
  CODE_FOR_smulsi3_highpart = 5150,
  CODE_FOR_umulsi3_highpart = 5151,
  CODE_FOR_smuldi3_highpart = 5152,
  CODE_FOR_umuldi3_highpart = 5153,
  CODE_FOR_mulxf3 = 5154,
  CODE_FOR_mulsf3 = 5155,
  CODE_FOR_muldf3 = 5156,
  CODE_FOR_divxf3 = 5157,
  CODE_FOR_divsf3 = 5158,
  CODE_FOR_divdf3 = 5159,
  CODE_FOR_divmodhi4 = 5160,
  CODE_FOR_divmodsi4 = 5161,
  CODE_FOR_divmoddi4 = 5162,
  CODE_FOR_divmodqi4 = 5163,
  CODE_FOR_udivmodhi4 = 5164,
  CODE_FOR_udivmodsi4 = 5165,
  CODE_FOR_udivmoddi4 = 5166,
  CODE_FOR_udivmodqi4 = 5167,
  CODE_FOR_testsi_ccno_1 = 5168,
  CODE_FOR_testqi_ccz_1 = 5169,
  CODE_FOR_testdi_ccno_1 = 5170,
  CODE_FOR_testqi_ext_1_ccno = 5171,
  CODE_FOR_andqi3 = 5172,
  CODE_FOR_andhi3 = 5173,
  CODE_FOR_andsi3 = 5174,
  CODE_FOR_anddi3 = 5175,
  CODE_FOR_iorqi3 = 5176,
  CODE_FOR_xorqi3 = 5177,
  CODE_FOR_iorhi3 = 5178,
  CODE_FOR_xorhi3 = 5179,
  CODE_FOR_iorsi3 = 5180,
  CODE_FOR_xorsi3 = 5181,
  CODE_FOR_iordi3 = 5182,
  CODE_FOR_xordi3 = 5183,
  CODE_FOR_xorqi_ext_1_cc = 5184,
  CODE_FOR_negqi2 = 5185,
  CODE_FOR_neghi2 = 5186,
  CODE_FOR_negsi2 = 5187,
  CODE_FOR_negdi2 = 5188,
  CODE_FOR_negti2 = 5189,
  CODE_FOR_negvqi3 = 5190,
  CODE_FOR_negvhi3 = 5191,
  CODE_FOR_negvsi3 = 5192,
  CODE_FOR_negvdi3 = 5193,
  CODE_FOR_abssf2 = 5194,
  CODE_FOR_negsf2 = 5195,
  CODE_FOR_absdf2 = 5196,
  CODE_FOR_negdf2 = 5197,
  CODE_FOR_absxf2 = 5198,
  CODE_FOR_negxf2 = 5199,
  CODE_FOR_abstf2 = 5200,
  CODE_FOR_negtf2 = 5201,
  CODE_FOR_copysignsf3 = 5202,
  CODE_FOR_copysigndf3 = 5203,
  CODE_FOR_copysigntf3 = 5204,
  CODE_FOR_one_cmplqi2 = 5205,
  CODE_FOR_one_cmplhi2 = 5206,
  CODE_FOR_one_cmplsi2 = 5207,
  CODE_FOR_one_cmpldi2 = 5208,
  CODE_FOR_ashlqi3 = 5209,
  CODE_FOR_ashlhi3 = 5210,
  CODE_FOR_ashlsi3 = 5211,
  CODE_FOR_ashldi3 = 5212,
  CODE_FOR_ashlti3 = 5213,
  CODE_FOR_x86_shiftsi_adj_1 = 5214,
  CODE_FOR_x86_shiftdi_adj_1 = 5215,
  CODE_FOR_x86_shiftsi_adj_2 = 5216,
  CODE_FOR_x86_shiftdi_adj_2 = 5217,
  CODE_FOR_lshrqi3 = 5218,
  CODE_FOR_ashrqi3 = 5219,
  CODE_FOR_lshrhi3 = 5220,
  CODE_FOR_ashrhi3 = 5221,
  CODE_FOR_lshrsi3 = 5222,
  CODE_FOR_ashrsi3 = 5223,
  CODE_FOR_lshrdi3 = 5224,
  CODE_FOR_ashrdi3 = 5225,
  CODE_FOR_lshrti3 = 5226,
  CODE_FOR_ashrti3 = 5227,
  CODE_FOR_x86_shiftsi_adj_3 = 5228,
  CODE_FOR_x86_shiftdi_adj_3 = 5229,
  CODE_FOR_rotlti3 = 5230,
  CODE_FOR_rotrti3 = 5231,
  CODE_FOR_rotldi3 = 5232,
  CODE_FOR_rotrdi3 = 5233,
  CODE_FOR_rotlqi3 = 5234,
  CODE_FOR_rotrqi3 = 5235,
  CODE_FOR_rotlhi3 = 5236,
  CODE_FOR_rotrhi3 = 5237,
  CODE_FOR_rotlsi3 = 5238,
  CODE_FOR_rotrsi3 = 5239,
  CODE_FOR_indirect_jump = 5240,
  CODE_FOR_tablejump = 5241,
  CODE_FOR_call = 5242,
  CODE_FOR_sibcall = 5243,
  CODE_FOR_call_pop = 5244,
  CODE_FOR_call_value = 5245,
  CODE_FOR_sibcall_value = 5246,
  CODE_FOR_call_value_pop = 5247,
  CODE_FOR_untyped_call = 5248,
  CODE_FOR_memory_blockage = 5249,
  CODE_FOR_return = 5250,
  CODE_FOR_simple_return = 5251,
  CODE_FOR_prologue = 5252,
  CODE_FOR_set_got = 5253,
  CODE_FOR_set_got_labelled = 5254,
  CODE_FOR_epilogue = 5255,
  CODE_FOR_sibcall_epilogue = 5256,
  CODE_FOR_eh_return = 5257,
  CODE_FOR_split_stack_prologue = 5258,
  CODE_FOR_split_stack_space_check = 5259,
  CODE_FOR_ffssi2 = 5260,
  CODE_FOR_ffsdi2 = 5261,
  CODE_FOR_clzsi2 = 5262,
  CODE_FOR_clzdi2 = 5263,
  CODE_FOR_bmi2_bzhi_si3 = 5264,
  CODE_FOR_bmi2_bzhi_di3 = 5265,
  CODE_FOR_bswapdi2 = 5266,
  CODE_FOR_bswapsi2 = 5267,
  CODE_FOR_paritydi2 = 5268,
  CODE_FOR_paritysi2 = 5269,
  CODE_FOR_tls_global_dynamic_32 = 5270,
  CODE_FOR_tls_global_dynamic_64_si = 5271,
  CODE_FOR_tls_global_dynamic_64_di = 5272,
  CODE_FOR_tls_local_dynamic_base_32 = 5273,
  CODE_FOR_tls_local_dynamic_base_64_si = 5274,
  CODE_FOR_tls_local_dynamic_base_64_di = 5275,
  CODE_FOR_tls_dynamic_gnu2_32 = 5276,
  CODE_FOR_tls_dynamic_gnu2_64 = 5277,
  CODE_FOR_rsqrtsf2 = 5278,
  CODE_FOR_sqrtsf2 = 5279,
  CODE_FOR_sqrtdf2 = 5280,
  CODE_FOR_fmodxf3 = 5281,
  CODE_FOR_fmodsf3 = 5282,
  CODE_FOR_fmoddf3 = 5283,
  CODE_FOR_remainderxf3 = 5284,
  CODE_FOR_remaindersf3 = 5285,
  CODE_FOR_remainderdf3 = 5286,
  CODE_FOR_sincossf3 = 5287,
  CODE_FOR_sincosdf3 = 5288,
  CODE_FOR_tanxf2 = 5289,
  CODE_FOR_tansf2 = 5290,
  CODE_FOR_tandf2 = 5291,
  CODE_FOR_atan2xf3 = 5292,
  CODE_FOR_atan2sf3 = 5293,
  CODE_FOR_atan2df3 = 5294,
  CODE_FOR_atanxf2 = 5295,
  CODE_FOR_atansf2 = 5296,
  CODE_FOR_atandf2 = 5297,
  CODE_FOR_asinxf2 = 5298,
  CODE_FOR_asinsf2 = 5299,
  CODE_FOR_asindf2 = 5300,
  CODE_FOR_acosxf2 = 5301,
  CODE_FOR_acossf2 = 5302,
  CODE_FOR_acosdf2 = 5303,
  CODE_FOR_logxf2 = 5304,
  CODE_FOR_logsf2 = 5305,
  CODE_FOR_logdf2 = 5306,
  CODE_FOR_log10xf2 = 5307,
  CODE_FOR_log10sf2 = 5308,
  CODE_FOR_log10df2 = 5309,
  CODE_FOR_log2xf2 = 5310,
  CODE_FOR_log2sf2 = 5311,
  CODE_FOR_log2df2 = 5312,
  CODE_FOR_log1pxf2 = 5313,
  CODE_FOR_log1psf2 = 5314,
  CODE_FOR_log1pdf2 = 5315,
  CODE_FOR_logbxf2 = 5316,
  CODE_FOR_logbsf2 = 5317,
  CODE_FOR_logbdf2 = 5318,
  CODE_FOR_ilogbxf2 = 5319,
  CODE_FOR_ilogbsf2 = 5320,
  CODE_FOR_ilogbdf2 = 5321,
  CODE_FOR_expNcorexf3 = 5322,
  CODE_FOR_expxf2 = 5323,
  CODE_FOR_expsf2 = 5324,
  CODE_FOR_expdf2 = 5325,
  CODE_FOR_exp10xf2 = 5326,
  CODE_FOR_exp10sf2 = 5327,
  CODE_FOR_exp10df2 = 5328,
  CODE_FOR_exp2xf2 = 5329,
  CODE_FOR_exp2sf2 = 5330,
  CODE_FOR_exp2df2 = 5331,
  CODE_FOR_expm1xf2 = 5332,
  CODE_FOR_expm1sf2 = 5333,
  CODE_FOR_expm1df2 = 5334,
  CODE_FOR_ldexpxf3 = 5335,
  CODE_FOR_ldexpsf3 = 5336,
  CODE_FOR_ldexpdf3 = 5337,
  CODE_FOR_scalbxf3 = 5338,
  CODE_FOR_scalbsf3 = 5339,
  CODE_FOR_scalbdf3 = 5340,
  CODE_FOR_significandxf2 = 5341,
  CODE_FOR_significandsf2 = 5342,
  CODE_FOR_significanddf2 = 5343,
  CODE_FOR_rintsf2 = 5344,
  CODE_FOR_rintdf2 = 5345,
  CODE_FOR_roundsf2 = 5346,
  CODE_FOR_rounddf2 = 5347,
  CODE_FOR_roundxf2 = 5348,
  CODE_FOR_lrintxfhi2 = 5349,
  CODE_FOR_lrintxfsi2 = 5350,
  CODE_FOR_lrintxfdi2 = 5351,
  CODE_FOR_lrintsfsi2 = 5352,
  CODE_FOR_lrintsfdi2 = 5353,
  CODE_FOR_lrintdfsi2 = 5354,
  CODE_FOR_lrintdfdi2 = 5355,
  CODE_FOR_lroundsfhi2 = 5356,
  CODE_FOR_lrounddfhi2 = 5357,
  CODE_FOR_lroundxfhi2 = 5358,
  CODE_FOR_lroundsfsi2 = 5359,
  CODE_FOR_lrounddfsi2 = 5360,
  CODE_FOR_lroundxfsi2 = 5361,
  CODE_FOR_lroundsfdi2 = 5362,
  CODE_FOR_lrounddfdi2 = 5363,
  CODE_FOR_lroundxfdi2 = 5364,
  CODE_FOR_floorxf2 = 5365,
  CODE_FOR_ceilxf2 = 5366,
  CODE_FOR_btruncxf2 = 5367,
  CODE_FOR_floorsf2 = 5368,
  CODE_FOR_ceilsf2 = 5369,
  CODE_FOR_btruncsf2 = 5370,
  CODE_FOR_floordf2 = 5371,
  CODE_FOR_ceildf2 = 5372,
  CODE_FOR_btruncdf2 = 5373,
  CODE_FOR_nearbyintxf2 = 5374,
  CODE_FOR_nearbyintsf2 = 5375,
  CODE_FOR_nearbyintdf2 = 5376,
  CODE_FOR_lfloorxfhi2 = 5377,
  CODE_FOR_lceilxfhi2 = 5378,
  CODE_FOR_lfloorxfsi2 = 5379,
  CODE_FOR_lceilxfsi2 = 5380,
  CODE_FOR_lfloorxfdi2 = 5381,
  CODE_FOR_lceilxfdi2 = 5382,
  CODE_FOR_lfloorsfsi2 = 5383,
  CODE_FOR_lceilsfsi2 = 5384,
  CODE_FOR_lfloorsfdi2 = 5385,
  CODE_FOR_lceilsfdi2 = 5386,
  CODE_FOR_lfloordfsi2 = 5387,
  CODE_FOR_lceildfsi2 = 5388,
  CODE_FOR_lfloordfdi2 = 5389,
  CODE_FOR_lceildfdi2 = 5390,
  CODE_FOR_isinfxf2 = 5391,
  CODE_FOR_isinfsf2 = 5392,
  CODE_FOR_isinfdf2 = 5393,
  CODE_FOR_signbittf2 = 5394,
  CODE_FOR_signbitxf2 = 5395,
  CODE_FOR_signbitdf2 = 5396,
  CODE_FOR_signbitsf2 = 5397,
  CODE_FOR_movmemsi = 5398,
  CODE_FOR_movmemdi = 5399,
  CODE_FOR_strmov = 5400,
  CODE_FOR_strmov_singleop = 5401,
  CODE_FOR_rep_mov = 5402,
  CODE_FOR_setmemsi = 5403,
  CODE_FOR_setmemdi = 5404,
  CODE_FOR_strset = 5405,
  CODE_FOR_strset_singleop = 5406,
  CODE_FOR_rep_stos = 5407,
  CODE_FOR_cmpstrnsi = 5408,
  CODE_FOR_cmpintqi = 5409,
  CODE_FOR_cmpstrnqi_nz_1 = 5410,
  CODE_FOR_cmpstrnqi_1 = 5411,
  CODE_FOR_strlensi = 5412,
  CODE_FOR_strlendi = 5413,
  CODE_FOR_strlenqi_1 = 5414,
  CODE_FOR_movqicc = 5415,
  CODE_FOR_movhicc = 5416,
  CODE_FOR_movsicc = 5417,
  CODE_FOR_movdicc = 5418,
  CODE_FOR_x86_movsicc_0_m1 = 5419,
  CODE_FOR_x86_movdicc_0_m1 = 5420,
  CODE_FOR_movsfcc = 5421,
  CODE_FOR_movdfcc = 5422,
  CODE_FOR_movxfcc = 5423,
  CODE_FOR_addqicc = 5424,
  CODE_FOR_addhicc = 5425,
  CODE_FOR_addsicc = 5426,
  CODE_FOR_adddicc = 5427,
  CODE_FOR_allocate_stack = 5428,
  CODE_FOR_probe_stack = 5429,
  CODE_FOR_builtin_setjmp_receiver = 5430,
  CODE_FOR_prefetch = 5431,
  CODE_FOR_stack_protect_set = 5432,
  CODE_FOR_stack_protect_test = 5433,
  CODE_FOR_lwp_llwpcb = 5434,
  CODE_FOR_lwp_slwpcb = 5435,
  CODE_FOR_lwp_lwpvalsi3 = 5436,
  CODE_FOR_lwp_lwpvaldi3 = 5437,
  CODE_FOR_lwp_lwpinssi3 = 5438,
  CODE_FOR_lwp_lwpinsdi3 = 5439,
  CODE_FOR_pause = 5440,
  CODE_FOR_xbegin = 5441,
  CODE_FOR_xtest = 5442,
  CODE_FOR_bnd32_mk = 5443,
  CODE_FOR_bnd64_mk = 5444,
  CODE_FOR_movbnd32 = 5445,
  CODE_FOR_movbnd64 = 5446,
  CODE_FOR_bnd32_cl = 5447,
  CODE_FOR_bnd32_cu = 5448,
  CODE_FOR_bnd32_cn = 5449,
  CODE_FOR_bnd64_cl = 5450,
  CODE_FOR_bnd64_cu = 5451,
  CODE_FOR_bnd64_cn = 5452,
  CODE_FOR_bnd32_ldx = 5453,
  CODE_FOR_bnd64_ldx = 5454,
  CODE_FOR_bnd32_stx = 5455,
  CODE_FOR_bnd64_stx = 5456,
  CODE_FOR_rdpkru = 5457,
  CODE_FOR_wrpkru = 5458,
  CODE_FOR_movv8qi = 5459,
  CODE_FOR_movv4hi = 5460,
  CODE_FOR_movv2si = 5461,
  CODE_FOR_movv1di = 5462,
  CODE_FOR_movv2sf = 5463,
  CODE_FOR_movmisalignv8qi = 5464,
  CODE_FOR_movmisalignv4hi = 5465,
  CODE_FOR_movmisalignv2si = 5466,
  CODE_FOR_movmisalignv1di = 5467,
  CODE_FOR_movmisalignv2sf = 5468,
  CODE_FOR_mmx_addv2sf3 = 5469,
  CODE_FOR_mmx_subv2sf3 = 5470,
  CODE_FOR_mmx_subrv2sf3 = 5471,
  CODE_FOR_mmx_mulv2sf3 = 5472,
  CODE_FOR_mmx_smaxv2sf3 = 5473,
  CODE_FOR_mmx_sminv2sf3 = 5474,
  CODE_FOR_mmx_eqv2sf3 = 5475,
  CODE_FOR_vec_setv2sf = 5476,
  CODE_FOR_vec_extractv2sf = 5477,
  CODE_FOR_vec_initv2sf = 5478,
  CODE_FOR_mmx_addv8qi3 = 5479,
  CODE_FOR_mmx_subv8qi3 = 5480,
  CODE_FOR_mmx_addv4hi3 = 5481,
  CODE_FOR_mmx_subv4hi3 = 5482,
  CODE_FOR_mmx_addv2si3 = 5483,
  CODE_FOR_mmx_subv2si3 = 5484,
  CODE_FOR_mmx_addv1di3 = 5485,
  CODE_FOR_mmx_subv1di3 = 5486,
  CODE_FOR_mmx_ssaddv8qi3 = 5487,
  CODE_FOR_mmx_usaddv8qi3 = 5488,
  CODE_FOR_mmx_sssubv8qi3 = 5489,
  CODE_FOR_mmx_ussubv8qi3 = 5490,
  CODE_FOR_mmx_ssaddv4hi3 = 5491,
  CODE_FOR_mmx_usaddv4hi3 = 5492,
  CODE_FOR_mmx_sssubv4hi3 = 5493,
  CODE_FOR_mmx_ussubv4hi3 = 5494,
  CODE_FOR_mmx_mulv4hi3 = 5495,
  CODE_FOR_mmx_smulv4hi3_highpart = 5496,
  CODE_FOR_mmx_umulv4hi3_highpart = 5497,
  CODE_FOR_mmx_pmaddwd = 5498,
  CODE_FOR_mmx_pmulhrwv4hi3 = 5499,
  CODE_FOR_sse2_umulv1siv1di3 = 5500,
  CODE_FOR_mmx_smaxv4hi3 = 5501,
  CODE_FOR_mmx_sminv4hi3 = 5502,
  CODE_FOR_mmx_umaxv8qi3 = 5503,
  CODE_FOR_mmx_uminv8qi3 = 5504,
  CODE_FOR_mmx_eqv8qi3 = 5505,
  CODE_FOR_mmx_eqv4hi3 = 5506,
  CODE_FOR_mmx_eqv2si3 = 5507,
  CODE_FOR_mmx_andv8qi3 = 5508,
  CODE_FOR_mmx_iorv8qi3 = 5509,
  CODE_FOR_mmx_xorv8qi3 = 5510,
  CODE_FOR_mmx_andv4hi3 = 5511,
  CODE_FOR_mmx_iorv4hi3 = 5512,
  CODE_FOR_mmx_xorv4hi3 = 5513,
  CODE_FOR_mmx_andv2si3 = 5514,
  CODE_FOR_mmx_iorv2si3 = 5515,
  CODE_FOR_mmx_xorv2si3 = 5516,
  CODE_FOR_mmx_pinsrw = 5517,
  CODE_FOR_mmx_pshufw = 5518,
  CODE_FOR_vec_setv2si = 5519,
  CODE_FOR_vec_extractv2si = 5520,
  CODE_FOR_vec_initv2si = 5521,
  CODE_FOR_vec_setv4hi = 5522,
  CODE_FOR_vec_extractv4hi = 5523,
  CODE_FOR_vec_initv4hi = 5524,
  CODE_FOR_vec_setv8qi = 5525,
  CODE_FOR_vec_extractv8qi = 5526,
  CODE_FOR_vec_initv8qi = 5527,
  CODE_FOR_mmx_uavgv8qi3 = 5528,
  CODE_FOR_mmx_uavgv4hi3 = 5529,
  CODE_FOR_mmx_maskmovq = 5530,
  CODE_FOR_mmx_emms = 5531,
  CODE_FOR_mmx_femms = 5532,
  CODE_FOR_movv64qi = 5533,
  CODE_FOR_movv32qi = 5534,
  CODE_FOR_movv16qi = 5535,
  CODE_FOR_movv32hi = 5536,
  CODE_FOR_movv16hi = 5537,
  CODE_FOR_movv8hi = 5538,
  CODE_FOR_movv16si = 5539,
  CODE_FOR_movv8si = 5540,
  CODE_FOR_movv4si = 5541,
  CODE_FOR_movv8di = 5542,
  CODE_FOR_movv4di = 5543,
  CODE_FOR_movv2di = 5544,
  CODE_FOR_movv4ti = 5545,
  CODE_FOR_movv2ti = 5546,
  CODE_FOR_movv1ti = 5547,
  CODE_FOR_movv16sf = 5548,
  CODE_FOR_movv8sf = 5549,
  CODE_FOR_movv4sf = 5550,
  CODE_FOR_movv8df = 5551,
  CODE_FOR_movv4df = 5552,
  CODE_FOR_movv2df = 5553,
  CODE_FOR_movmisalignv64qi = 5554,
  CODE_FOR_movmisalignv32qi = 5555,
  CODE_FOR_movmisalignv16qi = 5556,
  CODE_FOR_movmisalignv32hi = 5557,
  CODE_FOR_movmisalignv16hi = 5558,
  CODE_FOR_movmisalignv8hi = 5559,
  CODE_FOR_movmisalignv16si = 5560,
  CODE_FOR_movmisalignv8si = 5561,
  CODE_FOR_movmisalignv4si = 5562,
  CODE_FOR_movmisalignv8di = 5563,
  CODE_FOR_movmisalignv4di = 5564,
  CODE_FOR_movmisalignv2di = 5565,
  CODE_FOR_movmisalignv4ti = 5566,
  CODE_FOR_movmisalignv2ti = 5567,
  CODE_FOR_movmisalignv1ti = 5568,
  CODE_FOR_movmisalignv16sf = 5569,
  CODE_FOR_movmisalignv8sf = 5570,
  CODE_FOR_movmisalignv4sf = 5571,
  CODE_FOR_movmisalignv8df = 5572,
  CODE_FOR_movmisalignv4df = 5573,
  CODE_FOR_movmisalignv2df = 5574,
  CODE_FOR_storentdi = 5575,
  CODE_FOR_storentsi = 5576,
  CODE_FOR_storentsf = 5577,
  CODE_FOR_storentdf = 5578,
  CODE_FOR_storentv8di = 5579,
  CODE_FOR_storentv4di = 5580,
  CODE_FOR_storentv2di = 5581,
  CODE_FOR_storentv16sf = 5582,
  CODE_FOR_storentv8sf = 5583,
  CODE_FOR_storentv4sf = 5584,
  CODE_FOR_storentv8df = 5585,
  CODE_FOR_storentv4df = 5586,
  CODE_FOR_storentv2df = 5587,
  CODE_FOR_kmovb = 5588,
  CODE_FOR_kmovw = 5589,
  CODE_FOR_kmovd = 5590,
  CODE_FOR_kmovq = 5591,
  CODE_FOR_absv16sf2 = 5592,
  CODE_FOR_negv16sf2 = 5593,
  CODE_FOR_absv8sf2 = 5594,
  CODE_FOR_negv8sf2 = 5595,
  CODE_FOR_absv4sf2 = 5596,
  CODE_FOR_negv4sf2 = 5597,
  CODE_FOR_absv8df2 = 5598,
  CODE_FOR_negv8df2 = 5599,
  CODE_FOR_absv4df2 = 5600,
  CODE_FOR_negv4df2 = 5601,
  CODE_FOR_absv2df2 = 5602,
  CODE_FOR_negv2df2 = 5603,
  CODE_FOR_addv16sf3 = 5604,
  CODE_FOR_addv16sf3_round = 5605,
  CODE_FOR_addv16sf3_mask = 5606,
  CODE_FOR_addv16sf3_mask_round = 5607,
  CODE_FOR_subv16sf3 = 5608,
  CODE_FOR_subv16sf3_round = 5609,
  CODE_FOR_subv16sf3_mask = 5610,
  CODE_FOR_subv16sf3_mask_round = 5611,
  CODE_FOR_addv8sf3 = 5612,
   CODE_FOR_addv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv8sf3_mask = 5613,
   CODE_FOR_addv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3 = 5614,
   CODE_FOR_subv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv8sf3_mask = 5615,
   CODE_FOR_subv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3 = 5616,
   CODE_FOR_addv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_addv4sf3_mask = 5617,
   CODE_FOR_addv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3 = 5618,
   CODE_FOR_subv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_subv4sf3_mask = 5619,
   CODE_FOR_subv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv8df3 = 5620,
  CODE_FOR_addv8df3_round = 5621,
  CODE_FOR_addv8df3_mask = 5622,
  CODE_FOR_addv8df3_mask_round = 5623,
  CODE_FOR_subv8df3 = 5624,
  CODE_FOR_subv8df3_round = 5625,
  CODE_FOR_subv8df3_mask = 5626,
  CODE_FOR_subv8df3_mask_round = 5627,
  CODE_FOR_addv4df3 = 5628,
   CODE_FOR_addv4df3_round = CODE_FOR_nothing,
  CODE_FOR_addv4df3_mask = 5629,
   CODE_FOR_addv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3 = 5630,
   CODE_FOR_subv4df3_round = CODE_FOR_nothing,
  CODE_FOR_subv4df3_mask = 5631,
   CODE_FOR_subv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3 = 5632,
   CODE_FOR_addv2df3_round = CODE_FOR_nothing,
  CODE_FOR_addv2df3_mask = 5633,
   CODE_FOR_addv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3 = 5634,
   CODE_FOR_subv2df3_round = CODE_FOR_nothing,
  CODE_FOR_subv2df3_mask = 5635,
   CODE_FOR_subv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv16sf3 = 5636,
  CODE_FOR_mulv16sf3_round = 5637,
  CODE_FOR_mulv16sf3_mask = 5638,
  CODE_FOR_mulv16sf3_mask_round = 5639,
  CODE_FOR_mulv8sf3 = 5640,
   CODE_FOR_mulv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv8sf3_mask = 5641,
   CODE_FOR_mulv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3 = 5642,
   CODE_FOR_mulv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4sf3_mask = 5643,
   CODE_FOR_mulv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv8df3 = 5644,
  CODE_FOR_mulv8df3_round = 5645,
  CODE_FOR_mulv8df3_mask = 5646,
  CODE_FOR_mulv8df3_mask_round = 5647,
  CODE_FOR_mulv4df3 = 5648,
   CODE_FOR_mulv4df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv4df3_mask = 5649,
   CODE_FOR_mulv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3 = 5650,
   CODE_FOR_mulv2df3_round = CODE_FOR_nothing,
  CODE_FOR_mulv2df3_mask = 5651,
   CODE_FOR_mulv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_divv8df3 = 5652,
  CODE_FOR_divv4df3 = 5653,
  CODE_FOR_divv2df3 = 5654,
  CODE_FOR_divv16sf3 = 5655,
  CODE_FOR_divv8sf3 = 5656,
  CODE_FOR_divv4sf3 = 5657,
  CODE_FOR_sqrtv8df2 = 5658,
  CODE_FOR_sqrtv4df2 = 5659,
  CODE_FOR_sqrtv2df2 = 5660,
  CODE_FOR_sqrtv16sf2 = 5661,
  CODE_FOR_sqrtv8sf2 = 5662,
  CODE_FOR_sqrtv4sf2 = 5663,
  CODE_FOR_rsqrtv8sf2 = 5664,
  CODE_FOR_rsqrtv4sf2 = 5665,
  CODE_FOR_rsqrtv16sf2 = 5666,
  CODE_FOR_smaxv16sf3 = 5667,
  CODE_FOR_smaxv16sf3_round = 5668,
  CODE_FOR_smaxv16sf3_mask = 5669,
  CODE_FOR_smaxv16sf3_mask_round = 5670,
  CODE_FOR_sminv16sf3 = 5671,
  CODE_FOR_sminv16sf3_round = 5672,
  CODE_FOR_sminv16sf3_mask = 5673,
  CODE_FOR_sminv16sf3_mask_round = 5674,
  CODE_FOR_smaxv8sf3 = 5675,
   CODE_FOR_smaxv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8sf3_mask = 5676,
   CODE_FOR_smaxv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3 = 5677,
   CODE_FOR_sminv8sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv8sf3_mask = 5678,
   CODE_FOR_sminv8sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3 = 5679,
   CODE_FOR_smaxv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4sf3_mask = 5680,
   CODE_FOR_smaxv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3 = 5681,
   CODE_FOR_sminv4sf3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4sf3_mask = 5682,
   CODE_FOR_sminv4sf3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv8df3 = 5683,
  CODE_FOR_smaxv8df3_round = 5684,
  CODE_FOR_smaxv8df3_mask = 5685,
  CODE_FOR_smaxv8df3_mask_round = 5686,
  CODE_FOR_sminv8df3 = 5687,
  CODE_FOR_sminv8df3_round = 5688,
  CODE_FOR_sminv8df3_mask = 5689,
  CODE_FOR_sminv8df3_mask_round = 5690,
  CODE_FOR_smaxv4df3 = 5691,
   CODE_FOR_smaxv4df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv4df3_mask = 5692,
   CODE_FOR_smaxv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3 = 5693,
   CODE_FOR_sminv4df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv4df3_mask = 5694,
   CODE_FOR_sminv4df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3 = 5695,
   CODE_FOR_smaxv2df3_round = CODE_FOR_nothing,
  CODE_FOR_smaxv2df3_mask = 5696,
   CODE_FOR_smaxv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3 = 5697,
   CODE_FOR_sminv2df3_round = CODE_FOR_nothing,
  CODE_FOR_sminv2df3_mask = 5698,
   CODE_FOR_sminv2df3_mask_round = CODE_FOR_nothing,
  CODE_FOR_sse3_haddv2df3 = 5699,
  CODE_FOR_reduc_plus_scal_v8df = 5700,
  CODE_FOR_reduc_plus_scal_v4df = 5701,
  CODE_FOR_reduc_plus_scal_v2df = 5702,
  CODE_FOR_reduc_plus_scal_v16sf = 5703,
  CODE_FOR_reduc_plus_scal_v8sf = 5704,
  CODE_FOR_reduc_plus_scal_v4sf = 5705,
  CODE_FOR_reduc_smax_scal_v32qi = 5706,
  CODE_FOR_reduc_smin_scal_v32qi = 5707,
  CODE_FOR_reduc_smax_scal_v16hi = 5708,
  CODE_FOR_reduc_smin_scal_v16hi = 5709,
  CODE_FOR_reduc_smax_scal_v8si = 5710,
  CODE_FOR_reduc_smin_scal_v8si = 5711,
  CODE_FOR_reduc_smax_scal_v4di = 5712,
  CODE_FOR_reduc_smin_scal_v4di = 5713,
  CODE_FOR_reduc_smax_scal_v8sf = 5714,
  CODE_FOR_reduc_smin_scal_v8sf = 5715,
  CODE_FOR_reduc_smax_scal_v4df = 5716,
  CODE_FOR_reduc_smin_scal_v4df = 5717,
  CODE_FOR_reduc_smax_scal_v4sf = 5718,
  CODE_FOR_reduc_smin_scal_v4sf = 5719,
  CODE_FOR_reduc_smax_scal_v64qi = 5720,
  CODE_FOR_reduc_smin_scal_v64qi = 5721,
  CODE_FOR_reduc_smax_scal_v32hi = 5722,
  CODE_FOR_reduc_smin_scal_v32hi = 5723,
  CODE_FOR_reduc_smax_scal_v16si = 5724,
  CODE_FOR_reduc_smin_scal_v16si = 5725,
  CODE_FOR_reduc_smax_scal_v8di = 5726,
  CODE_FOR_reduc_smin_scal_v8di = 5727,
  CODE_FOR_reduc_smax_scal_v16sf = 5728,
  CODE_FOR_reduc_smin_scal_v16sf = 5729,
  CODE_FOR_reduc_smax_scal_v8df = 5730,
  CODE_FOR_reduc_smin_scal_v8df = 5731,
  CODE_FOR_reduc_umax_scal_v16si = 5732,
  CODE_FOR_reduc_umin_scal_v16si = 5733,
  CODE_FOR_reduc_umax_scal_v8di = 5734,
  CODE_FOR_reduc_umin_scal_v8di = 5735,
  CODE_FOR_reduc_umax_scal_v32hi = 5736,
  CODE_FOR_reduc_umin_scal_v32hi = 5737,
  CODE_FOR_reduc_umax_scal_v64qi = 5738,
  CODE_FOR_reduc_umin_scal_v64qi = 5739,
  CODE_FOR_reduc_umax_scal_v32qi = 5740,
  CODE_FOR_reduc_umin_scal_v32qi = 5741,
  CODE_FOR_reduc_umax_scal_v16hi = 5742,
  CODE_FOR_reduc_umin_scal_v16hi = 5743,
  CODE_FOR_reduc_umax_scal_v8si = 5744,
  CODE_FOR_reduc_umin_scal_v8si = 5745,
  CODE_FOR_reduc_umax_scal_v4di = 5746,
  CODE_FOR_reduc_umin_scal_v4di = 5747,
  CODE_FOR_reduc_umin_scal_v8hi = 5748,
  CODE_FOR_vec_cmpv16sihi = 5749,
  CODE_FOR_vec_cmpv8siqi = 5750,
  CODE_FOR_vec_cmpv4siqi = 5751,
  CODE_FOR_vec_cmpv8diqi = 5752,
  CODE_FOR_vec_cmpv4diqi = 5753,
  CODE_FOR_vec_cmpv2diqi = 5754,
  CODE_FOR_vec_cmpv16sfhi = 5755,
  CODE_FOR_vec_cmpv8sfqi = 5756,
  CODE_FOR_vec_cmpv4sfqi = 5757,
  CODE_FOR_vec_cmpv8dfqi = 5758,
  CODE_FOR_vec_cmpv4dfqi = 5759,
  CODE_FOR_vec_cmpv2dfqi = 5760,
  CODE_FOR_vec_cmpv64qidi = 5761,
  CODE_FOR_vec_cmpv16qihi = 5762,
  CODE_FOR_vec_cmpv32qisi = 5763,
  CODE_FOR_vec_cmpv32hisi = 5764,
  CODE_FOR_vec_cmpv16hihi = 5765,
  CODE_FOR_vec_cmpv8hiqi = 5766,
  CODE_FOR_vec_cmpv32qiv32qi = 5767,
  CODE_FOR_vec_cmpv16hiv16hi = 5768,
  CODE_FOR_vec_cmpv8siv8si = 5769,
  CODE_FOR_vec_cmpv4div4di = 5770,
  CODE_FOR_vec_cmpv16qiv16qi = 5771,
  CODE_FOR_vec_cmpv8hiv8hi = 5772,
  CODE_FOR_vec_cmpv4siv4si = 5773,
  CODE_FOR_vec_cmpv2div2di = 5774,
  CODE_FOR_vec_cmpv8sfv8si = 5775,
  CODE_FOR_vec_cmpv4dfv4di = 5776,
  CODE_FOR_vec_cmpv4sfv4si = 5777,
  CODE_FOR_vec_cmpv2dfv2di = 5778,
  CODE_FOR_vec_cmpuv16sihi = 5779,
  CODE_FOR_vec_cmpuv8siqi = 5780,
  CODE_FOR_vec_cmpuv4siqi = 5781,
  CODE_FOR_vec_cmpuv8diqi = 5782,
  CODE_FOR_vec_cmpuv4diqi = 5783,
  CODE_FOR_vec_cmpuv2diqi = 5784,
  CODE_FOR_vec_cmpuv64qidi = 5785,
  CODE_FOR_vec_cmpuv16qihi = 5786,
  CODE_FOR_vec_cmpuv32qisi = 5787,
  CODE_FOR_vec_cmpuv32hisi = 5788,
  CODE_FOR_vec_cmpuv16hihi = 5789,
  CODE_FOR_vec_cmpuv8hiqi = 5790,
  CODE_FOR_vec_cmpuv32qiv32qi = 5791,
  CODE_FOR_vec_cmpuv16hiv16hi = 5792,
  CODE_FOR_vec_cmpuv8siv8si = 5793,
  CODE_FOR_vec_cmpuv4div4di = 5794,
  CODE_FOR_vec_cmpuv16qiv16qi = 5795,
  CODE_FOR_vec_cmpuv8hiv8hi = 5796,
  CODE_FOR_vec_cmpuv4siv4si = 5797,
  CODE_FOR_vec_cmpuv2div2di = 5798,
  CODE_FOR_vec_cmpeqv2div2di = 5799,
  CODE_FOR_vcondv64qiv16sf = 5800,
  CODE_FOR_vcondv32hiv16sf = 5801,
  CODE_FOR_vcondv16siv16sf = 5802,
  CODE_FOR_vcondv8div16sf = 5803,
  CODE_FOR_vcondv16sfv16sf = 5804,
  CODE_FOR_vcondv8dfv16sf = 5805,
  CODE_FOR_vcondv64qiv8df = 5806,
  CODE_FOR_vcondv32hiv8df = 5807,
  CODE_FOR_vcondv16siv8df = 5808,
  CODE_FOR_vcondv8div8df = 5809,
  CODE_FOR_vcondv16sfv8df = 5810,
  CODE_FOR_vcondv8dfv8df = 5811,
  CODE_FOR_vcondv32qiv8sf = 5812,
  CODE_FOR_vcondv32qiv4df = 5813,
  CODE_FOR_vcondv16hiv8sf = 5814,
  CODE_FOR_vcondv16hiv4df = 5815,
  CODE_FOR_vcondv8siv8sf = 5816,
  CODE_FOR_vcondv8siv4df = 5817,
  CODE_FOR_vcondv4div8sf = 5818,
  CODE_FOR_vcondv4div4df = 5819,
  CODE_FOR_vcondv8sfv8sf = 5820,
  CODE_FOR_vcondv8sfv4df = 5821,
  CODE_FOR_vcondv4dfv8sf = 5822,
  CODE_FOR_vcondv4dfv4df = 5823,
  CODE_FOR_vcondv16qiv4sf = 5824,
  CODE_FOR_vcondv16qiv2df = 5825,
  CODE_FOR_vcondv8hiv4sf = 5826,
  CODE_FOR_vcondv8hiv2df = 5827,
  CODE_FOR_vcondv4siv4sf = 5828,
  CODE_FOR_vcondv4siv2df = 5829,
  CODE_FOR_vcondv2div4sf = 5830,
  CODE_FOR_vcondv2div2df = 5831,
  CODE_FOR_vcondv4sfv4sf = 5832,
  CODE_FOR_vcondv4sfv2df = 5833,
  CODE_FOR_vcondv2dfv4sf = 5834,
  CODE_FOR_vcondv2dfv2df = 5835,
  CODE_FOR_vcond_mask_v16sihi = 5836,
  CODE_FOR_vcond_mask_v8siqi = 5837,
  CODE_FOR_vcond_mask_v4siqi = 5838,
  CODE_FOR_vcond_mask_v8diqi = 5839,
  CODE_FOR_vcond_mask_v4diqi = 5840,
  CODE_FOR_vcond_mask_v2diqi = 5841,
  CODE_FOR_vcond_mask_v16sfhi = 5842,
  CODE_FOR_vcond_mask_v8sfqi = 5843,
  CODE_FOR_vcond_mask_v4sfqi = 5844,
  CODE_FOR_vcond_mask_v8dfqi = 5845,
  CODE_FOR_vcond_mask_v4dfqi = 5846,
  CODE_FOR_vcond_mask_v2dfqi = 5847,
  CODE_FOR_vcond_mask_v64qidi = 5848,
  CODE_FOR_vcond_mask_v16qihi = 5849,
  CODE_FOR_vcond_mask_v32qisi = 5850,
  CODE_FOR_vcond_mask_v32hisi = 5851,
  CODE_FOR_vcond_mask_v16hihi = 5852,
  CODE_FOR_vcond_mask_v8hiqi = 5853,
  CODE_FOR_vcond_mask_v32qiv32qi = 5854,
  CODE_FOR_vcond_mask_v16hiv16hi = 5855,
  CODE_FOR_vcond_mask_v8siv8si = 5856,
  CODE_FOR_vcond_mask_v4div4di = 5857,
  CODE_FOR_vcond_mask_v16qiv16qi = 5858,
  CODE_FOR_vcond_mask_v8hiv8hi = 5859,
  CODE_FOR_vcond_mask_v4siv4si = 5860,
  CODE_FOR_vcond_mask_v2div2di = 5861,
  CODE_FOR_vcond_mask_v8sfv8si = 5862,
  CODE_FOR_vcond_mask_v4dfv4di = 5863,
  CODE_FOR_vcond_mask_v4sfv4si = 5864,
  CODE_FOR_vcond_mask_v2dfv2di = 5865,
  CODE_FOR_andv8sf3 = 5866,
  CODE_FOR_andv8sf3_mask = 5867,
  CODE_FOR_iorv8sf3 = 5868,
  CODE_FOR_iorv8sf3_mask = 5869,
  CODE_FOR_xorv8sf3 = 5870,
  CODE_FOR_xorv8sf3_mask = 5871,
  CODE_FOR_andv4sf3 = 5872,
  CODE_FOR_andv4sf3_mask = 5873,
  CODE_FOR_iorv4sf3 = 5874,
  CODE_FOR_iorv4sf3_mask = 5875,
  CODE_FOR_xorv4sf3 = 5876,
  CODE_FOR_xorv4sf3_mask = 5877,
  CODE_FOR_andv4df3 = 5878,
  CODE_FOR_andv4df3_mask = 5879,
  CODE_FOR_iorv4df3 = 5880,
  CODE_FOR_iorv4df3_mask = 5881,
  CODE_FOR_xorv4df3 = 5882,
  CODE_FOR_xorv4df3_mask = 5883,
  CODE_FOR_andv2df3 = 5884,
  CODE_FOR_andv2df3_mask = 5885,
  CODE_FOR_iorv2df3 = 5886,
  CODE_FOR_iorv2df3_mask = 5887,
  CODE_FOR_xorv2df3 = 5888,
  CODE_FOR_xorv2df3_mask = 5889,
  CODE_FOR_andv16sf3 = 5890,
  CODE_FOR_andv16sf3_mask = 5891,
  CODE_FOR_iorv16sf3 = 5892,
  CODE_FOR_iorv16sf3_mask = 5893,
  CODE_FOR_xorv16sf3 = 5894,
  CODE_FOR_xorv16sf3_mask = 5895,
  CODE_FOR_andv8df3 = 5896,
  CODE_FOR_andv8df3_mask = 5897,
  CODE_FOR_iorv8df3 = 5898,
  CODE_FOR_iorv8df3_mask = 5899,
  CODE_FOR_xorv8df3 = 5900,
  CODE_FOR_xorv8df3_mask = 5901,
  CODE_FOR_copysignv16sf3 = 5902,
  CODE_FOR_copysignv8sf3 = 5903,
  CODE_FOR_copysignv4sf3 = 5904,
  CODE_FOR_copysignv8df3 = 5905,
  CODE_FOR_copysignv4df3 = 5906,
  CODE_FOR_copysignv2df3 = 5907,
  CODE_FOR_andtf3 = 5908,
  CODE_FOR_iortf3 = 5909,
  CODE_FOR_xortf3 = 5910,
  CODE_FOR_fmasf4 = 5911,
  CODE_FOR_fmadf4 = 5912,
  CODE_FOR_fmav4sf4 = 5913,
  CODE_FOR_fmav2df4 = 5914,
  CODE_FOR_fmav8sf4 = 5915,
  CODE_FOR_fmav4df4 = 5916,
  CODE_FOR_fmav16sf4 = 5917,
  CODE_FOR_fmav8df4 = 5918,
  CODE_FOR_fmssf4 = 5919,
  CODE_FOR_fmsdf4 = 5920,
  CODE_FOR_fmsv4sf4 = 5921,
  CODE_FOR_fmsv2df4 = 5922,
  CODE_FOR_fmsv8sf4 = 5923,
  CODE_FOR_fmsv4df4 = 5924,
  CODE_FOR_fmsv16sf4 = 5925,
  CODE_FOR_fmsv8df4 = 5926,
  CODE_FOR_fnmasf4 = 5927,
  CODE_FOR_fnmadf4 = 5928,
  CODE_FOR_fnmav4sf4 = 5929,
  CODE_FOR_fnmav2df4 = 5930,
  CODE_FOR_fnmav8sf4 = 5931,
  CODE_FOR_fnmav4df4 = 5932,
  CODE_FOR_fnmav16sf4 = 5933,
  CODE_FOR_fnmav8df4 = 5934,
  CODE_FOR_fnmssf4 = 5935,
  CODE_FOR_fnmsdf4 = 5936,
  CODE_FOR_fnmsv4sf4 = 5937,
  CODE_FOR_fnmsv2df4 = 5938,
  CODE_FOR_fnmsv8sf4 = 5939,
  CODE_FOR_fnmsv4df4 = 5940,
  CODE_FOR_fnmsv16sf4 = 5941,
  CODE_FOR_fnmsv8df4 = 5942,
  CODE_FOR_fma4i_fmadd_sf = 5943,
  CODE_FOR_fma4i_fmadd_df = 5944,
  CODE_FOR_fma4i_fmadd_v4sf = 5945,
  CODE_FOR_fma4i_fmadd_v2df = 5946,
  CODE_FOR_fma4i_fmadd_v8sf = 5947,
  CODE_FOR_fma4i_fmadd_v4df = 5948,
  CODE_FOR_fma4i_fmadd_v16sf = 5949,
  CODE_FOR_fma4i_fmadd_v8df = 5950,
  CODE_FOR_avx512f_fmadd_v16sf_maskz = 5951,
  CODE_FOR_avx512f_fmadd_v16sf_maskz_round = 5952,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz = 5953,
  CODE_FOR_avx512vl_fmadd_v8sf_maskz_round = 5954,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz = 5955,
  CODE_FOR_avx512vl_fmadd_v4sf_maskz_round = 5956,
  CODE_FOR_avx512f_fmadd_v8df_maskz = 5957,
  CODE_FOR_avx512f_fmadd_v8df_maskz_round = 5958,
  CODE_FOR_avx512vl_fmadd_v4df_maskz = 5959,
  CODE_FOR_avx512vl_fmadd_v4df_maskz_round = 5960,
  CODE_FOR_avx512vl_fmadd_v2df_maskz = 5961,
  CODE_FOR_avx512vl_fmadd_v2df_maskz_round = 5962,
  CODE_FOR_fmaddsub_v16sf = 5963,
  CODE_FOR_fmaddsub_v8sf = 5964,
  CODE_FOR_fmaddsub_v4sf = 5965,
  CODE_FOR_fmaddsub_v8df = 5966,
  CODE_FOR_fmaddsub_v4df = 5967,
  CODE_FOR_fmaddsub_v2df = 5968,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz = 5969,
  CODE_FOR_avx512f_fmaddsub_v16sf_maskz_round = 5970,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz = 5971,
  CODE_FOR_avx512vl_fmaddsub_v8sf_maskz_round = 5972,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz = 5973,
  CODE_FOR_avx512vl_fmaddsub_v4sf_maskz_round = 5974,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz = 5975,
  CODE_FOR_avx512f_fmaddsub_v8df_maskz_round = 5976,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz = 5977,
  CODE_FOR_avx512vl_fmaddsub_v4df_maskz_round = 5978,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz = 5979,
  CODE_FOR_avx512vl_fmaddsub_v2df_maskz_round = 5980,
  CODE_FOR_fmai_vmfmadd_v4sf = 5981,
  CODE_FOR_fmai_vmfmadd_v4sf_round = 5982,
  CODE_FOR_fmai_vmfmadd_v2df = 5983,
  CODE_FOR_fmai_vmfmadd_v2df_round = 5984,
  CODE_FOR_fma4i_vmfmadd_v4sf = 5985,
  CODE_FOR_fma4i_vmfmadd_v2df = 5986,
  CODE_FOR_floatunsv16siv16sf2 = 5987,
  CODE_FOR_floatunsv8siv8sf2 = 5988,
  CODE_FOR_floatunsv4siv4sf2 = 5989,
  CODE_FOR_fixuns_truncv16sfv16si2 = 5990,
  CODE_FOR_fixuns_truncv8sfv8si2 = 5991,
  CODE_FOR_fixuns_truncv4sfv4si2 = 5992,
  CODE_FOR_avx_cvtpd2dq256_2 = 5993,
  CODE_FOR_avx_cvttpd2dq256_2 = 5994,
  CODE_FOR_sse2_cvtpd2ps = 5995,
  CODE_FOR_sse2_cvtpd2ps_mask = 5996,
  CODE_FOR_avx512bw_cvtmask2bv64qi = 5997,
  CODE_FOR_avx512vl_cvtmask2bv16qi = 5998,
  CODE_FOR_avx512vl_cvtmask2bv32qi = 5999,
  CODE_FOR_avx512bw_cvtmask2wv32hi = 6000,
  CODE_FOR_avx512vl_cvtmask2wv16hi = 6001,
  CODE_FOR_avx512vl_cvtmask2wv8hi = 6002,
  CODE_FOR_avx512f_cvtmask2dv16si = 6003,
  CODE_FOR_avx512vl_cvtmask2dv8si = 6004,
  CODE_FOR_avx512vl_cvtmask2dv4si = 6005,
  CODE_FOR_avx512f_cvtmask2qv8di = 6006,
  CODE_FOR_avx512vl_cvtmask2qv4di = 6007,
  CODE_FOR_avx512vl_cvtmask2qv2di = 6008,
  CODE_FOR_vec_unpacks_hi_v4sf = 6009,
  CODE_FOR_vec_unpacks_hi_v8sf = 6010,
  CODE_FOR_vec_unpacks_hi_v16sf = 6011,
  CODE_FOR_vec_unpacks_lo_v4sf = 6012,
  CODE_FOR_vec_unpacks_lo_v8sf = 6013,
  CODE_FOR_vec_unpacks_float_hi_v32hi = 6014,
  CODE_FOR_vec_unpacks_float_hi_v16hi = 6015,
  CODE_FOR_vec_unpacks_float_hi_v8hi = 6016,
  CODE_FOR_vec_unpacks_float_lo_v32hi = 6017,
  CODE_FOR_vec_unpacks_float_lo_v16hi = 6018,
  CODE_FOR_vec_unpacks_float_lo_v8hi = 6019,
  CODE_FOR_vec_unpacku_float_hi_v32hi = 6020,
  CODE_FOR_vec_unpacku_float_hi_v16hi = 6021,
  CODE_FOR_vec_unpacku_float_hi_v8hi = 6022,
  CODE_FOR_vec_unpacku_float_lo_v32hi = 6023,
  CODE_FOR_vec_unpacku_float_lo_v16hi = 6024,
  CODE_FOR_vec_unpacku_float_lo_v8hi = 6025,
  CODE_FOR_vec_unpacks_float_hi_v4si = 6026,
  CODE_FOR_vec_unpacks_float_lo_v4si = 6027,
  CODE_FOR_vec_unpacks_float_hi_v8si = 6028,
  CODE_FOR_vec_unpacks_float_lo_v8si = 6029,
  CODE_FOR_vec_unpacks_float_hi_v16si = 6030,
  CODE_FOR_vec_unpacks_float_lo_v16si = 6031,
  CODE_FOR_vec_unpacku_float_hi_v4si = 6032,
  CODE_FOR_vec_unpacku_float_lo_v4si = 6033,
  CODE_FOR_vec_unpacku_float_hi_v8si = 6034,
  CODE_FOR_vec_unpacku_float_hi_v16si = 6035,
  CODE_FOR_vec_unpacku_float_lo_v8si = 6036,
  CODE_FOR_vec_unpacku_float_lo_v16si = 6037,
  CODE_FOR_vec_pack_trunc_v8df = 6038,
  CODE_FOR_vec_pack_trunc_v4df = 6039,
  CODE_FOR_vec_pack_trunc_v2df = 6040,
  CODE_FOR_vec_pack_sfix_trunc_v8df = 6041,
  CODE_FOR_vec_pack_sfix_trunc_v4df = 6042,
  CODE_FOR_vec_pack_sfix_trunc_v2df = 6043,
  CODE_FOR_vec_pack_ufix_trunc_v8df = 6044,
  CODE_FOR_vec_pack_ufix_trunc_v4df = 6045,
  CODE_FOR_vec_pack_ufix_trunc_v2df = 6046,
  CODE_FOR_avx512f_vec_pack_sfix_v8df = 6047,
  CODE_FOR_vec_pack_sfix_v4df = 6048,
  CODE_FOR_vec_pack_sfix_v2df = 6049,
  CODE_FOR_sse_movhlps_exp = 6050,
  CODE_FOR_sse_movlhps_exp = 6051,
  CODE_FOR_vec_interleave_highv8sf = 6052,
  CODE_FOR_vec_interleave_lowv8sf = 6053,
  CODE_FOR_avx_shufps256 = 6054,
  CODE_FOR_avx_shufps256_mask = 6055,
  CODE_FOR_sse_shufps = 6056,
  CODE_FOR_sse_shufps_mask = 6057,
  CODE_FOR_sse_loadhps_exp = 6058,
  CODE_FOR_sse_loadlps_exp = 6059,
  CODE_FOR_vec_initv16qi = 6060,
  CODE_FOR_vec_initv8hi = 6061,
  CODE_FOR_vec_initv4si = 6062,
  CODE_FOR_vec_initv2di = 6063,
  CODE_FOR_vec_initv4sf = 6064,
  CODE_FOR_vec_initv2df = 6065,
  CODE_FOR_vec_setv32qi = 6066,
  CODE_FOR_vec_setv16qi = 6067,
  CODE_FOR_vec_setv16hi = 6068,
  CODE_FOR_vec_setv8hi = 6069,
  CODE_FOR_vec_setv16si = 6070,
  CODE_FOR_vec_setv8si = 6071,
  CODE_FOR_vec_setv4si = 6072,
  CODE_FOR_vec_setv8di = 6073,
  CODE_FOR_vec_setv4di = 6074,
  CODE_FOR_vec_setv2di = 6075,
  CODE_FOR_vec_setv16sf = 6076,
  CODE_FOR_vec_setv8sf = 6077,
  CODE_FOR_vec_setv4sf = 6078,
  CODE_FOR_vec_setv8df = 6079,
  CODE_FOR_vec_setv4df = 6080,
  CODE_FOR_vec_setv2df = 6081,
  CODE_FOR_avx512dq_vextractf64x2_mask = 6082,
  CODE_FOR_avx512dq_vextracti64x2_mask = 6083,
  CODE_FOR_avx512f_vextractf32x4_mask = 6084,
  CODE_FOR_avx512f_vextracti32x4_mask = 6085,
  CODE_FOR_avx512dq_vextractf32x8_mask = 6086,
  CODE_FOR_avx512dq_vextracti32x8_mask = 6087,
  CODE_FOR_avx512f_vextractf64x4_mask = 6088,
  CODE_FOR_avx512f_vextracti64x4_mask = 6089,
  CODE_FOR_avx512vl_vextractf128v8si = 6090,
  CODE_FOR_avx512vl_vextractf128v8sf = 6091,
  CODE_FOR_avx512vl_vextractf128v4di = 6092,
  CODE_FOR_avx512vl_vextractf128v4df = 6093,
  CODE_FOR_avx_vextractf128v32qi = 6094,
  CODE_FOR_avx_vextractf128v16hi = 6095,
  CODE_FOR_avx_vextractf128v8si = 6096,
  CODE_FOR_avx_vextractf128v4di = 6097,
  CODE_FOR_avx_vextractf128v8sf = 6098,
  CODE_FOR_avx_vextractf128v4df = 6099,
  CODE_FOR_vec_extractv64qi = 6100,
  CODE_FOR_vec_extractv32qi = 6101,
  CODE_FOR_vec_extractv16qi = 6102,
  CODE_FOR_vec_extractv32hi = 6103,
  CODE_FOR_vec_extractv16hi = 6104,
  CODE_FOR_vec_extractv8hi = 6105,
  CODE_FOR_vec_extractv16si = 6106,
  CODE_FOR_vec_extractv8si = 6107,
  CODE_FOR_vec_extractv4si = 6108,
  CODE_FOR_vec_extractv8di = 6109,
  CODE_FOR_vec_extractv4di = 6110,
  CODE_FOR_vec_extractv2di = 6111,
  CODE_FOR_vec_extractv16sf = 6112,
  CODE_FOR_vec_extractv8sf = 6113,
  CODE_FOR_vec_extractv4sf = 6114,
  CODE_FOR_vec_extractv8df = 6115,
  CODE_FOR_vec_extractv4df = 6116,
  CODE_FOR_vec_extractv2df = 6117,
  CODE_FOR_vec_interleave_highv4df = 6118,
  CODE_FOR_vec_interleave_highv2df = 6119,
  CODE_FOR_avx512f_movddup512 = 6120,
  CODE_FOR_avx512f_movddup512_mask = 6121,
  CODE_FOR_avx512f_unpcklpd512 = 6122,
  CODE_FOR_avx512f_unpcklpd512_mask = 6123,
  CODE_FOR_avx_movddup256 = 6124,
  CODE_FOR_avx_movddup256_mask = 6125,
  CODE_FOR_avx_unpcklpd256 = 6126,
  CODE_FOR_avx_unpcklpd256_mask = 6127,
  CODE_FOR_vec_interleave_lowv4df = 6128,
  CODE_FOR_vec_interleave_lowv2df = 6129,
  CODE_FOR_avx512f_vternlogv16si_maskz = 6130,
  CODE_FOR_avx512vl_vternlogv8si_maskz = 6131,
  CODE_FOR_avx512vl_vternlogv4si_maskz = 6132,
  CODE_FOR_avx512f_vternlogv8di_maskz = 6133,
  CODE_FOR_avx512vl_vternlogv4di_maskz = 6134,
  CODE_FOR_avx512vl_vternlogv2di_maskz = 6135,
  CODE_FOR_avx512f_shufps512_mask = 6136,
  CODE_FOR_avx512f_fixupimmv16sf_maskz = 6137,
  CODE_FOR_avx512f_fixupimmv16sf_maskz_round = 6138,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz = 6139,
  CODE_FOR_avx512vl_fixupimmv8sf_maskz_round = 6140,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz = 6141,
  CODE_FOR_avx512vl_fixupimmv4sf_maskz_round = 6142,
  CODE_FOR_avx512f_fixupimmv8df_maskz = 6143,
  CODE_FOR_avx512f_fixupimmv8df_maskz_round = 6144,
  CODE_FOR_avx512vl_fixupimmv4df_maskz = 6145,
  CODE_FOR_avx512vl_fixupimmv4df_maskz_round = 6146,
  CODE_FOR_avx512vl_fixupimmv2df_maskz = 6147,
  CODE_FOR_avx512vl_fixupimmv2df_maskz_round = 6148,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz = 6149,
  CODE_FOR_avx512f_sfixupimmv4sf_maskz_round = 6150,
  CODE_FOR_avx512f_sfixupimmv2df_maskz = 6151,
  CODE_FOR_avx512f_sfixupimmv2df_maskz_round = 6152,
  CODE_FOR_avx512f_shufpd512_mask = 6153,
  CODE_FOR_avx_shufpd256 = 6154,
  CODE_FOR_avx_shufpd256_mask = 6155,
  CODE_FOR_sse2_shufpd = 6156,
  CODE_FOR_sse2_shufpd_mask = 6157,
  CODE_FOR_sse2_loadhpd_exp = 6158,
  CODE_FOR_sse2_loadlpd_exp = 6159,
  CODE_FOR_avx512f_ss_truncatev16siv16qi2_mask_store = 6160,
  CODE_FOR_avx512f_truncatev16siv16qi2_mask_store = 6161,
  CODE_FOR_avx512f_us_truncatev16siv16qi2_mask_store = 6162,
  CODE_FOR_avx512f_ss_truncatev16siv16hi2_mask_store = 6163,
  CODE_FOR_avx512f_truncatev16siv16hi2_mask_store = 6164,
  CODE_FOR_avx512f_us_truncatev16siv16hi2_mask_store = 6165,
  CODE_FOR_avx512f_ss_truncatev8div8si2_mask_store = 6166,
  CODE_FOR_avx512f_truncatev8div8si2_mask_store = 6167,
  CODE_FOR_avx512f_us_truncatev8div8si2_mask_store = 6168,
  CODE_FOR_avx512f_ss_truncatev8div8hi2_mask_store = 6169,
  CODE_FOR_avx512f_truncatev8div8hi2_mask_store = 6170,
  CODE_FOR_avx512f_us_truncatev8div8hi2_mask_store = 6171,
  CODE_FOR_avx512bw_ss_truncatev32hiv32qi2_mask_store = 6172,
  CODE_FOR_avx512bw_truncatev32hiv32qi2_mask_store = 6173,
  CODE_FOR_avx512bw_us_truncatev32hiv32qi2_mask_store = 6174,
  CODE_FOR_avx512vl_ss_truncatev4div4si2_mask_store = 6175,
  CODE_FOR_avx512vl_truncatev4div4si2_mask_store = 6176,
  CODE_FOR_avx512vl_us_truncatev4div4si2_mask_store = 6177,
  CODE_FOR_avx512vl_ss_truncatev8siv8hi2_mask_store = 6178,
  CODE_FOR_avx512vl_truncatev8siv8hi2_mask_store = 6179,
  CODE_FOR_avx512vl_us_truncatev8siv8hi2_mask_store = 6180,
  CODE_FOR_avx512vl_ss_truncatev16hiv16qi2_mask_store = 6181,
  CODE_FOR_avx512vl_truncatev16hiv16qi2_mask_store = 6182,
  CODE_FOR_avx512vl_us_truncatev16hiv16qi2_mask_store = 6183,
  CODE_FOR_negv64qi2 = 6184,
  CODE_FOR_negv32qi2 = 6185,
  CODE_FOR_negv16qi2 = 6186,
  CODE_FOR_negv32hi2 = 6187,
  CODE_FOR_negv16hi2 = 6188,
  CODE_FOR_negv8hi2 = 6189,
  CODE_FOR_negv16si2 = 6190,
  CODE_FOR_negv8si2 = 6191,
  CODE_FOR_negv4si2 = 6192,
  CODE_FOR_negv8di2 = 6193,
  CODE_FOR_negv4di2 = 6194,
  CODE_FOR_negv2di2 = 6195,
  CODE_FOR_addv64qi3 = 6196,
  CODE_FOR_subv64qi3 = 6197,
  CODE_FOR_addv32qi3 = 6198,
  CODE_FOR_subv32qi3 = 6199,
  CODE_FOR_addv16qi3 = 6200,
  CODE_FOR_subv16qi3 = 6201,
  CODE_FOR_addv32hi3 = 6202,
  CODE_FOR_subv32hi3 = 6203,
  CODE_FOR_addv16hi3 = 6204,
  CODE_FOR_subv16hi3 = 6205,
  CODE_FOR_addv8hi3 = 6206,
  CODE_FOR_subv8hi3 = 6207,
  CODE_FOR_addv16si3 = 6208,
  CODE_FOR_subv16si3 = 6209,
  CODE_FOR_addv8si3 = 6210,
  CODE_FOR_subv8si3 = 6211,
  CODE_FOR_addv4si3 = 6212,
  CODE_FOR_subv4si3 = 6213,
  CODE_FOR_addv8di3 = 6214,
  CODE_FOR_subv8di3 = 6215,
  CODE_FOR_addv4di3 = 6216,
  CODE_FOR_subv4di3 = 6217,
  CODE_FOR_addv2di3 = 6218,
  CODE_FOR_subv2di3 = 6219,
  CODE_FOR_addv16si3_mask = 6220,
  CODE_FOR_subv16si3_mask = 6221,
  CODE_FOR_addv8si3_mask = 6222,
  CODE_FOR_subv8si3_mask = 6223,
  CODE_FOR_addv4si3_mask = 6224,
  CODE_FOR_subv4si3_mask = 6225,
  CODE_FOR_addv8di3_mask = 6226,
  CODE_FOR_subv8di3_mask = 6227,
  CODE_FOR_addv4di3_mask = 6228,
  CODE_FOR_subv4di3_mask = 6229,
  CODE_FOR_addv2di3_mask = 6230,
  CODE_FOR_subv2di3_mask = 6231,
  CODE_FOR_addv64qi3_mask = 6232,
  CODE_FOR_subv64qi3_mask = 6233,
  CODE_FOR_addv16qi3_mask = 6234,
  CODE_FOR_subv16qi3_mask = 6235,
  CODE_FOR_addv32qi3_mask = 6236,
  CODE_FOR_subv32qi3_mask = 6237,
  CODE_FOR_addv32hi3_mask = 6238,
  CODE_FOR_subv32hi3_mask = 6239,
  CODE_FOR_addv16hi3_mask = 6240,
  CODE_FOR_subv16hi3_mask = 6241,
  CODE_FOR_addv8hi3_mask = 6242,
  CODE_FOR_subv8hi3_mask = 6243,
  CODE_FOR_avx512bw_ssaddv64qi3 = 6244,
  CODE_FOR_avx512bw_ssaddv64qi3_mask = 6245,
  CODE_FOR_avx512bw_usaddv64qi3 = 6246,
  CODE_FOR_avx512bw_usaddv64qi3_mask = 6247,
  CODE_FOR_avx512bw_sssubv64qi3 = 6248,
  CODE_FOR_avx512bw_sssubv64qi3_mask = 6249,
  CODE_FOR_avx512bw_ussubv64qi3 = 6250,
  CODE_FOR_avx512bw_ussubv64qi3_mask = 6251,
  CODE_FOR_avx2_ssaddv32qi3 = 6252,
  CODE_FOR_avx2_ssaddv32qi3_mask = 6253,
  CODE_FOR_avx2_usaddv32qi3 = 6254,
  CODE_FOR_avx2_usaddv32qi3_mask = 6255,
  CODE_FOR_avx2_sssubv32qi3 = 6256,
  CODE_FOR_avx2_sssubv32qi3_mask = 6257,
  CODE_FOR_avx2_ussubv32qi3 = 6258,
  CODE_FOR_avx2_ussubv32qi3_mask = 6259,
  CODE_FOR_sse2_ssaddv16qi3 = 6260,
  CODE_FOR_sse2_ssaddv16qi3_mask = 6261,
  CODE_FOR_sse2_usaddv16qi3 = 6262,
  CODE_FOR_sse2_usaddv16qi3_mask = 6263,
  CODE_FOR_sse2_sssubv16qi3 = 6264,
  CODE_FOR_sse2_sssubv16qi3_mask = 6265,
  CODE_FOR_sse2_ussubv16qi3 = 6266,
  CODE_FOR_sse2_ussubv16qi3_mask = 6267,
  CODE_FOR_avx512bw_ssaddv32hi3 = 6268,
  CODE_FOR_avx512bw_ssaddv32hi3_mask = 6269,
  CODE_FOR_avx512bw_usaddv32hi3 = 6270,
  CODE_FOR_avx512bw_usaddv32hi3_mask = 6271,
  CODE_FOR_avx512bw_sssubv32hi3 = 6272,
  CODE_FOR_avx512bw_sssubv32hi3_mask = 6273,
  CODE_FOR_avx512bw_ussubv32hi3 = 6274,
  CODE_FOR_avx512bw_ussubv32hi3_mask = 6275,
  CODE_FOR_avx2_ssaddv16hi3 = 6276,
  CODE_FOR_avx2_ssaddv16hi3_mask = 6277,
  CODE_FOR_avx2_usaddv16hi3 = 6278,
  CODE_FOR_avx2_usaddv16hi3_mask = 6279,
  CODE_FOR_avx2_sssubv16hi3 = 6280,
  CODE_FOR_avx2_sssubv16hi3_mask = 6281,
  CODE_FOR_avx2_ussubv16hi3 = 6282,
  CODE_FOR_avx2_ussubv16hi3_mask = 6283,
  CODE_FOR_sse2_ssaddv8hi3 = 6284,
  CODE_FOR_sse2_ssaddv8hi3_mask = 6285,
  CODE_FOR_sse2_usaddv8hi3 = 6286,
  CODE_FOR_sse2_usaddv8hi3_mask = 6287,
  CODE_FOR_sse2_sssubv8hi3 = 6288,
  CODE_FOR_sse2_sssubv8hi3_mask = 6289,
  CODE_FOR_sse2_ussubv8hi3 = 6290,
  CODE_FOR_sse2_ussubv8hi3_mask = 6291,
  CODE_FOR_mulv64qi3 = 6292,
  CODE_FOR_mulv64qi3_mask = 6293,
  CODE_FOR_mulv32qi3 = 6294,
  CODE_FOR_mulv32qi3_mask = 6295,
  CODE_FOR_mulv16qi3 = 6296,
  CODE_FOR_mulv16qi3_mask = 6297,
  CODE_FOR_mulv32hi3 = 6298,
  CODE_FOR_mulv32hi3_mask = 6299,
  CODE_FOR_mulv16hi3 = 6300,
  CODE_FOR_mulv16hi3_mask = 6301,
  CODE_FOR_mulv8hi3 = 6302,
  CODE_FOR_mulv8hi3_mask = 6303,
  CODE_FOR_smulv32hi3_highpart = 6304,
  CODE_FOR_smulv32hi3_highpart_mask = 6305,
  CODE_FOR_umulv32hi3_highpart = 6306,
  CODE_FOR_umulv32hi3_highpart_mask = 6307,
  CODE_FOR_smulv16hi3_highpart = 6308,
  CODE_FOR_smulv16hi3_highpart_mask = 6309,
  CODE_FOR_umulv16hi3_highpart = 6310,
  CODE_FOR_umulv16hi3_highpart_mask = 6311,
  CODE_FOR_smulv8hi3_highpart = 6312,
  CODE_FOR_smulv8hi3_highpart_mask = 6313,
  CODE_FOR_umulv8hi3_highpart = 6314,
  CODE_FOR_umulv8hi3_highpart_mask = 6315,
  CODE_FOR_vec_widen_umult_even_v16si = 6316,
  CODE_FOR_vec_widen_umult_even_v16si_mask = 6317,
  CODE_FOR_vec_widen_umult_even_v8si = 6318,
  CODE_FOR_vec_widen_umult_even_v8si_mask = 6319,
  CODE_FOR_vec_widen_umult_even_v4si = 6320,
  CODE_FOR_vec_widen_umult_even_v4si_mask = 6321,
  CODE_FOR_vec_widen_smult_even_v16si = 6322,
  CODE_FOR_vec_widen_smult_even_v16si_mask = 6323,
  CODE_FOR_vec_widen_smult_even_v8si = 6324,
  CODE_FOR_vec_widen_smult_even_v8si_mask = 6325,
  CODE_FOR_sse4_1_mulv2siv2di3 = 6326,
  CODE_FOR_sse4_1_mulv2siv2di3_mask = 6327,
  CODE_FOR_avx2_pmaddwd = 6328,
  CODE_FOR_sse2_pmaddwd = 6329,
  CODE_FOR_mulv16si3 = 6330,
  CODE_FOR_mulv16si3_mask = 6331,
  CODE_FOR_mulv8si3 = 6332,
  CODE_FOR_mulv8si3_mask = 6333,
  CODE_FOR_mulv4si3 = 6334,
  CODE_FOR_mulv4si3_mask = 6335,
  CODE_FOR_mulv8di3 = 6336,
  CODE_FOR_mulv4di3 = 6337,
  CODE_FOR_mulv2di3 = 6338,
  CODE_FOR_vec_widen_smult_hi_v32qi = 6339,
  CODE_FOR_vec_widen_umult_hi_v32qi = 6340,
  CODE_FOR_vec_widen_smult_hi_v16qi = 6341,
  CODE_FOR_vec_widen_umult_hi_v16qi = 6342,
  CODE_FOR_vec_widen_smult_hi_v16hi = 6343,
  CODE_FOR_vec_widen_umult_hi_v16hi = 6344,
  CODE_FOR_vec_widen_smult_hi_v8hi = 6345,
  CODE_FOR_vec_widen_umult_hi_v8hi = 6346,
  CODE_FOR_vec_widen_smult_hi_v8si = 6347,
  CODE_FOR_vec_widen_umult_hi_v8si = 6348,
  CODE_FOR_vec_widen_smult_hi_v4si = 6349,
  CODE_FOR_vec_widen_umult_hi_v4si = 6350,
  CODE_FOR_vec_widen_smult_lo_v32qi = 6351,
  CODE_FOR_vec_widen_umult_lo_v32qi = 6352,
  CODE_FOR_vec_widen_smult_lo_v16qi = 6353,
  CODE_FOR_vec_widen_umult_lo_v16qi = 6354,
  CODE_FOR_vec_widen_smult_lo_v16hi = 6355,
  CODE_FOR_vec_widen_umult_lo_v16hi = 6356,
  CODE_FOR_vec_widen_smult_lo_v8hi = 6357,
  CODE_FOR_vec_widen_umult_lo_v8hi = 6358,
  CODE_FOR_vec_widen_smult_lo_v8si = 6359,
  CODE_FOR_vec_widen_umult_lo_v8si = 6360,
  CODE_FOR_vec_widen_smult_lo_v4si = 6361,
  CODE_FOR_vec_widen_umult_lo_v4si = 6362,
  CODE_FOR_vec_widen_smult_even_v4si = 6363,
  CODE_FOR_vec_widen_smult_odd_v16si = 6364,
  CODE_FOR_vec_widen_umult_odd_v16si = 6365,
  CODE_FOR_vec_widen_smult_odd_v8si = 6366,
  CODE_FOR_vec_widen_umult_odd_v8si = 6367,
  CODE_FOR_vec_widen_smult_odd_v4si = 6368,
  CODE_FOR_vec_widen_umult_odd_v4si = 6369,
  CODE_FOR_sdot_prodv32hi = 6370,
  CODE_FOR_sdot_prodv16hi = 6371,
  CODE_FOR_sdot_prodv8hi = 6372,
  CODE_FOR_sdot_prodv4si = 6373,
  CODE_FOR_usadv16qi = 6374,
  CODE_FOR_usadv32qi = 6375,
  CODE_FOR_vec_shl_v16qi = 6376,
  CODE_FOR_vec_shl_v8hi = 6377,
  CODE_FOR_vec_shl_v4si = 6378,
  CODE_FOR_vec_shl_v2di = 6379,
  CODE_FOR_vec_shr_v16qi = 6380,
  CODE_FOR_vec_shr_v8hi = 6381,
  CODE_FOR_vec_shr_v4si = 6382,
  CODE_FOR_vec_shr_v2di = 6383,
  CODE_FOR_smaxv32qi3 = 6384,
  CODE_FOR_sminv32qi3 = 6385,
  CODE_FOR_umaxv32qi3 = 6386,
  CODE_FOR_uminv32qi3 = 6387,
  CODE_FOR_smaxv16hi3 = 6388,
  CODE_FOR_sminv16hi3 = 6389,
  CODE_FOR_umaxv16hi3 = 6390,
  CODE_FOR_uminv16hi3 = 6391,
  CODE_FOR_smaxv8si3 = 6392,
  CODE_FOR_sminv8si3 = 6393,
  CODE_FOR_umaxv8si3 = 6394,
  CODE_FOR_uminv8si3 = 6395,
  CODE_FOR_smaxv64qi3 = 6396,
  CODE_FOR_sminv64qi3 = 6397,
  CODE_FOR_umaxv64qi3 = 6398,
  CODE_FOR_uminv64qi3 = 6399,
  CODE_FOR_smaxv32hi3 = 6400,
  CODE_FOR_sminv32hi3 = 6401,
  CODE_FOR_umaxv32hi3 = 6402,
  CODE_FOR_uminv32hi3 = 6403,
  CODE_FOR_smaxv16si3 = 6404,
  CODE_FOR_sminv16si3 = 6405,
  CODE_FOR_umaxv16si3 = 6406,
  CODE_FOR_uminv16si3 = 6407,
  CODE_FOR_smaxv16si3_mask = 6408,
  CODE_FOR_sminv16si3_mask = 6409,
  CODE_FOR_umaxv16si3_mask = 6410,
  CODE_FOR_uminv16si3_mask = 6411,
  CODE_FOR_smaxv8si3_mask = 6412,
  CODE_FOR_sminv8si3_mask = 6413,
  CODE_FOR_umaxv8si3_mask = 6414,
  CODE_FOR_uminv8si3_mask = 6415,
  CODE_FOR_smaxv4si3_mask = 6416,
  CODE_FOR_sminv4si3_mask = 6417,
  CODE_FOR_umaxv4si3_mask = 6418,
  CODE_FOR_uminv4si3_mask = 6419,
  CODE_FOR_smaxv8di3_mask = 6420,
  CODE_FOR_sminv8di3_mask = 6421,
  CODE_FOR_umaxv8di3_mask = 6422,
  CODE_FOR_uminv8di3_mask = 6423,
  CODE_FOR_smaxv4di3_mask = 6424,
  CODE_FOR_sminv4di3_mask = 6425,
  CODE_FOR_umaxv4di3_mask = 6426,
  CODE_FOR_uminv4di3_mask = 6427,
  CODE_FOR_smaxv2di3_mask = 6428,
  CODE_FOR_sminv2di3_mask = 6429,
  CODE_FOR_umaxv2di3_mask = 6430,
  CODE_FOR_uminv2di3_mask = 6431,
  CODE_FOR_smaxv8di3 = 6432,
  CODE_FOR_sminv8di3 = 6433,
  CODE_FOR_umaxv8di3 = 6434,
  CODE_FOR_uminv8di3 = 6435,
  CODE_FOR_smaxv4di3 = 6436,
  CODE_FOR_sminv4di3 = 6437,
  CODE_FOR_umaxv4di3 = 6438,
  CODE_FOR_uminv4di3 = 6439,
  CODE_FOR_smaxv2di3 = 6440,
  CODE_FOR_sminv2di3 = 6441,
  CODE_FOR_umaxv2di3 = 6442,
  CODE_FOR_uminv2di3 = 6443,
  CODE_FOR_smaxv16qi3 = 6444,
  CODE_FOR_sminv16qi3 = 6445,
  CODE_FOR_smaxv8hi3 = 6446,
  CODE_FOR_sminv8hi3 = 6447,
  CODE_FOR_smaxv4si3 = 6448,
  CODE_FOR_sminv4si3 = 6449,
  CODE_FOR_umaxv16qi3 = 6450,
  CODE_FOR_uminv16qi3 = 6451,
  CODE_FOR_umaxv8hi3 = 6452,
  CODE_FOR_uminv8hi3 = 6453,
  CODE_FOR_umaxv4si3 = 6454,
  CODE_FOR_uminv4si3 = 6455,
  CODE_FOR_avx2_eqv32qi3 = 6456,
  CODE_FOR_avx2_eqv16hi3 = 6457,
  CODE_FOR_avx2_eqv8si3 = 6458,
  CODE_FOR_avx2_eqv4di3 = 6459,
  CODE_FOR_avx512bw_eqv64qi3 = 6460,
  CODE_FOR_avx512bw_eqv64qi3_mask = 6461,
  CODE_FOR_avx512vl_eqv16qi3 = 6462,
  CODE_FOR_avx512vl_eqv16qi3_mask = 6463,
  CODE_FOR_avx512vl_eqv32qi3 = 6464,
  CODE_FOR_avx512vl_eqv32qi3_mask = 6465,
  CODE_FOR_avx512bw_eqv32hi3 = 6466,
  CODE_FOR_avx512bw_eqv32hi3_mask = 6467,
  CODE_FOR_avx512vl_eqv16hi3 = 6468,
  CODE_FOR_avx512vl_eqv16hi3_mask = 6469,
  CODE_FOR_avx512vl_eqv8hi3 = 6470,
  CODE_FOR_avx512vl_eqv8hi3_mask = 6471,
  CODE_FOR_avx512f_eqv16si3 = 6472,
  CODE_FOR_avx512f_eqv16si3_mask = 6473,
  CODE_FOR_avx512vl_eqv8si3 = 6474,
  CODE_FOR_avx512vl_eqv8si3_mask = 6475,
  CODE_FOR_avx512vl_eqv4si3 = 6476,
  CODE_FOR_avx512vl_eqv4si3_mask = 6477,
  CODE_FOR_avx512f_eqv8di3 = 6478,
  CODE_FOR_avx512f_eqv8di3_mask = 6479,
  CODE_FOR_avx512vl_eqv4di3 = 6480,
  CODE_FOR_avx512vl_eqv4di3_mask = 6481,
  CODE_FOR_avx512vl_eqv2di3 = 6482,
  CODE_FOR_avx512vl_eqv2di3_mask = 6483,
  CODE_FOR_sse2_eqv16qi3 = 6484,
  CODE_FOR_sse2_eqv8hi3 = 6485,
  CODE_FOR_sse2_eqv4si3 = 6486,
  CODE_FOR_sse4_1_eqv2di3 = 6487,
  CODE_FOR_vcondv64qiv16si = 6488,
  CODE_FOR_vcondv32hiv16si = 6489,
  CODE_FOR_vcondv16siv16si = 6490,
  CODE_FOR_vcondv8div16si = 6491,
  CODE_FOR_vcondv16sfv16si = 6492,
  CODE_FOR_vcondv8dfv16si = 6493,
  CODE_FOR_vcondv64qiv8di = 6494,
  CODE_FOR_vcondv32hiv8di = 6495,
  CODE_FOR_vcondv16siv8di = 6496,
  CODE_FOR_vcondv8div8di = 6497,
  CODE_FOR_vcondv16sfv8di = 6498,
  CODE_FOR_vcondv8dfv8di = 6499,
  CODE_FOR_vcondv64qiv32hi = 6500,
  CODE_FOR_vcondv32hiv32hi = 6501,
  CODE_FOR_vcondv16siv32hi = 6502,
  CODE_FOR_vcondv8div32hi = 6503,
  CODE_FOR_vcondv16sfv32hi = 6504,
  CODE_FOR_vcondv8dfv32hi = 6505,
  CODE_FOR_vcondv64qiv64qi = 6506,
  CODE_FOR_vcondv32hiv64qi = 6507,
  CODE_FOR_vcondv16siv64qi = 6508,
  CODE_FOR_vcondv8div64qi = 6509,
  CODE_FOR_vcondv16sfv64qi = 6510,
  CODE_FOR_vcondv8dfv64qi = 6511,
  CODE_FOR_vcondv32qiv32qi = 6512,
  CODE_FOR_vcondv32qiv16hi = 6513,
  CODE_FOR_vcondv32qiv8si = 6514,
  CODE_FOR_vcondv32qiv4di = 6515,
  CODE_FOR_vcondv16hiv32qi = 6516,
  CODE_FOR_vcondv16hiv16hi = 6517,
  CODE_FOR_vcondv16hiv8si = 6518,
  CODE_FOR_vcondv16hiv4di = 6519,
  CODE_FOR_vcondv8siv32qi = 6520,
  CODE_FOR_vcondv8siv16hi = 6521,
  CODE_FOR_vcondv8siv8si = 6522,
  CODE_FOR_vcondv8siv4di = 6523,
  CODE_FOR_vcondv4div32qi = 6524,
  CODE_FOR_vcondv4div16hi = 6525,
  CODE_FOR_vcondv4div8si = 6526,
  CODE_FOR_vcondv4div4di = 6527,
  CODE_FOR_vcondv8sfv32qi = 6528,
  CODE_FOR_vcondv8sfv16hi = 6529,
  CODE_FOR_vcondv8sfv8si = 6530,
  CODE_FOR_vcondv8sfv4di = 6531,
  CODE_FOR_vcondv4dfv32qi = 6532,
  CODE_FOR_vcondv4dfv16hi = 6533,
  CODE_FOR_vcondv4dfv8si = 6534,
  CODE_FOR_vcondv4dfv4di = 6535,
  CODE_FOR_vcondv16qiv16qi = 6536,
  CODE_FOR_vcondv8hiv16qi = 6537,
  CODE_FOR_vcondv4siv16qi = 6538,
  CODE_FOR_vcondv2div16qi = 6539,
  CODE_FOR_vcondv4sfv16qi = 6540,
  CODE_FOR_vcondv2dfv16qi = 6541,
  CODE_FOR_vcondv16qiv8hi = 6542,
  CODE_FOR_vcondv8hiv8hi = 6543,
  CODE_FOR_vcondv4siv8hi = 6544,
  CODE_FOR_vcondv2div8hi = 6545,
  CODE_FOR_vcondv4sfv8hi = 6546,
  CODE_FOR_vcondv2dfv8hi = 6547,
  CODE_FOR_vcondv16qiv4si = 6548,
  CODE_FOR_vcondv8hiv4si = 6549,
  CODE_FOR_vcondv4siv4si = 6550,
  CODE_FOR_vcondv2div4si = 6551,
  CODE_FOR_vcondv4sfv4si = 6552,
  CODE_FOR_vcondv2dfv4si = 6553,
  CODE_FOR_vcondv2div2di = 6554,
  CODE_FOR_vcondv2dfv2di = 6555,
  CODE_FOR_vconduv64qiv16si = 6556,
  CODE_FOR_vconduv32hiv16si = 6557,
  CODE_FOR_vconduv16siv16si = 6558,
  CODE_FOR_vconduv8div16si = 6559,
  CODE_FOR_vconduv16sfv16si = 6560,
  CODE_FOR_vconduv8dfv16si = 6561,
  CODE_FOR_vconduv64qiv8di = 6562,
  CODE_FOR_vconduv32hiv8di = 6563,
  CODE_FOR_vconduv16siv8di = 6564,
  CODE_FOR_vconduv8div8di = 6565,
  CODE_FOR_vconduv16sfv8di = 6566,
  CODE_FOR_vconduv8dfv8di = 6567,
  CODE_FOR_vconduv64qiv32hi = 6568,
  CODE_FOR_vconduv32hiv32hi = 6569,
  CODE_FOR_vconduv16siv32hi = 6570,
  CODE_FOR_vconduv8div32hi = 6571,
  CODE_FOR_vconduv16sfv32hi = 6572,
  CODE_FOR_vconduv8dfv32hi = 6573,
  CODE_FOR_vconduv64qiv64qi = 6574,
  CODE_FOR_vconduv32hiv64qi = 6575,
  CODE_FOR_vconduv16siv64qi = 6576,
  CODE_FOR_vconduv8div64qi = 6577,
  CODE_FOR_vconduv16sfv64qi = 6578,
  CODE_FOR_vconduv8dfv64qi = 6579,
  CODE_FOR_vconduv32qiv32qi = 6580,
  CODE_FOR_vconduv32qiv16hi = 6581,
  CODE_FOR_vconduv32qiv8si = 6582,
  CODE_FOR_vconduv32qiv4di = 6583,
  CODE_FOR_vconduv16hiv32qi = 6584,
  CODE_FOR_vconduv16hiv16hi = 6585,
  CODE_FOR_vconduv16hiv8si = 6586,
  CODE_FOR_vconduv16hiv4di = 6587,
  CODE_FOR_vconduv8siv32qi = 6588,
  CODE_FOR_vconduv8siv16hi = 6589,
  CODE_FOR_vconduv8siv8si = 6590,
  CODE_FOR_vconduv8siv4di = 6591,
  CODE_FOR_vconduv4div32qi = 6592,
  CODE_FOR_vconduv4div16hi = 6593,
  CODE_FOR_vconduv4div8si = 6594,
  CODE_FOR_vconduv4div4di = 6595,
  CODE_FOR_vconduv8sfv32qi = 6596,
  CODE_FOR_vconduv8sfv16hi = 6597,
  CODE_FOR_vconduv8sfv8si = 6598,
  CODE_FOR_vconduv8sfv4di = 6599,
  CODE_FOR_vconduv4dfv32qi = 6600,
  CODE_FOR_vconduv4dfv16hi = 6601,
  CODE_FOR_vconduv4dfv8si = 6602,
  CODE_FOR_vconduv4dfv4di = 6603,
  CODE_FOR_vconduv16qiv16qi = 6604,
  CODE_FOR_vconduv8hiv16qi = 6605,
  CODE_FOR_vconduv4siv16qi = 6606,
  CODE_FOR_vconduv2div16qi = 6607,
  CODE_FOR_vconduv4sfv16qi = 6608,
  CODE_FOR_vconduv2dfv16qi = 6609,
  CODE_FOR_vconduv16qiv8hi = 6610,
  CODE_FOR_vconduv8hiv8hi = 6611,
  CODE_FOR_vconduv4siv8hi = 6612,
  CODE_FOR_vconduv2div8hi = 6613,
  CODE_FOR_vconduv4sfv8hi = 6614,
  CODE_FOR_vconduv2dfv8hi = 6615,
  CODE_FOR_vconduv16qiv4si = 6616,
  CODE_FOR_vconduv8hiv4si = 6617,
  CODE_FOR_vconduv4siv4si = 6618,
  CODE_FOR_vconduv2div4si = 6619,
  CODE_FOR_vconduv4sfv4si = 6620,
  CODE_FOR_vconduv2dfv4si = 6621,
  CODE_FOR_vconduv2div2di = 6622,
  CODE_FOR_vconduv2dfv2di = 6623,
  CODE_FOR_vcondeqv2div2di = 6624,
  CODE_FOR_vcondeqv2dfv2di = 6625,
  CODE_FOR_vec_permv16qi = 6626,
  CODE_FOR_vec_permv8hi = 6627,
  CODE_FOR_vec_permv4si = 6628,
  CODE_FOR_vec_permv2di = 6629,
  CODE_FOR_vec_permv4sf = 6630,
  CODE_FOR_vec_permv2df = 6631,
  CODE_FOR_vec_permv32qi = 6632,
  CODE_FOR_vec_permv16hi = 6633,
  CODE_FOR_vec_permv8si = 6634,
  CODE_FOR_vec_permv4di = 6635,
  CODE_FOR_vec_permv8sf = 6636,
  CODE_FOR_vec_permv4df = 6637,
  CODE_FOR_vec_permv16sf = 6638,
  CODE_FOR_vec_permv8df = 6639,
  CODE_FOR_vec_permv16si = 6640,
  CODE_FOR_vec_permv8di = 6641,
  CODE_FOR_vec_permv32hi = 6642,
  CODE_FOR_vec_permv64qi = 6643,
  CODE_FOR_vec_perm_constv4sf = 6644,
  CODE_FOR_vec_perm_constv4si = 6645,
  CODE_FOR_vec_perm_constv2df = 6646,
  CODE_FOR_vec_perm_constv2di = 6647,
  CODE_FOR_vec_perm_constv16qi = 6648,
  CODE_FOR_vec_perm_constv8hi = 6649,
  CODE_FOR_vec_perm_constv8sf = 6650,
  CODE_FOR_vec_perm_constv4df = 6651,
  CODE_FOR_vec_perm_constv8si = 6652,
  CODE_FOR_vec_perm_constv4di = 6653,
  CODE_FOR_vec_perm_constv32qi = 6654,
  CODE_FOR_vec_perm_constv16hi = 6655,
  CODE_FOR_vec_perm_constv16si = 6656,
  CODE_FOR_vec_perm_constv8di = 6657,
  CODE_FOR_vec_perm_constv16sf = 6658,
  CODE_FOR_vec_perm_constv8df = 6659,
  CODE_FOR_vec_perm_constv32hi = 6660,
  CODE_FOR_vec_perm_constv64qi = 6661,
  CODE_FOR_one_cmplv16si2 = 6662,
  CODE_FOR_one_cmplv8di2 = 6663,
  CODE_FOR_one_cmplv64qi2 = 6664,
  CODE_FOR_one_cmplv32qi2 = 6665,
  CODE_FOR_one_cmplv16qi2 = 6666,
  CODE_FOR_one_cmplv32hi2 = 6667,
  CODE_FOR_one_cmplv16hi2 = 6668,
  CODE_FOR_one_cmplv8hi2 = 6669,
  CODE_FOR_one_cmplv8si2 = 6670,
  CODE_FOR_one_cmplv4si2 = 6671,
  CODE_FOR_one_cmplv4di2 = 6672,
  CODE_FOR_one_cmplv2di2 = 6673,
  CODE_FOR_avx512bw_andnotv64qi3 = 6674,
  CODE_FOR_avx2_andnotv32qi3 = 6675,
  CODE_FOR_sse2_andnotv16qi3 = 6676,
  CODE_FOR_avx512bw_andnotv32hi3 = 6677,
  CODE_FOR_avx2_andnotv16hi3 = 6678,
  CODE_FOR_sse2_andnotv8hi3 = 6679,
  CODE_FOR_avx512f_andnotv16si3 = 6680,
  CODE_FOR_avx2_andnotv8si3 = 6681,
  CODE_FOR_sse2_andnotv4si3 = 6682,
  CODE_FOR_avx512f_andnotv8di3 = 6683,
  CODE_FOR_avx2_andnotv4di3 = 6684,
  CODE_FOR_sse2_andnotv2di3 = 6685,
  CODE_FOR_avx512f_andnotv16si3_mask = 6686,
  CODE_FOR_avx2_andnotv8si3_mask = 6687,
  CODE_FOR_sse2_andnotv4si3_mask = 6688,
  CODE_FOR_avx512f_andnotv8di3_mask = 6689,
  CODE_FOR_avx2_andnotv4di3_mask = 6690,
  CODE_FOR_sse2_andnotv2di3_mask = 6691,
  CODE_FOR_avx512bw_andnotv64qi3_mask = 6692,
  CODE_FOR_sse2_andnotv16qi3_mask = 6693,
  CODE_FOR_avx2_andnotv32qi3_mask = 6694,
  CODE_FOR_avx512bw_andnotv32hi3_mask = 6695,
  CODE_FOR_avx2_andnotv16hi3_mask = 6696,
  CODE_FOR_sse2_andnotv8hi3_mask = 6697,
  CODE_FOR_andv16si3 = 6698,
  CODE_FOR_iorv16si3 = 6699,
  CODE_FOR_xorv16si3 = 6700,
  CODE_FOR_andv8di3 = 6701,
  CODE_FOR_iorv8di3 = 6702,
  CODE_FOR_xorv8di3 = 6703,
  CODE_FOR_andv64qi3 = 6704,
  CODE_FOR_iorv64qi3 = 6705,
  CODE_FOR_xorv64qi3 = 6706,
  CODE_FOR_andv32qi3 = 6707,
  CODE_FOR_iorv32qi3 = 6708,
  CODE_FOR_xorv32qi3 = 6709,
  CODE_FOR_andv16qi3 = 6710,
  CODE_FOR_iorv16qi3 = 6711,
  CODE_FOR_xorv16qi3 = 6712,
  CODE_FOR_andv32hi3 = 6713,
  CODE_FOR_iorv32hi3 = 6714,
  CODE_FOR_xorv32hi3 = 6715,
  CODE_FOR_andv16hi3 = 6716,
  CODE_FOR_iorv16hi3 = 6717,
  CODE_FOR_xorv16hi3 = 6718,
  CODE_FOR_andv8hi3 = 6719,
  CODE_FOR_iorv8hi3 = 6720,
  CODE_FOR_xorv8hi3 = 6721,
  CODE_FOR_andv8si3 = 6722,
  CODE_FOR_iorv8si3 = 6723,
  CODE_FOR_xorv8si3 = 6724,
  CODE_FOR_andv4si3 = 6725,
  CODE_FOR_iorv4si3 = 6726,
  CODE_FOR_xorv4si3 = 6727,
  CODE_FOR_andv4di3 = 6728,
  CODE_FOR_iorv4di3 = 6729,
  CODE_FOR_xorv4di3 = 6730,
  CODE_FOR_andv2di3 = 6731,
  CODE_FOR_iorv2di3 = 6732,
  CODE_FOR_xorv2di3 = 6733,
  CODE_FOR_vec_pack_trunc_v32hi = 6734,
  CODE_FOR_vec_pack_trunc_v16hi = 6735,
  CODE_FOR_vec_pack_trunc_v8hi = 6736,
  CODE_FOR_vec_pack_trunc_v16si = 6737,
  CODE_FOR_vec_pack_trunc_v8si = 6738,
  CODE_FOR_vec_pack_trunc_v4si = 6739,
  CODE_FOR_vec_pack_trunc_v8di = 6740,
  CODE_FOR_vec_pack_trunc_v4di = 6741,
  CODE_FOR_vec_pack_trunc_v2di = 6742,
  CODE_FOR_vec_pack_trunc_qi = 6743,
  CODE_FOR_vec_pack_trunc_hi = 6744,
  CODE_FOR_vec_pack_trunc_si = 6745,
  CODE_FOR_vec_interleave_highv32qi = 6746,
  CODE_FOR_vec_interleave_highv16hi = 6747,
  CODE_FOR_vec_interleave_highv8si = 6748,
  CODE_FOR_vec_interleave_highv4di = 6749,
  CODE_FOR_vec_interleave_lowv32qi = 6750,
  CODE_FOR_vec_interleave_lowv16hi = 6751,
  CODE_FOR_vec_interleave_lowv8si = 6752,
  CODE_FOR_vec_interleave_lowv4di = 6753,
  CODE_FOR_avx512dq_vinsertf64x2_mask = 6754,
  CODE_FOR_avx512dq_vinserti64x2_mask = 6755,
  CODE_FOR_avx512f_vinsertf32x4_mask = 6756,
  CODE_FOR_avx512f_vinserti32x4_mask = 6757,
  CODE_FOR_avx512dq_vinsertf32x8_mask = 6758,
  CODE_FOR_avx512dq_vinserti32x8_mask = 6759,
  CODE_FOR_avx512f_vinsertf64x4_mask = 6760,
  CODE_FOR_avx512f_vinserti64x4_mask = 6761,
  CODE_FOR_avx512dq_shuf_i64x2_mask = 6762,
  CODE_FOR_avx512dq_shuf_f64x2_mask = 6763,
  CODE_FOR_avx512f_shuf_f64x2_mask = 6764,
  CODE_FOR_avx512f_shuf_i64x2_mask = 6765,
  CODE_FOR_avx512vl_shuf_i32x4_mask = 6766,
  CODE_FOR_avx512vl_shuf_f32x4_mask = 6767,
  CODE_FOR_avx512f_shuf_f32x4_mask = 6768,
  CODE_FOR_avx512f_shuf_i32x4_mask = 6769,
  CODE_FOR_avx512f_pshufdv3_mask = 6770,
  CODE_FOR_avx512vl_pshufdv3_mask = 6771,
  CODE_FOR_avx2_pshufdv3 = 6772,
  CODE_FOR_avx512vl_pshufd_mask = 6773,
  CODE_FOR_sse2_pshufd = 6774,
  CODE_FOR_avx512vl_pshuflwv3_mask = 6775,
  CODE_FOR_avx2_pshuflwv3 = 6776,
  CODE_FOR_avx512vl_pshuflw_mask = 6777,
  CODE_FOR_sse2_pshuflw = 6778,
  CODE_FOR_avx2_pshufhwv3 = 6779,
  CODE_FOR_avx512vl_pshufhwv3_mask = 6780,
  CODE_FOR_avx512vl_pshufhw_mask = 6781,
  CODE_FOR_sse2_pshufhw = 6782,
  CODE_FOR_sse2_loadd = 6783,
  CODE_FOR_vec_unpacks_lo_v64qi = 6784,
  CODE_FOR_vec_unpacks_lo_v32qi = 6785,
  CODE_FOR_vec_unpacks_lo_v16qi = 6786,
  CODE_FOR_vec_unpacks_lo_v32hi = 6787,
  CODE_FOR_vec_unpacks_lo_v16hi = 6788,
  CODE_FOR_vec_unpacks_lo_v8hi = 6789,
  CODE_FOR_vec_unpacks_lo_v16si = 6790,
  CODE_FOR_vec_unpacks_lo_v8si = 6791,
  CODE_FOR_vec_unpacks_lo_v4si = 6792,
  CODE_FOR_vec_unpacks_hi_v64qi = 6793,
  CODE_FOR_vec_unpacks_hi_v32qi = 6794,
  CODE_FOR_vec_unpacks_hi_v16qi = 6795,
  CODE_FOR_vec_unpacks_hi_v32hi = 6796,
  CODE_FOR_vec_unpacks_hi_v16hi = 6797,
  CODE_FOR_vec_unpacks_hi_v8hi = 6798,
  CODE_FOR_vec_unpacks_hi_v16si = 6799,
  CODE_FOR_vec_unpacks_hi_v8si = 6800,
  CODE_FOR_vec_unpacks_hi_v4si = 6801,
  CODE_FOR_vec_unpacku_lo_v64qi = 6802,
  CODE_FOR_vec_unpacku_lo_v32qi = 6803,
  CODE_FOR_vec_unpacku_lo_v16qi = 6804,
  CODE_FOR_vec_unpacku_lo_v32hi = 6805,
  CODE_FOR_vec_unpacku_lo_v16hi = 6806,
  CODE_FOR_vec_unpacku_lo_v8hi = 6807,
  CODE_FOR_vec_unpacku_lo_v16si = 6808,
  CODE_FOR_vec_unpacku_lo_v8si = 6809,
  CODE_FOR_vec_unpacku_lo_v4si = 6810,
  CODE_FOR_vec_unpacks_lo_hi = 6811,
  CODE_FOR_vec_unpacks_lo_si = 6812,
  CODE_FOR_vec_unpacks_lo_di = 6813,
  CODE_FOR_vec_unpacku_hi_v64qi = 6814,
  CODE_FOR_vec_unpacku_hi_v32qi = 6815,
  CODE_FOR_vec_unpacku_hi_v16qi = 6816,
  CODE_FOR_vec_unpacku_hi_v32hi = 6817,
  CODE_FOR_vec_unpacku_hi_v16hi = 6818,
  CODE_FOR_vec_unpacku_hi_v8hi = 6819,
  CODE_FOR_vec_unpacku_hi_v16si = 6820,
  CODE_FOR_vec_unpacku_hi_v8si = 6821,
  CODE_FOR_vec_unpacku_hi_v4si = 6822,
  CODE_FOR_vec_unpacks_hi_hi = 6823,
  CODE_FOR_vec_unpacks_hi_si = 6824,
  CODE_FOR_vec_unpacks_hi_di = 6825,
  CODE_FOR_avx512bw_uavgv64qi3 = 6826,
  CODE_FOR_avx512bw_uavgv64qi3_mask = 6827,
  CODE_FOR_avx2_uavgv32qi3 = 6828,
  CODE_FOR_avx2_uavgv32qi3_mask = 6829,
  CODE_FOR_sse2_uavgv16qi3 = 6830,
  CODE_FOR_sse2_uavgv16qi3_mask = 6831,
  CODE_FOR_avx512bw_uavgv32hi3 = 6832,
  CODE_FOR_avx512bw_uavgv32hi3_mask = 6833,
  CODE_FOR_avx2_uavgv16hi3 = 6834,
  CODE_FOR_avx2_uavgv16hi3_mask = 6835,
  CODE_FOR_sse2_uavgv8hi3 = 6836,
  CODE_FOR_sse2_uavgv8hi3_mask = 6837,
  CODE_FOR_sse2_maskmovdqu = 6838,
  CODE_FOR_ssse3_pmulhrswv4hi3_mask = 6839,
  CODE_FOR_ssse3_pmulhrswv8hi3_mask = 6840,
  CODE_FOR_avx2_pmulhrswv16hi3_mask = 6841,
  CODE_FOR_ssse3_pmulhrswv4hi3 = 6842,
  CODE_FOR_ssse3_pmulhrswv8hi3 = 6843,
  CODE_FOR_avx2_pmulhrswv16hi3 = 6844,
  CODE_FOR_absv64qi2 = 6845,
  CODE_FOR_absv32qi2 = 6846,
  CODE_FOR_absv16qi2 = 6847,
  CODE_FOR_absv32hi2 = 6848,
  CODE_FOR_absv16hi2 = 6849,
  CODE_FOR_absv8hi2 = 6850,
  CODE_FOR_absv16si2 = 6851,
  CODE_FOR_absv8si2 = 6852,
  CODE_FOR_absv4si2 = 6853,
  CODE_FOR_absv8di2 = 6854,
  CODE_FOR_absv4di2 = 6855,
  CODE_FOR_absv2di2 = 6856,
  CODE_FOR_avx2_pblendw = 6857,
  CODE_FOR_avx_roundps_sfix256 = 6858,
  CODE_FOR_sse4_1_roundps_sfix = 6859,
  CODE_FOR_avx512f_roundps512 = 6860,
  CODE_FOR_avx512f_roundpd512 = 6861,
  CODE_FOR_avx512f_roundps512_sfix = 6862,
  CODE_FOR_avx512f_roundpd_vec_pack_sfix512 = 6863,
  CODE_FOR_avx_roundpd_vec_pack_sfix256 = 6864,
  CODE_FOR_sse4_1_roundpd_vec_pack_sfix = 6865,
  CODE_FOR_roundv16sf2 = 6866,
  CODE_FOR_roundv8sf2 = 6867,
  CODE_FOR_roundv4sf2 = 6868,
  CODE_FOR_roundv8df2 = 6869,
  CODE_FOR_roundv4df2 = 6870,
  CODE_FOR_roundv2df2 = 6871,
  CODE_FOR_roundv16sf2_sfix = 6872,
  CODE_FOR_roundv8sf2_sfix = 6873,
  CODE_FOR_roundv4sf2_sfix = 6874,
  CODE_FOR_roundv8df2_vec_pack_sfix = 6875,
  CODE_FOR_roundv4df2_vec_pack_sfix = 6876,
  CODE_FOR_roundv2df2_vec_pack_sfix = 6877,
  CODE_FOR_avx512pf_gatherpfv16sisf = 6878,
  CODE_FOR_avx512pf_gatherpfv8disf = 6879,
  CODE_FOR_avx512pf_gatherpfv8sidf = 6880,
  CODE_FOR_avx512pf_gatherpfv8didf = 6881,
  CODE_FOR_avx512pf_scatterpfv16sisf = 6882,
  CODE_FOR_avx512pf_scatterpfv8disf = 6883,
  CODE_FOR_avx512pf_scatterpfv8sidf = 6884,
  CODE_FOR_avx512pf_scatterpfv8didf = 6885,
  CODE_FOR_rotlv16qi3 = 6886,
  CODE_FOR_rotlv8hi3 = 6887,
  CODE_FOR_rotlv4si3 = 6888,
  CODE_FOR_rotlv2di3 = 6889,
  CODE_FOR_rotrv16qi3 = 6890,
  CODE_FOR_rotrv8hi3 = 6891,
  CODE_FOR_rotrv4si3 = 6892,
  CODE_FOR_rotrv2di3 = 6893,
  CODE_FOR_vrotrv16qi3 = 6894,
  CODE_FOR_vrotrv8hi3 = 6895,
  CODE_FOR_vrotrv4si3 = 6896,
  CODE_FOR_vrotrv2di3 = 6897,
  CODE_FOR_vrotlv16qi3 = 6898,
  CODE_FOR_vrotlv8hi3 = 6899,
  CODE_FOR_vrotlv4si3 = 6900,
  CODE_FOR_vrotlv2di3 = 6901,
  CODE_FOR_vlshrv16qi3 = 6902,
  CODE_FOR_vlshrv8hi3 = 6903,
  CODE_FOR_vlshrv4si3 = 6904,
  CODE_FOR_vlshrv2di3 = 6905,
  CODE_FOR_vlshrv16si3 = 6906,
  CODE_FOR_vlshrv8di3 = 6907,
  CODE_FOR_vlshrv8si3 = 6908,
  CODE_FOR_vlshrv4di3 = 6909,
  CODE_FOR_vashrv8hi3 = 6910,
  CODE_FOR_vashrv8hi3_mask = 6911,
  CODE_FOR_vashrv16qi3 = 6912,
  CODE_FOR_vashrv2di3 = 6913,
  CODE_FOR_vashrv2di3_mask = 6914,
  CODE_FOR_vashrv4si3 = 6915,
  CODE_FOR_vashrv16si3 = 6916,
  CODE_FOR_vashrv8si3 = 6917,
  CODE_FOR_vashlv16qi3 = 6918,
  CODE_FOR_vashlv8hi3 = 6919,
  CODE_FOR_vashlv4si3 = 6920,
  CODE_FOR_vashlv2di3 = 6921,
  CODE_FOR_vashlv16si3 = 6922,
  CODE_FOR_vashlv8di3 = 6923,
  CODE_FOR_vashlv8si3 = 6924,
  CODE_FOR_vashlv4di3 = 6925,
  CODE_FOR_ashlv64qi3 = 6926,
  CODE_FOR_lshrv64qi3 = 6927,
  CODE_FOR_ashrv64qi3 = 6928,
  CODE_FOR_ashlv32qi3 = 6929,
  CODE_FOR_lshrv32qi3 = 6930,
  CODE_FOR_ashrv32qi3 = 6931,
  CODE_FOR_ashlv16qi3 = 6932,
  CODE_FOR_lshrv16qi3 = 6933,
  CODE_FOR_ashrv16qi3 = 6934,
  CODE_FOR_ashrv2di3 = 6935,
  CODE_FOR_xop_vmfrczv4sf2 = 6936,
  CODE_FOR_xop_vmfrczv2df2 = 6937,
  CODE_FOR_avx_vzeroall = 6938,
  CODE_FOR_avx2_permv4di = 6939,
  CODE_FOR_avx2_permv4df = 6940,
  CODE_FOR_avx512vl_permv4di_mask = 6941,
  CODE_FOR_avx512vl_permv4df_mask = 6942,
  CODE_FOR_avx512f_permv8df = 6943,
  CODE_FOR_avx512f_permv8di = 6944,
  CODE_FOR_avx512f_permv8df_mask = 6945,
  CODE_FOR_avx512f_permv8di_mask = 6946,
  CODE_FOR_avx512f_vpermilv8df = 6947,
  CODE_FOR_avx512f_vpermilv8df_mask = 6948,
  CODE_FOR_avx_vpermilv4df = 6949,
  CODE_FOR_avx_vpermilv4df_mask = 6950,
  CODE_FOR_avx_vpermilv2df = 6951,
  CODE_FOR_avx_vpermilv2df_mask = 6952,
  CODE_FOR_avx512f_vpermilv16sf = 6953,
  CODE_FOR_avx512f_vpermilv16sf_mask = 6954,
  CODE_FOR_avx_vpermilv8sf = 6955,
  CODE_FOR_avx_vpermilv8sf_mask = 6956,
  CODE_FOR_avx_vpermilv4sf = 6957,
  CODE_FOR_avx_vpermilv4sf_mask = 6958,
  CODE_FOR_avx512f_vpermi2varv16si3_maskz = 6959,
  CODE_FOR_avx512f_vpermi2varv16sf3_maskz = 6960,
  CODE_FOR_avx512f_vpermi2varv8di3_maskz = 6961,
  CODE_FOR_avx512f_vpermi2varv8df3_maskz = 6962,
  CODE_FOR_avx512vl_vpermi2varv8si3_maskz = 6963,
  CODE_FOR_avx512vl_vpermi2varv8sf3_maskz = 6964,
  CODE_FOR_avx512vl_vpermi2varv4di3_maskz = 6965,
  CODE_FOR_avx512vl_vpermi2varv4df3_maskz = 6966,
  CODE_FOR_avx512vl_vpermi2varv4si3_maskz = 6967,
  CODE_FOR_avx512vl_vpermi2varv4sf3_maskz = 6968,
  CODE_FOR_avx512vl_vpermi2varv2di3_maskz = 6969,
  CODE_FOR_avx512vl_vpermi2varv2df3_maskz = 6970,
  CODE_FOR_avx512bw_vpermi2varv64qi3_maskz = 6971,
  CODE_FOR_avx512vl_vpermi2varv16qi3_maskz = 6972,
  CODE_FOR_avx512vl_vpermi2varv32qi3_maskz = 6973,
  CODE_FOR_avx512vl_vpermi2varv8hi3_maskz = 6974,
  CODE_FOR_avx512vl_vpermi2varv16hi3_maskz = 6975,
  CODE_FOR_avx512bw_vpermi2varv32hi3_maskz = 6976,
  CODE_FOR_avx512f_vpermt2varv16si3_maskz = 6977,
  CODE_FOR_avx512f_vpermt2varv16sf3_maskz = 6978,
  CODE_FOR_avx512f_vpermt2varv8di3_maskz = 6979,
  CODE_FOR_avx512f_vpermt2varv8df3_maskz = 6980,
  CODE_FOR_avx512vl_vpermt2varv8si3_maskz = 6981,
  CODE_FOR_avx512vl_vpermt2varv8sf3_maskz = 6982,
  CODE_FOR_avx512vl_vpermt2varv4di3_maskz = 6983,
  CODE_FOR_avx512vl_vpermt2varv4df3_maskz = 6984,
  CODE_FOR_avx512vl_vpermt2varv4si3_maskz = 6985,
  CODE_FOR_avx512vl_vpermt2varv4sf3_maskz = 6986,
  CODE_FOR_avx512vl_vpermt2varv2di3_maskz = 6987,
  CODE_FOR_avx512vl_vpermt2varv2df3_maskz = 6988,
  CODE_FOR_avx512bw_vpermt2varv64qi3_maskz = 6989,
  CODE_FOR_avx512vl_vpermt2varv16qi3_maskz = 6990,
  CODE_FOR_avx512vl_vpermt2varv32qi3_maskz = 6991,
  CODE_FOR_avx512vl_vpermt2varv8hi3_maskz = 6992,
  CODE_FOR_avx512vl_vpermt2varv16hi3_maskz = 6993,
  CODE_FOR_avx512bw_vpermt2varv32hi3_maskz = 6994,
  CODE_FOR_avx_vperm2f128v8si3 = 6995,
  CODE_FOR_avx_vperm2f128v8sf3 = 6996,
  CODE_FOR_avx_vperm2f128v4df3 = 6997,
  CODE_FOR_avx512vl_vinsertv8si = 6998,
  CODE_FOR_avx512vl_vinsertv8sf = 6999,
  CODE_FOR_avx512vl_vinsertv4di = 7000,
  CODE_FOR_avx512vl_vinsertv4df = 7001,
  CODE_FOR_avx_vinsertf128v32qi = 7002,
  CODE_FOR_avx_vinsertf128v16hi = 7003,
  CODE_FOR_avx_vinsertf128v8si = 7004,
  CODE_FOR_avx_vinsertf128v4di = 7005,
  CODE_FOR_avx_vinsertf128v8sf = 7006,
  CODE_FOR_avx_vinsertf128v4df = 7007,
  CODE_FOR_maskloadv4sfv4si = 7008,
  CODE_FOR_maskloadv2dfv2di = 7009,
  CODE_FOR_maskloadv8sfv8si = 7010,
  CODE_FOR_maskloadv4dfv4di = 7011,
  CODE_FOR_maskloadv4siv4si = 7012,
  CODE_FOR_maskloadv2div2di = 7013,
  CODE_FOR_maskloadv8siv8si = 7014,
  CODE_FOR_maskloadv4div4di = 7015,
  CODE_FOR_maskloadv16sihi = 7016,
  CODE_FOR_maskloadv8siqi = 7017,
  CODE_FOR_maskloadv4siqi = 7018,
  CODE_FOR_maskloadv8diqi = 7019,
  CODE_FOR_maskloadv4diqi = 7020,
  CODE_FOR_maskloadv2diqi = 7021,
  CODE_FOR_maskloadv16sfhi = 7022,
  CODE_FOR_maskloadv8sfqi = 7023,
  CODE_FOR_maskloadv4sfqi = 7024,
  CODE_FOR_maskloadv8dfqi = 7025,
  CODE_FOR_maskloadv4dfqi = 7026,
  CODE_FOR_maskloadv2dfqi = 7027,
  CODE_FOR_maskloadv64qidi = 7028,
  CODE_FOR_maskloadv16qihi = 7029,
  CODE_FOR_maskloadv32qisi = 7030,
  CODE_FOR_maskloadv32hisi = 7031,
  CODE_FOR_maskloadv16hihi = 7032,
  CODE_FOR_maskloadv8hiqi = 7033,
  CODE_FOR_maskstorev4sfv4si = 7034,
  CODE_FOR_maskstorev2dfv2di = 7035,
  CODE_FOR_maskstorev8sfv8si = 7036,
  CODE_FOR_maskstorev4dfv4di = 7037,
  CODE_FOR_maskstorev4siv4si = 7038,
  CODE_FOR_maskstorev2div2di = 7039,
  CODE_FOR_maskstorev8siv8si = 7040,
  CODE_FOR_maskstorev4div4di = 7041,
  CODE_FOR_maskstorev16sihi = 7042,
  CODE_FOR_maskstorev8siqi = 7043,
  CODE_FOR_maskstorev4siqi = 7044,
  CODE_FOR_maskstorev8diqi = 7045,
  CODE_FOR_maskstorev4diqi = 7046,
  CODE_FOR_maskstorev2diqi = 7047,
  CODE_FOR_maskstorev16sfhi = 7048,
  CODE_FOR_maskstorev8sfqi = 7049,
  CODE_FOR_maskstorev4sfqi = 7050,
  CODE_FOR_maskstorev8dfqi = 7051,
  CODE_FOR_maskstorev4dfqi = 7052,
  CODE_FOR_maskstorev2dfqi = 7053,
  CODE_FOR_maskstorev64qidi = 7054,
  CODE_FOR_maskstorev16qihi = 7055,
  CODE_FOR_maskstorev32qisi = 7056,
  CODE_FOR_maskstorev32hisi = 7057,
  CODE_FOR_maskstorev16hihi = 7058,
  CODE_FOR_maskstorev8hiqi = 7059,
  CODE_FOR_cbranchv4si4 = 7060,
  CODE_FOR_cbranchv2di4 = 7061,
  CODE_FOR_cbranchv8si4 = 7062,
  CODE_FOR_cbranchv4di4 = 7063,
  CODE_FOR_vec_initv32qi = 7064,
  CODE_FOR_vec_initv16hi = 7065,
  CODE_FOR_vec_initv8si = 7066,
  CODE_FOR_vec_initv4di = 7067,
  CODE_FOR_vec_initv8sf = 7068,
  CODE_FOR_vec_initv4df = 7069,
  CODE_FOR_vec_initv16si = 7070,
  CODE_FOR_vec_initv16sf = 7071,
  CODE_FOR_vec_initv8di = 7072,
  CODE_FOR_vec_initv8df = 7073,
  CODE_FOR_vec_initv32hi = 7074,
  CODE_FOR_vec_initv64qi = 7075,
  CODE_FOR_vcvtps2ph_mask = 7076,
  CODE_FOR_vcvtps2ph = 7077,
  CODE_FOR_avx2_gathersiv2di = 7078,
  CODE_FOR_avx2_gathersiv2df = 7079,
  CODE_FOR_avx2_gathersiv4di = 7080,
  CODE_FOR_avx2_gathersiv4df = 7081,
  CODE_FOR_avx2_gathersiv4si = 7082,
  CODE_FOR_avx2_gathersiv4sf = 7083,
  CODE_FOR_avx2_gathersiv8si = 7084,
  CODE_FOR_avx2_gathersiv8sf = 7085,
  CODE_FOR_avx2_gatherdiv2di = 7086,
  CODE_FOR_avx2_gatherdiv2df = 7087,
  CODE_FOR_avx2_gatherdiv4di = 7088,
  CODE_FOR_avx2_gatherdiv4df = 7089,
  CODE_FOR_avx2_gatherdiv4si = 7090,
  CODE_FOR_avx2_gatherdiv4sf = 7091,
  CODE_FOR_avx2_gatherdiv8si = 7092,
  CODE_FOR_avx2_gatherdiv8sf = 7093,
  CODE_FOR_avx512f_gathersiv16si = 7094,
  CODE_FOR_avx512f_gathersiv16sf = 7095,
  CODE_FOR_avx512f_gathersiv8di = 7096,
  CODE_FOR_avx512f_gathersiv8df = 7097,
  CODE_FOR_avx512vl_gathersiv8si = 7098,
  CODE_FOR_avx512vl_gathersiv8sf = 7099,
  CODE_FOR_avx512vl_gathersiv4di = 7100,
  CODE_FOR_avx512vl_gathersiv4df = 7101,
  CODE_FOR_avx512vl_gathersiv4si = 7102,
  CODE_FOR_avx512vl_gathersiv4sf = 7103,
  CODE_FOR_avx512vl_gathersiv2di = 7104,
  CODE_FOR_avx512vl_gathersiv2df = 7105,
  CODE_FOR_avx512f_gatherdiv16si = 7106,
  CODE_FOR_avx512f_gatherdiv16sf = 7107,
  CODE_FOR_avx512f_gatherdiv8di = 7108,
  CODE_FOR_avx512f_gatherdiv8df = 7109,
  CODE_FOR_avx512vl_gatherdiv8si = 7110,
  CODE_FOR_avx512vl_gatherdiv8sf = 7111,
  CODE_FOR_avx512vl_gatherdiv4di = 7112,
  CODE_FOR_avx512vl_gatherdiv4df = 7113,
  CODE_FOR_avx512vl_gatherdiv4si = 7114,
  CODE_FOR_avx512vl_gatherdiv4sf = 7115,
  CODE_FOR_avx512vl_gatherdiv2di = 7116,
  CODE_FOR_avx512vl_gatherdiv2df = 7117,
  CODE_FOR_avx512f_scattersiv16si = 7118,
  CODE_FOR_avx512f_scattersiv16sf = 7119,
  CODE_FOR_avx512f_scattersiv8di = 7120,
  CODE_FOR_avx512f_scattersiv8df = 7121,
  CODE_FOR_avx512vl_scattersiv8si = 7122,
  CODE_FOR_avx512vl_scattersiv8sf = 7123,
  CODE_FOR_avx512vl_scattersiv4di = 7124,
  CODE_FOR_avx512vl_scattersiv4df = 7125,
  CODE_FOR_avx512vl_scattersiv4si = 7126,
  CODE_FOR_avx512vl_scattersiv4sf = 7127,
  CODE_FOR_avx512vl_scattersiv2di = 7128,
  CODE_FOR_avx512vl_scattersiv2df = 7129,
  CODE_FOR_avx512f_scatterdiv16si = 7130,
  CODE_FOR_avx512f_scatterdiv16sf = 7131,
  CODE_FOR_avx512f_scatterdiv8di = 7132,
  CODE_FOR_avx512f_scatterdiv8df = 7133,
  CODE_FOR_avx512vl_scatterdiv8si = 7134,
  CODE_FOR_avx512vl_scatterdiv8sf = 7135,
  CODE_FOR_avx512vl_scatterdiv4di = 7136,
  CODE_FOR_avx512vl_scatterdiv4df = 7137,
  CODE_FOR_avx512vl_scatterdiv4si = 7138,
  CODE_FOR_avx512vl_scatterdiv4sf = 7139,
  CODE_FOR_avx512vl_scatterdiv2di = 7140,
  CODE_FOR_avx512vl_scatterdiv2df = 7141,
  CODE_FOR_avx512f_expandv16si_maskz = 7142,
  CODE_FOR_avx512f_expandv16sf_maskz = 7143,
  CODE_FOR_avx512f_expandv8di_maskz = 7144,
  CODE_FOR_avx512f_expandv8df_maskz = 7145,
  CODE_FOR_avx512vl_expandv8si_maskz = 7146,
  CODE_FOR_avx512vl_expandv8sf_maskz = 7147,
  CODE_FOR_avx512vl_expandv4di_maskz = 7148,
  CODE_FOR_avx512vl_expandv4df_maskz = 7149,
  CODE_FOR_avx512vl_expandv4si_maskz = 7150,
  CODE_FOR_avx512vl_expandv4sf_maskz = 7151,
  CODE_FOR_avx512vl_expandv2di_maskz = 7152,
  CODE_FOR_avx512vl_expandv2df_maskz = 7153,
  CODE_FOR_vpamdd52huqv8di_maskz = 7154,
  CODE_FOR_vpamdd52huqv4di_maskz = 7155,
  CODE_FOR_vpamdd52huqv2di_maskz = 7156,
  CODE_FOR_vpamdd52luqv8di_maskz = 7157,
  CODE_FOR_vpamdd52luqv4di_maskz = 7158,
  CODE_FOR_vpamdd52luqv2di_maskz = 7159,
  CODE_FOR_movv64sf = 7160,
  CODE_FOR_movv64si = 7161,
  CODE_FOR_sse2_lfence = 7162,
  CODE_FOR_sse_sfence = 7163,
  CODE_FOR_sse2_mfence = 7164,
  CODE_FOR_mem_thread_fence = 7165,
  CODE_FOR_atomic_loadqi = 7166,
  CODE_FOR_atomic_loadhi = 7167,
  CODE_FOR_atomic_loadsi = 7168,
  CODE_FOR_atomic_loaddi = 7169,
  CODE_FOR_atomic_storeqi = 7170,
  CODE_FOR_atomic_storehi = 7171,
  CODE_FOR_atomic_storesi = 7172,
  CODE_FOR_atomic_storedi = 7173,
  CODE_FOR_atomic_compare_and_swapqi = 7174,
  CODE_FOR_atomic_compare_and_swaphi = 7175,
  CODE_FOR_atomic_compare_and_swapsi = 7176,
  CODE_FOR_atomic_compare_and_swapdi = 7177,
  CODE_FOR_atomic_compare_and_swapti = 7178,
  CODE_FOR_atomic_bit_test_and_sethi = 7179,
  CODE_FOR_atomic_bit_test_and_setsi = 7180,
  CODE_FOR_atomic_bit_test_and_setdi = 7181,
  CODE_FOR_atomic_bit_test_and_complementhi = 7182,
  CODE_FOR_atomic_bit_test_and_complementsi = 7183,
  CODE_FOR_atomic_bit_test_and_complementdi = 7184,
  CODE_FOR_atomic_bit_test_and_resethi = 7185,
  CODE_FOR_atomic_bit_test_and_resetsi = 7186,
  CODE_FOR_atomic_bit_test_and_resetdi = 7187
};

const unsigned int NUM_INSN_CODES = 7188;
#endif /* GCC_INSN_CODES_H */
